@@ -13,8 +13,8 @@ See the webpage for more information and documentation:
 
 
 import sys
-if sys.version_info < (3, 5):
-    raise ImportError("Python version 3.5 or above is required for SymPy.")
+if sys.version_info < (3, 6):
+    raise ImportError("Python version 3.6 or above is required for SymPy.")
 del sys
 
 
@@ -95,7 +95,7 @@ from .polys import (Poly, PurePoly, poly_from_expr, parallel_poly_from_expr,
         GMPYFiniteField, PythonIntegerRing, GMPYIntegerRing, PythonRational,
         GMPYRationalField, AlgebraicField, PolynomialRing, FractionField,
         ExpressionDomain, FF_python, FF_gmpy, ZZ_python, ZZ_gmpy, QQ_python,
-        QQ_gmpy, GF, FF, ZZ, QQ, RR, CC, EX, construct_domain,
+        QQ_gmpy, GF, FF, ZZ, QQ, ZZ_I, QQ_I, RR, CC, EX, construct_domain,
         swinnerton_dyer_poly, cyclotomic_poly, symmetric_poly, random_poly,
         interpolating_poly, jacobi_poly, chebyshevt_poly, chebyshevu_poly,
         hermite_poly, legendre_poly, laguerre_poly, apart, apart_list,
@@ -108,7 +108,7 @@ from .series import (Order, O, limit, Limit, gruntz, series, approximants,
 
 from .functions import (factorial, factorial2, rf, ff, binomial,
         RisingFactorial, FallingFactorial, subfactorial, carmichael,
-        fibonacci, lucas, tribonacci, harmonic, bernoulli, bell, euler,
+        fibonacci, lucas, motzkin, tribonacci, harmonic, bernoulli, bell, euler,
         catalan, genocchi, partition, sqrt, root, Min, Max, Id, real_root,
         cbrt, re, im, sign, Abs, conjugate, arg, polar_lift,
         periodic_argument, unbranched_argument, principal_branch, transpose,
@@ -128,7 +128,7 @@ from .functions import (factorial, factorial2, rf, ff, binomial,
         chebyshevu, chebyshevu_root, chebyshevt_root, laguerre,
         assoc_laguerre, gegenbauer, jacobi, jacobi_normalized, Ynm, Ynm_c,
         Znm, elliptic_k, elliptic_f, elliptic_e, elliptic_pi, beta, mathieus,
-        mathieuc, mathieusprime, mathieucprime)
+        mathieuc, mathieusprime, mathieucprime, riemann_xi, betainc, betainc_regularized)
 
 from .ntheory import (nextprime, prevprime, prime, primepi, primerange,
         randprime, Sieve, sieve, primorial, cycle_length, composite,
@@ -190,7 +190,7 @@ from .matrices import (ShapeError, NonSquareMatrixError, GramSchmidt,
         Adjoint, hadamard_product, HadamardProduct, HadamardPower,
         Determinant, det, diagonalize_vector, DiagMatrix, DiagonalMatrix,
         DiagonalOf, trace, DotProduct, kronecker_product, KroneckerProduct,
-        PermutationMatrix, MatrixPermute)
+        PermutationMatrix, MatrixPermute, Permanent, per)
 
 from .geometry import (Point, Point2D, Point3D, Line, Ray, Segment, Line2D,
         Segment2D, Ray2D, Line3D, Segment3D, Ray3D, Plane, Ellipse, Circle,
@@ -216,10 +216,10 @@ from .integrals import (integrate, Integral, line_integrate, mellin_transform,
         HankelTransform, InverseHankelTransform, singularityintegrate)
 
 from .tensor import (IndexedBase, Idx, Indexed, get_contraction_structure,
-        get_indices, MutableDenseNDimArray, ImmutableDenseNDimArray,
+        get_indices, shape, MutableDenseNDimArray, ImmutableDenseNDimArray,
         MutableSparseNDimArray, ImmutableSparseNDimArray, NDimArray,
-        tensorproduct, tensorcontraction, derive_by_array, permutedims, Array,
-        DenseNDimArray, SparseNDimArray)
+        tensorproduct, tensorcontraction, tensordiagonal, derive_by_array,
+        permutedims, Array, DenseNDimArray, SparseNDimArray)
 
 from .parsing import parse_expr
 
@@ -253,11 +253,6 @@ from .interactive import init_session, init_printing
 
 evalf._create_evalf_table()
 
-# This is slow to import:
-#import abc
-
-from .deprecated import C, ClassRegistry, class_registry
-
 __all__ = [
     # sympy.core
     'sympify', 'SympifyError', 'cacheit', 'Basic', 'Atom',
@@ -314,7 +309,7 @@ __all__ = [
     'GMPYIntegerRing', 'PythonRational', 'GMPYRationalField',
     'AlgebraicField', 'PolynomialRing', 'FractionField', 'ExpressionDomain',
     'FF_python', 'FF_gmpy', 'ZZ_python', 'ZZ_gmpy', 'QQ_python', 'QQ_gmpy',
-    'GF', 'FF', 'ZZ', 'QQ', 'RR', 'CC', 'EX', 'construct_domain',
+    'GF', 'FF', 'ZZ', 'QQ', 'ZZ_I', 'QQ_I', 'RR', 'CC', 'EX', 'construct_domain',
     'swinnerton_dyer_poly', 'cyclotomic_poly', 'symmetric_poly',
     'random_poly', 'interpolating_poly', 'jacobi_poly', 'chebyshevt_poly',
     'chebyshevu_poly', 'hermite_poly', 'legendre_poly', 'laguerre_poly',
@@ -329,7 +324,7 @@ __all__ = [
     # sympy.functions
     'factorial', 'factorial2', 'rf', 'ff', 'binomial', 'RisingFactorial',
     'FallingFactorial', 'subfactorial', 'carmichael', 'fibonacci', 'lucas',
-    'tribonacci', 'harmonic', 'bernoulli', 'bell', 'euler', 'catalan',
+    'motzkin', 'tribonacci', 'harmonic', 'bernoulli', 'bell', 'euler', 'catalan',
     'genocchi', 'partition', 'sqrt', 'root', 'Min', 'Max', 'Id', 'real_root',
     'cbrt', 're', 'im', 'sign', 'Abs', 'conjugate', 'arg', 'polar_lift',
     'periodic_argument', 'unbranched_argument', 'principal_branch',
@@ -352,7 +347,8 @@ __all__ = [
     'chebyshevu_root', 'chebyshevt_root', 'laguerre', 'assoc_laguerre',
     'gegenbauer', 'jacobi', 'jacobi_normalized', 'Ynm', 'Ynm_c', 'Znm',
     'elliptic_k', 'elliptic_f', 'elliptic_e', 'elliptic_pi', 'beta',
-    'mathieus', 'mathieuc', 'mathieusprime', 'mathieucprime',
+    'mathieus', 'mathieuc', 'mathieusprime', 'mathieucprime', 'riemann_xi','betainc',
+    'betainc_regularized',
 
     # sympy.ntheory
     'nextprime', 'prevprime', 'prime', 'primepi', 'primerange', 'randprime',
@@ -424,7 +420,7 @@ __all__ = [
     'HadamardPower', 'Determinant', 'det', 'diagonalize_vector', 'DiagMatrix',
     'DiagonalMatrix', 'DiagonalOf', 'trace', 'DotProduct',
     'kronecker_product', 'KroneckerProduct', 'PermutationMatrix',
-    'MatrixPermute',
+    'MatrixPermute', 'Permanent', 'per',
 
     # sympy.geometry
     'Point', 'Point2D', 'Point3D', 'Line', 'Ray', 'Segment', 'Line2D',
@@ -456,10 +452,10 @@ __all__ = [
 
     # sympy.tensor
     'IndexedBase', 'Idx', 'Indexed', 'get_contraction_structure',
-    'get_indices', 'MutableDenseNDimArray', 'ImmutableDenseNDimArray',
+    'get_indices', 'shape', 'MutableDenseNDimArray', 'ImmutableDenseNDimArray',
     'MutableSparseNDimArray', 'ImmutableSparseNDimArray', 'NDimArray',
-    'tensorproduct', 'tensorcontraction', 'derive_by_array', 'permutedims',
-    'Array', 'DenseNDimArray', 'SparseNDimArray',
+    'tensorproduct', 'tensorcontraction', 'tensordiagonal', 'derive_by_array',
+    'permutedims', 'Array', 'DenseNDimArray', 'SparseNDimArray',
 
     # sympy.parsing
     'parse_expr',
@@ -492,9 +488,6 @@ __all__ = [
 
     # sympy.testing
     'test', 'doctest',
-
-    # sympy.deprecated:
-    'C', 'ClassRegistry', 'class_registry',
 ]
 
 
@@ -516,10 +509,7 @@ __all__.extend([
     'algebras',
     'assumptions',
     'calculus',
-    'codegen',
-    'combinatorics',
     'concrete',
-    'deprecated',
     'discrete',
     'external',
     'functions',
@@ -536,154 +526,3 @@ __all__.extend([
     'tensor',
     'utilities',
 ])
-
-
-#===========================================================================#
-#                                                                           #
-# XXX: The names listed in _DEPRECATED_IMPORTS below were importable before #
-# sympy 1.6 using                                                           #
-#                                                                           #
-#          from sympy import *                                              #
-#                                                                           #
-# This happened implicitly because there was no __all__ defined in this     #
-# __init__.py file. The plan is to remove them but for now they remain      #
-# importable but will give a deprecation warning when used. In future these #
-# names will be removed and will not be importable from here.               #
-#                                                                           #
-#===========================================================================#
-
-
-class DeprecatedImportModule:
-    # Add a docstring that someone can see if calling help on these objects
-    """Deprecated imported module object.
-
-    See https://github.com/sympy/sympy/pull/19316
-
-    This is a wrapper around a module that has been imported incorrectly.
-    Previously this module was importable using
-
-        from sympy import *
-
-    or (for example)
-
-        from sympy import add
-
-    However it was unintentional that this module would be imported in that
-    way and it will be removed in a future sympy version. If you do need to
-    use this module then the correct way to import it is to give its full
-    module path e.g.
-
-        import sympy.core.add as add
-    """
-
-    from sympy.utilities.exceptions import SymPyDeprecationWarning as Warn
-    import sys
-    sympy = sys.modules[__name__]
-
-    _DEPRECATED_IMPORTS = [
-        'sympy.concrete.expr_with_intlimits',
-        'sympy.concrete.expr_with_limits',
-        'sympy.concrete.gosper',
-        'sympy.concrete.products',
-        'sympy.concrete.summations',
-        'sympy.core.add',
-        'sympy.core.basic',
-        'sympy.core.cache',
-        'sympy.core.compatibility',
-        'sympy.core.containers',
-        'sympy.core.coreerrors',
-        'sympy.core.decorators',
-        'sympy.core.expr',
-        'sympy.core.exprtools',
-        'sympy.core.facts',
-        'sympy.core.function',
-        'sympy.core.logic',
-        'sympy.core.mod',
-        'sympy.core.mul',
-        'sympy.core.multidimensional',
-        'sympy.core.numbers',
-        'sympy.core.operations',
-        'sympy.core.power',
-        'sympy.core.relational',
-        'sympy.core.rules',
-        'sympy.core.singleton',
-        'sympy.core.symbol',
-        'sympy.discrete.convolutions',
-        'sympy.geometry.curve',
-        'sympy.geometry.ellipse',
-        'sympy.geometry.entity',
-        'sympy.geometry.exceptions',
-        'sympy.geometry.line',
-        'sympy.geometry.parabola',
-        'sympy.geometry.plane',
-        'sympy.geometry.point',
-        'sympy.geometry.polygon',
-        'sympy.geometry.util',
-        'sympy.integrals.integrals',
-        'sympy.integrals.manualintegrate',
-        'sympy.integrals.meijerint',
-        'sympy.integrals.singularityfunctions',
-        'sympy.integrals.transforms',
-        'sympy.integrals.trigonometry',
-        'sympy.logic.boolalg',
-        'sympy.logic.inference',
-        'sympy.matrices.common',
-        'sympy.matrices.dense',
-        'sympy.matrices.expressions',
-        'sympy.matrices.immutable',
-        'sympy.matrices.matrices',
-        'sympy.matrices.sparse',
-        'sympy.matrices.sparsetools',
-        'sympy.ntheory.factor_',
-        'sympy.ntheory.generate',
-        'sympy.ntheory.multinomial',
-        'sympy.ntheory.partitions_',
-        'sympy.ntheory.primetest',
-        'sympy.ntheory.residue_ntheory',
-        'sympy.sets.conditionset',
-        'sympy.sets.contains',
-        'sympy.sets.fancysets',
-        'sympy.sets.ordinals',
-        'sympy.sets.powerset',
-        'sympy.sets.sets',
-        'sympy.simplify.cse_main',
-        'sympy.simplify.cse_opts',
-        'sympy.simplify.epathtools',
-        'sympy.simplify.traversaltools',
-        'sympy.solvers.bivariate',
-        'sympy.solvers.deutils',
-        'sympy.solvers.inequalities',
-        'sympy.solvers.ode',
-        'sympy.solvers.pde',
-        'sympy.solvers.polysys',
-        'sympy.solvers.recurr',
-        'sympy.solvers.solvers',
-        'sympy.tensor.array',
-        'sympy.tensor.index_methods',
-        'sympy.tensor.indexed'
-    ]
-
-    def __init__(self, modname):
-        from importlib import import_module
-        self.modname = modname
-        self.mod = import_module(modname)
-
-    def __getattr__(self, name):
-        self.Warn(
-            feature="importing %s with 'from sympy import *'" % self.modname,
-            useinstead="import %s" % self.modname,
-            issue=18245,
-            deprecated_since_version="1.6").warn()
-        return getattr(self.mod, name)
-
-    @classmethod
-    def inject_imports(cls):
-        for modname in cls._DEPRECATED_IMPORTS:
-            name = modname.split('.')[-1]
-            deprecated_mod = cls(modname)
-            setattr(cls.sympy, name, deprecated_mod)
-            __all__.append(name)
-
-
-DeprecatedImportModule.inject_imports()
-del DeprecatedImportModule
