@@ -96,11 +96,11 @@ def get_known_facts_cnf():
         Q.invertible | Q.singular,
         Q.algebraic | ~Q.rational,
         Q.antihermitian | ~Q.imaginary,
+        Q.complex_elements | ~Q.real_elements,
         Q.complex | ~Q.algebraic,
         Q.complex | ~Q.imaginary,
         Q.complex | ~Q.real,
         Q.complex | ~Q.transcendental,
-        Q.complex_elements | ~Q.real_elements,
         Q.even | ~Q.zero,
         Q.extended_real | ~Q.infinite,
         Q.extended_real | ~Q.real,
@@ -124,15 +124,15 @@ def get_known_facts_cnf():
         Q.nonzero | ~Q.positive,
         Q.normal | ~Q.diagonal,
         Q.normal | ~Q.unitary,
-        Q.positive | ~Q.prime,
         Q.positive_definite | ~Q.orthogonal,
+        Q.positive | ~Q.prime,
         Q.rational | ~Q.integer,
+        Q.real_elements | ~Q.integer_elements,
         Q.real | ~Q.irrational,
         Q.real | ~Q.negative,
         Q.real | ~Q.positive,
         Q.real | ~Q.rational,
         Q.real | ~Q.zero,
-        Q.real_elements | ~Q.integer_elements,
         Q.square | ~Q.invertible,
         Q.square | ~Q.normal,
         Q.square | ~Q.symmetric,
@@ -179,11 +179,14 @@ def get_known_facts_dict():
         Q.composite: set([Q.composite]),
         Q.diagonal: set([Q.diagonal, Q.lower_triangular, Q.normal, Q.square,
         Q.symmetric, Q.triangular, Q.upper_triangular]),
+        Q.eq: set([Q.eq]),
         Q.even: set([Q.algebraic, Q.complex, Q.even, Q.extended_real,
         Q.finite, Q.hermitian, Q.integer, Q.rational, Q.real]),
         Q.extended_real: set([Q.extended_real]),
         Q.finite: set([Q.finite]),
         Q.fullrank: set([Q.fullrank]),
+        Q.ge: set([Q.ge]),
+        Q.gt: set([Q.gt]),
         Q.hermitian: set([Q.hermitian]),
         Q.imaginary: set([Q.antihermitian, Q.complex, Q.imaginary]),
         Q.infinite: set([Q.extended_real, Q.infinite]),
@@ -195,7 +198,10 @@ def get_known_facts_dict():
         Q.irrational: set([Q.complex, Q.extended_real, Q.finite, Q.hermitian,
         Q.irrational, Q.nonzero, Q.real]),
         Q.is_true: set([Q.is_true]),
+        Q.le: set([Q.le]),
         Q.lower_triangular: set([Q.lower_triangular, Q.triangular]),
+        Q.lt: set([Q.lt]),
+        Q.ne: set([Q.ne]),
         Q.negative: set([Q.complex, Q.extended_real, Q.hermitian, Q.negative,
         Q.nonpositive, Q.nonzero, Q.real]),
         Q.nonnegative: set([Q.complex, Q.extended_real, Q.hermitian,
