@@ -233,6 +233,9 @@ class Partition(FiniteSet):
         """
         Returns the "restricted growth string" of the partition.
 
+        Explanation
+        ===========
+
         The RGS is returned as a list of indices, L, where L[i] indicates
         the block in which element i appears. For example, in a partition
         of 3 elements (a, b, c) into 2 blocks ([c], [a, b]) the RGS is
@@ -265,6 +268,9 @@ class Partition(FiniteSet):
         """
         Creates a set partition from a restricted growth string.
 
+        Explanation
+        ===========
+
         The indices given in rgs are assumed to be the index
         of the element as given in elements *as provided* (the
         elements are not sorted by this routine). Block numbering
@@ -300,6 +306,9 @@ class IntegerPartition(Basic):
     """
     This class represents an integer partition.
 
+    Explanation
+    ===========
+
     In number theory and combinatorics, a partition of a positive integer,
     ``n``, also called an integer partition, is a way of writing ``n`` as a
     list of positive integers that sum to n. Two partitions that differ only
@@ -328,6 +337,9 @@ class IntegerPartition(Basic):
         """
         Generates a new IntegerPartition object from a list or dictionary.
 
+        Explantion
+        ==========
+
         The partition can be given as a list of positive integers or a
         dictionary of (integer, multiplicity) items. If the partition is
         preceded by an integer an error will be raised if the partition
@@ -376,7 +388,7 @@ class IntegerPartition(Basic):
         if not sum_ok and sum(partition) != integer:
             raise ValueError("Partition did not add to %s" % integer)
         if any(i < 1 for i in partition):
-            raise ValueError("The summands must all be positive.")
+            raise ValueError("All integer summands must be greater than one")
 
         obj = Basic.__new__(cls, integer, partition)
         obj.partition = list(partition)
