@@ -13,7 +13,7 @@ from sympy.core.power import integer_nthroot, isqrt, integer_log
 from sympy.polys.domains.groundtypes import PythonRational
 from sympy.utilities.decorator import conserve_mpmath_dps
 from sympy.utilities.iterables import permutations
-from sympy.testing.pytest import XFAIL, raises
+from sympy.testing.pytest import XFAIL, raises, _both_exp_pow
 
 from mpmath import mpf
 from mpmath.rational import mpq
@@ -187,6 +187,8 @@ def test_divmod():
     assert [divmod(i, -OO) for i in range(-2, 3)] == ANS
     assert divmod(S(3.5), S(-2)) == divmod(3.5, -2)
     assert divmod(-S(3.5), S(-2)) == divmod(-3.5, -2)
+    assert divmod(S(0.0), S(9)) == divmod(0.0, 9)
+    assert divmod(S(0), S(9.0)) == divmod(0, 9.0)
 
 
 def test_igcd():
@@ -434,6 +436,7 @@ def test_Float():
     a = Float(2) ** Float(4)
     assert eq(a.evalf(), Float(16))
     assert (S(.3) == S(.5)) is False
+
     mpf = (0, 5404319552844595, -52, 53)
     x_str =  Float((0, '13333333333333', -52, 53))
     x2_str = Float((0, '26666666666666', -53, 54))
@@ -580,7 +583,12 @@ def test_Float():
     for i, a in zip(u, v):
         assert Float(i) is a
 
-
+def test_zero_not_false():
+    # https://github.com/sympy/sympy/issues/20796
+    assert (S(0.0) == S.false) is False
+    assert (S.false == S(0.0)) is False
+    assert (S(0) == S.false) is False
+    assert (S.false == S(0)) is False
 
 @conserve_mpmath_dps
 def test_float_mpf():
@@ -846,6 +854,7 @@ def test_Div_By_Zero():
     assert -1/Float(0) is zoo
 
 
+@_both_exp_pow
 def test_Infinity_inequations():
     assert oo > pi
     assert not (oo < pi)
