@@ -1,4 +1,5 @@
 from sympy.external import import_module
+from sympy.testing.pytest import warns
 
 # fixes issue that arose in addressing issue 6533
 def test_no_stdlib_collections():
@@ -8,7 +9,7 @@ def test_no_stdlib_collections():
     '''
     import collections
     matplotlib = import_module('matplotlib',
-        __import__kwargs={'fromlist': ['cm', 'collections']},
+        import_kwargs={'fromlist': ['cm', 'collections']},
         min_module_version='1.1.0', catch=(RuntimeError,))
     if matplotlib:
         assert collections != matplotlib.collections
@@ -20,7 +21,7 @@ def test_no_stdlib_collections2():
     '''
     import collections
     matplotlib = import_module('matplotlib',
-        __import__kwargs={'fromlist': ['collections']},
+        import_kwargs={'fromlist': ['collections']},
         min_module_version='1.1.0', catch=(RuntimeError,))
     if matplotlib:
         assert collections != matplotlib.collections
@@ -29,7 +30,11 @@ def test_no_stdlib_collections3():
     '''make sure we get the right collections with no catch'''
     import collections
     matplotlib = import_module('matplotlib',
-        __import__kwargs={'fromlist': ['cm', 'collections']},
+        import_kwargs={'fromlist': ['cm', 'collections']},
         min_module_version='1.1.0')
     if matplotlib:
         assert collections != matplotlib.collections
+
+def test_min_module_version_python3_basestring_error():
+    with warns(UserWarning):
+        import_module('mpmath', min_module_version='1000.0.1')
