@@ -28,8 +28,8 @@ class re(Function):
     Examples
     ========
 
-    >>> from sympy import re, im, I, E
-    >>> from sympy.abc import x, y
+    >>> from sympy import re, im, I, E, symbols
+    >>> x, y = symbols('x y', real=True)
     >>> re(2*E)
     2*E
     >>> re(2*I + 17)
@@ -38,14 +38,30 @@ class re(Function):
     0
     >>> re(im(x) + x*I + 2)
     2
+    >>> re(5 + I + 2)
+    7
+
+    Parameters
+    ==========
+
+    arg : Expr
+        Real or complex expression.
+
+    Returns
+    =======
+
+    expr : Expr
+        Real part of expression.
 
     See Also
     ========
+
     im
     """
 
     is_extended_real = True
     unbranched = True  # implicitly works on the projection to C
+    _singularities = True  # non-holomorphic
 
     @classmethod
     def eval(cls, arg):
@@ -91,6 +107,7 @@ class re(Function):
     def as_real_imag(self, deep=True, **hints):
         """
         Returns the real number with a zero imaginary part.
+
         """
         return (self, S.Zero)
 
@@ -139,12 +156,26 @@ class im(Function):
     >>> from sympy.abc import x, y
     >>> im(2*E)
     0
-    >>> re(2*I + 17)
-    17
+    >>> im(2*I + 17)
+    2
     >>> im(x*I)
     re(x)
     >>> im(re(x) + y)
     im(y)
+    >>> im(2 + 3*I)
+    3
+
+    Parameters
+    ==========
+
+    arg : Expr
+        Real or complex expression.
+
+    Returns
+    =======
+
+    expr : Expr
+        Imaginary part of expression.
 
     See Also
     ========
@@ -154,6 +185,7 @@ class im(Function):
 
     is_extended_real = True
     unbranched = True  # implicitly works on the projection to C
+    _singularities = True  # non-holomorphic
 
     @classmethod
     def eval(cls, arg):
@@ -199,13 +231,6 @@ class im(Function):
         """
         Return the imaginary part with a zero real part.
 
-        Examples
-        ========
-
-        >>> from sympy.functions import im
-        >>> from sympy import I
-        >>> im(2 + 3*I).as_real_imag()
-        (3, 0)
         """
         return (self, S.Zero)
 
@@ -245,6 +270,9 @@ class sign(Function):
     """
     Returns the complex sign of an expression:
 
+    Explanation
+    ===========
+
     If the expression is real the sign will be:
 
         * 1 if expression is positive
@@ -276,6 +304,18 @@ class sign(Function):
     >>> _.evalf()
     0.707106781186548 + 0.707106781186548*I
 
+    Parameters
+    ==========
+
+    arg : Expr
+        Real or imaginary expression.
+
+    Returns
+    =======
+
+    expr : Expr
+        Complex sign of expression.
+
     See Also
     ========
 
@@ -283,6 +323,7 @@ class sign(Function):
     """
 
     is_complex = True
+    _singularities = True
 
     def doit(self, **hints):
         if self.args[0].is_zero is False:
@@ -302,13 +343,16 @@ class sign(Function):
                 elif a.is_extended_positive:
                     pass
                 else:
-                    ai = im(a)
-                    if a.is_imaginary and ai.is_comparable:  # i.e. a = I*real
-                        s *= S.ImaginaryUnit
-                        if ai.is_extended_negative:
-                            # can't use sign(ai) here since ai might not be
-                            # a Number
-                            s = -s
+                    if a.is_imaginary:
+                        ai = im(a)
+                        if ai.is_comparable:  # i.e. a = I*real
+                            s *= S.ImaginaryUnit
+                            if ai.is_extended_negative:
+                                # can't use sign(ai) here since ai might not be
+                                # a Number
+                                s = -s
+                        else:
+                            unk.append(a)
                     else:
                         unk.append(a)
             if c is S.One and len(unk) == len(args):
@@ -378,6 +422,15 @@ class sign(Function):
         ):
             return S.One
 
+    def _eval_nseries(self, x, n, logx, cdir=0):
+        arg0 = self.args[0]
+        x0 = arg0.subs(x, 0)
+        if x0 != 0:
+            return self.func(x0)
+        if cdir != 0:
+            cdir = arg0.dir(x, cdir)
+        return -S.One if re(cdir) < 0 else S.One
+
     def _sage_(self):
         import sage.all as sage
         return sage.sgn(self.args[0]._sage_())
@@ -391,14 +444,20 @@ class sign(Function):
         if arg.is_extended_real:
             return Heaviside(arg, H0=S(1)/2) * 2 - 1
 
+    def _eval_rewrite_as_Abs(self, arg, **kwargs):
+        return Piecewise((0, Eq(arg, 0)), (arg / Abs(arg), True))
+
     def _eval_simplify(self, **kwargs):
-        return self.func(self.args[0].factor())  # XXX include doit?
+        return self.func(factor_terms(self.args[0]))  # XXX include doit?
 
 
 class Abs(Function):
     """
     Return the absolute value of the argument.
 
+    Explanation
+    ===========
+
     This is an extension of the built-in function abs() to accept symbolic
     values.  If you pass a SymPy expression to the built-in abs(), it will
     pass it automatically to Abs().
@@ -406,7 +465,7 @@ class Abs(Function):
     Examples
     ========
 
-    >>> from sympy import Abs, Symbol, S
+    >>> from sympy import Abs, Symbol, S, I
     >>> Abs(-1)
     1
     >>> x = Symbol('x', real=True)
@@ -416,6 +475,10 @@ class Abs(Function):
     x**2
     >>> abs(-x) # The Python built-in
     Abs(x)
+    >>> Abs(3*x + 2*I)
+    sqrt(9*x**2 + 4)
+    >>> Abs(8*I)
+    8
 
     Note that the Python built-in will return either an Expr or int depending on
     the argument::
@@ -427,6 +490,19 @@ class Abs(Function):
 
     Abs will always return a sympy object.
 
+    Parameters
+    ==========
+
+    arg : Expr
+        Real or complex expression.
+
+    Returns
+    =======
+
+    expr : Expr
+        Absolute value returned can be an expression or integer depending on
+        input arg.
+
     See Also
     ========
 
@@ -437,18 +513,12 @@ class Abs(Function):
     is_extended_negative = False
     is_extended_nonnegative = True
     unbranched = True
+    _singularities = True  # non-holomorphic
 
     def fdiff(self, argindex=1):
         """
         Get the first derivative of the argument to Abs().
 
-        Examples
-        ========
-
-        >>> from sympy.abc import x
-        >>> from sympy.functions import Abs
-        >>> Abs(-x).fdiff()
-        sign(x)
         """
         if argindex == 1:
             return sign(self.args[0])
@@ -587,16 +657,12 @@ class Abs(Function):
                 return self.args[0]**(exponent - 1)*self
         return
 
-    def _eval_nseries(self, x, n, logx):
+    def _eval_nseries(self, x, n, logx, cdir=0):
         direction = self.args[0].leadterm(x)[0]
         if direction.has(log(x)):
             direction = direction.subs(log(x), logx)
         s = self.args[0]._eval_nseries(x, n=n, logx=logx)
-        when = Eq(direction, 0)
-        return Piecewise(
-            ((s.subs(direction, 0)), when),
-            (sign(direction)*s, True),
-        )
+        return (sign(direction)*s).expand()
 
     def _sage_(self):
         import sage.all as sage
@@ -647,12 +713,31 @@ class arg(Function):
     pi/2
     >>> arg(sqrt(2) + I*sqrt(2))
     pi/4
+    >>> arg(sqrt(3)/2 + I/2)
+    pi/6
+    >>> arg(4 + 3*I)
+    atan(3/4)
+    >>> arg(0.8 + 0.6*I)
+    0.643501108793284
+
+    Parameters
+    ==========
+
+    arg : Expr
+        Real or complex expression.
+
+    Returns
+    =======
+
+    value : Expr
+        Returns arc tangent of arg measured in radians.
 
     """
 
     is_extended_real = True
     is_real = True
     is_finite = True
+    _singularities = True  # non-holomorphic
 
     @classmethod
     def eval(cls, arg):
@@ -702,6 +787,22 @@ class conjugate(Function):
     2
     >>> conjugate(I)
     -I
+    >>> conjugate(3 + 2*I)
+    3 - 2*I
+    >>> conjugate(5 - I)
+    5 + I
+
+    Parameters
+    ==========
+
+    arg : Expr
+        Real or complex expression.
+
+    Returns
+    =======
+
+    arg : Expr
+        Complex conjugate of arg as real, imaginary or mixed expression.
 
     See Also
     ========
@@ -713,6 +814,7 @@ class conjugate(Function):
 
     .. [1] https://en.wikipedia.org/wiki/Complex_conjugation
     """
+    _singularities = True  # non-holomorphic
 
     @classmethod
     def eval(cls, arg):
@@ -745,6 +847,44 @@ class conjugate(Function):
 class transpose(Function):
     """
     Linear map transposition.
+
+    Examples
+    ========
+
+    >>> from sympy.functions import transpose
+    >>> from sympy.matrices import MatrixSymbol
+    >>> from sympy import Matrix
+    >>> A = MatrixSymbol('A', 25, 9)
+    >>> transpose(A)
+    A.T
+    >>> B = MatrixSymbol('B', 9, 22)
+    >>> transpose(B)
+    B.T
+    >>> transpose(A*B)
+    B.T*A.T
+    >>> M = Matrix([[4, 5], [2, 1], [90, 12]])
+    >>> M
+    Matrix([
+    [ 4,  5],
+    [ 2,  1],
+    [90, 12]])
+    >>> transpose(M)
+    Matrix([
+    [4, 2, 90],
+    [5, 1, 12]])
+
+    Parameters
+    ==========
+
+    arg : Matrix
+         Matrix or matrix expression to take the transpose of.
+
+    Returns
+    =======
+
+    value : Matrix
+        Transpose of arg.
+
     """
 
     @classmethod
@@ -766,6 +906,29 @@ class transpose(Function):
 class adjoint(Function):
     """
     Conjugate transpose or Hermite conjugation.
+
+    Examples
+    ========
+
+    >>> from sympy import adjoint
+    >>> from sympy.matrices import MatrixSymbol
+    >>> A = MatrixSymbol('A', 10, 5)
+    >>> adjoint(A)
+    Adjoint(A)
+
+    Parameters
+    ==========
+
+    arg : Matrix
+        Matrix or matrix expression to take the adjoint of.
+
+    Returns
+    =======
+
+    value : Matrix
+        Represents the conjugate transpose or Hermite
+        conjugation of arg.
+
     """
 
     @classmethod
@@ -790,7 +953,7 @@ class adjoint(Function):
         arg = printer._print(self.args[0])
         tex = r'%s^{\dagger}' % arg
         if exp:
-            tex = r'\left(%s\right)^{%s}' % (tex, printer._print(exp))
+            tex = r'\left(%s\right)^{%s}' % (tex, exp)
         return tex
 
     def _pretty(self, printer, *args):
@@ -812,6 +975,9 @@ class polar_lift(Function):
     Lift argument to the Riemann surface of the logarithm, using the
     standard branch.
 
+    Examples
+    ========
+
     >>> from sympy import Symbol, polar_lift, I
     >>> p = Symbol('p', polar=True)
     >>> x = Symbol('x')
@@ -829,6 +995,12 @@ class polar_lift(Function):
     >>> polar_lift(4*p)
     4*p
 
+    Parameters
+    ==========
+
+    arg : Expr
+        Real or complex expression.
+
     See Also
     ========
 
@@ -884,15 +1056,20 @@ class polar_lift(Function):
 class periodic_argument(Function):
     """
     Represent the argument on a quotient of the Riemann surface of the
-    logarithm. That is, given a period P, always return a value in
+    logarithm. That is, given a period $P$, always return a value in
     (-P/2, P/2], by using exp(P*I) == 1.
 
-    >>> from sympy import exp, exp_polar, periodic_argument, unbranched_argument
+    Examples
+    ========
+
+    >>> from sympy import exp_polar, periodic_argument
     >>> from sympy import I, pi
-    >>> unbranched_argument(exp(5*I*pi))
+    >>> periodic_argument(exp_polar(10*I*pi), 2*pi)
+    0
+    >>> periodic_argument(exp_polar(5*I*pi), 4*pi)
     pi
-    >>> unbranched_argument(exp_polar(5*I*pi))
-    5*pi
+    >>> from sympy import exp_polar, periodic_argument
+    >>> from sympy import I, pi
     >>> periodic_argument(exp_polar(5*I*pi), 2*pi)
     pi
     >>> periodic_argument(exp_polar(5*I*pi), 3*pi)
@@ -900,6 +1077,15 @@ class periodic_argument(Function):
     >>> periodic_argument(exp_polar(5*I*pi), pi)
     0
 
+    Parameters
+    ==========
+
+    ar : Expr
+        A polar number.
+
+    period : ExprT
+        The period $P$.
+
     See Also
     ========
 
@@ -970,6 +1156,24 @@ class periodic_argument(Function):
 
 
 def unbranched_argument(arg):
+    '''
+    Returns periodic argument of arg with period as infinity.
+
+    Examples
+    ========
+
+    >>> from sympy import exp_polar, unbranched_argument
+    >>> from sympy import I, pi
+    >>> unbranched_argument(exp_polar(15*I*pi))
+    15*pi
+    >>> unbranched_argument(exp_polar(7*I*pi))
+    7*pi
+
+    See also
+    ========
+
+    periodic_argument
+    '''
     return periodic_argument(arg, oo)
 
 
@@ -978,10 +1182,16 @@ class principal_branch(Function):
     Represent a polar number reduced to its principal branch on a quotient
     of the Riemann surface of the logarithm.
 
+    Explanation
+    ===========
+
     This is a function of two arguments. The first argument is a polar
-    number `z`, and the second one a positive real number of infinity, `p`.
+    number `z`, and the second one a positive real number or infinity, `p`.
     The result is "z mod exp_polar(I*p)".
 
+    Examples
+    ========
+
     >>> from sympy import exp_polar, principal_branch, oo, I, pi
     >>> from sympy.abc import z
     >>> principal_branch(z, oo)
@@ -991,6 +1201,15 @@ class principal_branch(Function):
     >>> principal_branch(exp_polar(2*pi*I)*3*z, 2*pi)
     3*principal_branch(z, 2*pi)
 
+    Parameters
+    ==========
+
+    x : Expr
+        A polar number.
+
+    period : Expr
+        Positive real number or infinity.
+
     See Also
     ========
 
@@ -1078,6 +1297,8 @@ def _polarify(eq, lift, pause=False):
         if lift:
             return polar_lift(r)
         return r
+    elif eq.is_Pow and eq.base == S.Exp1:
+        return eq.func(S.Exp1, _polarify(eq.exp, lift, pause=False))
     elif eq.is_Function:
         return eq.func(*[_polarify(arg, lift, pause=False) for arg in eq.args])
     elif isinstance(eq, Integral):
@@ -1112,6 +1333,9 @@ def polarify(eq, subs=True, lift=False):
     changed to their polar_lift()ed versions.
     Note that lift=True implies subs=False.
 
+    Examples
+    ========
+
     >>> from sympy import polarify, sin, I
     >>> from sympy.abc import x, y
     >>> expr = (-x)**y
@@ -1181,6 +1405,9 @@ def unpolarify(eq, subs={}, exponents_only=False):
     Also apply the substitution subs in the end. (This is a convenience, since
     ``unpolarify``, in a certain sense, undoes polarify.)
 
+    Examples
+    ========
+
     >>> from sympy import unpolarify, polar_lift, sin, I
     >>> unpolarify(polar_lift(I + 2))
     2 + I
