@@ -1,5 +1,3 @@
-from __future__ import print_function, division
-
 from sympy.core import Function, S, sympify
 from sympy.core.add import Add
 from sympy.core.containers import Tuple
@@ -15,7 +13,6 @@ from sympy.core.relational import Eq, Relational
 from sympy.core.singleton import Singleton
 from sympy.core.symbol import Dummy
 from sympy.core.rules import Transform
-from sympy.core.compatibility import with_metaclass, range
 from sympy.core.logic import fuzzy_and, fuzzy_or, _torf
 from sympy.logic.boolalg import And, Or
 
@@ -31,7 +28,7 @@ def _minmax_as_Piecewise(op, *args):
     return Piecewise(*ec)
 
 
-class IdentityFunction(with_metaclass(Singleton, Lambda)):
+class IdentityFunction(Lambda, metaclass=Singleton):
     """
     The identity function
 
@@ -50,6 +47,13 @@ class IdentityFunction(with_metaclass(Singleton, Lambda)):
         #construct "by hand" to avoid infinite loop
         return Expr.__new__(cls, Tuple(x), x)
 
+    @property
+    def args(self):
+        return ()
+
+    def __getnewargs__(self):
+        return ()
+
 Id = S.IdentityFunction
 
 ###############################################################################
@@ -58,17 +62,20 @@ Id = S.IdentityFunction
 
 
 def sqrt(arg, evaluate=None):
-    """The square root function
+    """Returns the principal square root.
 
-    sqrt(x) -> Returns the principal square root of x.
+    Parameters
+    ==========
 
-    The parameter evaluate determines if the expression should be evaluated.
-    If None, its value is taken from global_evaluate
+    evaluate : bool, optional
+        The parameter determines if the expression should be evaluated.
+        If ``None``, its value is taken from
+        ``global_parameters.evaluate``.
 
     Examples
     ========
 
-    >>> from sympy import sqrt, Symbol
+    >>> from sympy import sqrt, Symbol, S
     >>> x = Symbol('x')
 
     >>> sqrt(x)
@@ -113,6 +120,22 @@ def sqrt(arg, evaluate=None):
     >>> [rootof(x**2-3,i) for i in (0,1)]
     [-sqrt(3), sqrt(3)]
 
+    Although ``sqrt`` is printed, there is no ``sqrt`` function so looking for
+    ``sqrt`` in an expression will fail:
+
+    >>> from sympy.utilities.misc import func_name
+    >>> func_name(sqrt(x))
+    'Pow'
+    >>> sqrt(x).has(sqrt)
+    Traceback (most recent call last):
+      ...
+    sympy.core.sympify.SympifyError: SympifyError: <function sqrt at 0x10e8900d0>
+
+    To find ``sqrt`` look for ``Pow`` with an exponent of ``1/2``:
+
+    >>> (x + 1/sqrt(x)).find(lambda i: i.is_Pow and abs(i.exp) is S.Half)
+    {1/sqrt(x)}
+
     See Also
     ========
 
@@ -129,11 +152,15 @@ def sqrt(arg, evaluate=None):
 
 
 def cbrt(arg, evaluate=None):
-    """This function computes the principal cube root of `arg`, so
-    it's just a shortcut for `arg**Rational(1, 3)`.
+    """Returns the principal cube root.
 
-    The parameter evaluate determines if the expression should be evaluated.
-    If None, its value is taken from global_evaluate.
+    Parameters
+    ==========
+
+    evaluate : bool, optional
+        The parameter determines if the expression should be evaluated.
+        If ``None``, its value is taken from
+        ``global_parameters.evaluate``.
 
     Examples
     ========
@@ -182,11 +209,19 @@ def cbrt(arg, evaluate=None):
 
 
 def root(arg, n, k=0, evaluate=None):
-    """root(x, n, k) -> Returns the k-th n-th root of x, defaulting to the
-    principal root (k=0).
+    r"""Returns the *k*-th *n*-th root of ``arg``.
 
-    The parameter evaluate determines if the expression should be evaluated.
-    If None, its value is taken from global_evaluate.
+    Parameters
+    ==========
+
+    k : int, optional
+        Should be an integer in $\{0, 1, ..., n-1\}$.
+        Defaults to the principal root if $0$.
+
+    evaluate : bool, optional
+        The parameter determines if the expression should be evaluated.
+        If ``None``, its value is taken from
+        ``global_parameters.evaluate``.
 
     Examples
     ========
@@ -215,7 +250,7 @@ def root(arg, n, k=0, evaluate=None):
     The following examples show the roots of unity for n
     equal 2, 3 and 4:
 
-    >>> from sympy import rootof, I
+    >>> from sympy import rootof
 
     >>> [rootof(x**2 - 1, i) for i in range(2)]
     [-1, 1]
@@ -274,19 +309,27 @@ def root(arg, n, k=0, evaluate=None):
 
 
 def real_root(arg, n=None, evaluate=None):
-    """Return the real nth-root of arg if possible. If n is omitted then
-    all instances of (-n)**(1/odd) will be changed to -n**(1/odd); this
-    will only create a real root of a principal root -- the presence of
-    other factors may cause the result to not be real.
+    """Return the real *n*'th-root of *arg* if possible.
 
-    The parameter evaluate determines if the expression should be evaluated.
-    If None, its value is taken from global_evaluate.
+    Parameters
+    ==========
+
+    n : int or None, optional
+        If *n* is ``None``, then all instances of
+        ``(-n)**(1/odd)`` will be changed to ``-n**(1/odd)``.
+        This will only create a real root of a principal root.
+        The presence of other factors may cause the result to not be
+        real.
+
+    evaluate : bool, optional
+        The parameter determines if the expression should be evaluated.
+        If ``None``, its value is taken from
+        ``global_parameters.evaluate``.
 
     Examples
     ========
 
-    >>> from sympy import root, real_root, Rational
-    >>> from sympy.abc import x, n
+    >>> from sympy import root, real_root
 
     >>> real_root(-8, 3)
     -2
@@ -303,7 +346,6 @@ def real_root(arg, n=None, evaluate=None):
     >>> real_root(_)
     -2*(-1)**(2/3)
 
-
     See Also
     ========
 
@@ -523,8 +565,7 @@ class MinMaxBase(Expr, LatticeOp):
             elif arg == cls.identity:
                 continue
             elif arg.func == cls:
-                for x in arg.args:
-                    yield x
+                yield from arg.args
             else:
                 yield arg
 
@@ -608,9 +649,11 @@ class MinMaxBase(Expr, LatticeOp):
         d = abs(args[0] - self.func(*args[1:]))/2
         return (s + d if isinstance(self, Max) else s - d).rewrite(Abs)
 
-    def evalf(self, prec=None, **options):
-        return self.func(*[a.evalf(prec, **options) for a in self.args])
-    n = evalf
+    def evalf(self, n=15, **options):
+        return self.func(*[a.evalf(n, **options) for a in self.args])
+
+    def n(self, *args, **kwargs):
+        return self.evalf(*args, **kwargs)
 
     _eval_is_algebraic = lambda s: _torf(i.is_algebraic for i in s.args)
     _eval_is_antihermitian = lambda s: _torf(i.is_antihermitian for i in s.args)
