@@ -189,7 +189,7 @@ class KroneckerDelta(Function):
         # to make KroneckerDelta canonical
         # following lines will check if inputs are in order
         # if not, will return KroneckerDelta with correct order
-        if i is not min(i, j, key=default_sort_key):
+        if i != min(i, j, key=default_sort_key):
             if delta_range:
                 return cls(j, i, delta_range)
             else:
