@@ -1,19 +1,18 @@
 from sympy import (
     symbols, expand, expand_func, nan, oo, Float, conjugate, diff,
-    re, im, Abs, O, exp_polar, polar_lift, gruntz, limit,
+    re, im, O, exp_polar, polar_lift, gruntz, limit,
     Symbol, I, integrate, Integral, S,
     sqrt, sin, cos, sinc, sinh, cosh, exp, log, pi, EulerGamma,
     erf, erfc, erfi, erf2, erfinv, erfcinv, erf2inv,
     gamma, uppergamma,
     Ei, expint, E1, li, Li, Si, Ci, Shi, Chi,
     fresnels, fresnelc,
-    hyper, meijerg)
-
-from sympy.functions.special.error_functions import _erfs, _eis
+    hyper, meijerg, E, Rational)
 
+from sympy.core.expr import unchanged
 from sympy.core.function import ArgumentIndexError
-
-from sympy.utilities.pytest import raises
+from sympy.functions.special.error_functions import _erfs, _eis
+from sympy.testing.pytest import raises, slow
 
 x, y, z = symbols('x,y,z')
 w = Symbol("w", real=True)
@@ -21,7 +20,7 @@ n = Symbol("n", integer=True)
 
 
 def test_erf():
-    assert erf(nan) == nan
+    assert erf(nan) is nan
 
     assert erf(oo) == 1
     assert erf(-oo) == -1
@@ -38,6 +37,7 @@ def test_erf():
     assert erf(erfinv(x)) == x
     assert erf(erfcinv(x)) == 1 - x
     assert erf(erf2inv(0, x)) == x
+    assert erf(erf2inv(0, x, evaluate=False)) == x # To cover code in erf
     assert erf(erf2inv(0, erf(erfcinv(1 - erf(erfinv(x)))))) == x
 
     assert erf(I).is_real is False
@@ -46,6 +46,8 @@ def test_erf():
     assert conjugate(erf(z)) == erf(conjugate(z))
 
     assert erf(x).as_leading_term(x) == 2*x/sqrt(pi)
+    assert erf(x*y).as_leading_term(y) == 2*x*y/sqrt(pi)
+    assert (erf(x*y)/erf(y)).as_leading_term(y) == x
     assert erf(1/x).as_leading_term(x) == erf(1/x)
 
     assert erf(z).rewrite('uppergamma') == sqrt(z**2)*(1 - erfc(sqrt(z**2)))/z
@@ -56,7 +58,7 @@ def test_erf():
     assert erf(z).rewrite('fresnelc') == (1 + I)*(fresnelc(z*(1 - I)/sqrt(pi)) -
         I*fresnels(z*(1 - I)/sqrt(pi)))
     assert erf(z).rewrite('hyper') == 2*z*hyper([S.Half], [3*S.Half], -z**2)/sqrt(pi)
-    assert erf(z).rewrite('meijerg') == z*meijerg([S.Half], [], [0], [-S.Half], z**2)/sqrt(pi)
+    assert erf(z).rewrite('meijerg') == z*meijerg([S.Half], [], [0], [Rational(-1, 2)], z**2)/sqrt(pi)
     assert erf(z).rewrite('expint') == sqrt(z**2)/z - z*expint(S.Half, z**2)/sqrt(S.Pi)
 
     assert limit(exp(x)*exp(x**2)*(erf(x + 1/exp(x)) - erf(x)), x, oo) == \
@@ -64,21 +66,34 @@ def test_erf():
     assert limit((1 - erf(z))*exp(z**2)*z, z, oo) == 1/sqrt(pi)
     assert limit((1 - erf(x))*exp(x**2)*sqrt(pi)*x, x, oo) == 1
     assert limit(((1 - erf(x))*exp(x**2)*sqrt(pi)*x - 1)*2*x**2, x, oo) == -1
+    assert limit(erf(x)/x, x, 0) == 2/sqrt(pi)
+    assert limit(x**(-4) - sqrt(pi)*erf(x**2) / (2*x**6), x, 0) == S(1)/3
 
     assert erf(x).as_real_imag() == \
-        ((erf(re(x) - I*re(x)*Abs(im(x))/Abs(re(x)))/2 +
-         erf(re(x) + I*re(x)*Abs(im(x))/Abs(re(x)))/2,
-         I*(erf(re(x) - I*re(x)*Abs(im(x))/Abs(re(x))) -
-         erf(re(x) + I*re(x)*Abs(im(x))/Abs(re(x)))) *
-         re(x)*Abs(im(x))/(2*im(x)*Abs(re(x)))))
+        (erf(re(x) - I*im(x))/2 + erf(re(x) + I*im(x))/2,
+         -I*(-erf(re(x) - I*im(x)) + erf(re(x) + I*im(x)))/2)
+
+    assert erf(x).as_real_imag(deep=False) == \
+        (erf(re(x) - I*im(x))/2 + erf(re(x) + I*im(x))/2,
+         -I*(-erf(re(x) - I*im(x)) + erf(re(x) + I*im(x)))/2)
+
+    assert erf(w).as_real_imag() == (erf(w), 0)
+    assert erf(w).as_real_imag(deep=False) == (erf(w), 0)
+    # issue 13575
+    assert erf(I).as_real_imag() == (0, -I*erf(I))
 
     raises(ArgumentIndexError, lambda: erf(x).fdiff(2))
 
+    assert erf(x).inverse() == erfinv
+
 
 def test_erf_series():
     assert erf(x).series(x, 0, 7) == 2*x/sqrt(pi) - \
         2*x**3/3/sqrt(pi) + x**5/5/sqrt(pi) + O(x**7)
 
+    assert erf(x).series(x, oo) == \
+        -exp(-x**2)*(3/(4*x**5) - 1/(2*x**3) + 1/x + O(x**(-6), (x, oo)))/sqrt(pi) + 1
+
 
 def test_erf_evalf():
     assert abs( erf(Float(2.0)) - 0.995322265 ) < 1E-8 # XXX
@@ -93,10 +108,11 @@ def test__erfs():
     assert expand(erf(z).rewrite('tractable').diff(z).rewrite('intractable')) \
         == erf(z).diff(z)
     assert _erfs(z).rewrite("intractable") == (-erf(z) + 1)*exp(z**2)
+    raises(ArgumentIndexError, lambda: _erfs(z).fdiff(2))
 
 
 def test_erfc():
-    assert erfc(nan) == nan
+    assert erfc(nan) is nan
 
     assert erfc(oo) == 0
     assert erfc(-oo) == 2
@@ -112,9 +128,11 @@ def test_erfc():
     assert erfc(I).is_real is False
     assert erfc(0).is_real is True
 
+    assert erfc(erfinv(x)) == 1 - x
+
     assert conjugate(erfc(z)) == erfc(conjugate(z))
 
-    assert erfc(x).as_leading_term(x) == S.One
+    assert erfc(x).as_leading_term(x) is S.One
     assert erfc(1/x).as_leading_term(x) == erfc(1/x)
 
     assert erfc(z).rewrite('erf') == 1 - erf(z)
@@ -124,37 +142,46 @@ def test_erfc():
     assert erfc(z).rewrite('fresnelc') == 1 - (1 + I)*(fresnelc(z*(1 - I)/sqrt(pi)) -
         I*fresnels(z*(1 - I)/sqrt(pi)))
     assert erfc(z).rewrite('hyper') == 1 - 2*z*hyper([S.Half], [3*S.Half], -z**2)/sqrt(pi)
-    assert erfc(z).rewrite('meijerg') == 1 - z*meijerg([S.Half], [], [0], [-S.Half], z**2)/sqrt(pi)
+    assert erfc(z).rewrite('meijerg') == 1 - z*meijerg([S.Half], [], [0], [Rational(-1, 2)], z**2)/sqrt(pi)
     assert erfc(z).rewrite('uppergamma') == 1 - sqrt(z**2)*(1 - erfc(sqrt(z**2)))/z
     assert erfc(z).rewrite('expint') == S.One - sqrt(z**2)/z + z*expint(S.Half, z**2)/sqrt(S.Pi)
-    assert expand_func(erf(x) + erfc(x)) == S.One
+    assert erfc(z).rewrite('tractable') == _erfs(z)*exp(-z**2)
+    assert expand_func(erf(x) + erfc(x)) is S.One
 
     assert erfc(x).as_real_imag() == \
-        ((erfc(re(x) - I*re(x)*Abs(im(x))/Abs(re(x)))/2 +
-         erfc(re(x) + I*re(x)*Abs(im(x))/Abs(re(x)))/2,
-         I*(erfc(re(x) - I*re(x)*Abs(im(x))/Abs(re(x))) -
-         erfc(re(x) + I*re(x)*Abs(im(x))/Abs(re(x)))) *
-         re(x)*Abs(im(x))/(2*im(x)*Abs(re(x)))))
+        (erfc(re(x) - I*im(x))/2 + erfc(re(x) + I*im(x))/2,
+         -I*(-erfc(re(x) - I*im(x)) + erfc(re(x) + I*im(x)))/2)
 
+    assert erfc(x).as_real_imag(deep=False) == \
+        (erfc(re(x) - I*im(x))/2 + erfc(re(x) + I*im(x))/2,
+         -I*(-erfc(re(x) - I*im(x)) + erfc(re(x) + I*im(x)))/2)
+
+    assert erfc(w).as_real_imag() == (erfc(w), 0)
+    assert erfc(w).as_real_imag(deep=False) == (erfc(w), 0)
     raises(ArgumentIndexError, lambda: erfc(x).fdiff(2))
 
+    assert erfc(x).inverse() == erfcinv
+
 
 def test_erfc_series():
     assert erfc(x).series(x, 0, 7) == 1 - 2*x/sqrt(pi) + \
         2*x**3/3/sqrt(pi) - x**5/5/sqrt(pi) + O(x**7)
 
+    assert erfc(x).series(x, oo) == \
+            (3/(4*x**5) - 1/(2*x**3) + 1/x + O(x**(-6), (x, oo)))*exp(-x**2)/sqrt(pi)
+
 
 def test_erfc_evalf():
     assert abs( erfc(Float(2.0)) - 0.00467773 ) < 1E-8 # XXX
 
 
 def test_erfi():
-    assert erfi(nan) == nan
+    assert erfi(nan) is nan
 
-    assert erfi(oo) == S.Infinity
-    assert erfi(-oo) == S.NegativeInfinity
+    assert erfi(oo) is S.Infinity
+    assert erfi(-oo) is S.NegativeInfinity
 
-    assert erfi(0) == S.Zero
+    assert erfi(0) is S.Zero
 
     assert erfi(I*oo) == I
     assert erfi(-I*oo) == -I
@@ -164,6 +191,7 @@ def test_erfi():
     assert erfi(I*erfinv(x)) == I*x
     assert erfi(I*erfcinv(x)) == I*(1 - x)
     assert erfi(I*erf2inv(0, x)) == I*x
+    assert erfi(I*erf2inv(0, x, evaluate=False)) == I*x # To cover code in erfi
 
     assert erfi(I).is_real is False
     assert erfi(0).is_real is True
@@ -177,18 +205,22 @@ def test_erfi():
     assert erfi(z).rewrite('fresnelc') == (1 - I)*(fresnelc(z*(1 + I)/sqrt(pi)) -
         I*fresnels(z*(1 + I)/sqrt(pi)))
     assert erfi(z).rewrite('hyper') == 2*z*hyper([S.Half], [3*S.Half], z**2)/sqrt(pi)
-    assert erfi(z).rewrite('meijerg') == z*meijerg([S.Half], [], [0], [-S.Half], -z**2)/sqrt(pi)
+    assert erfi(z).rewrite('meijerg') == z*meijerg([S.Half], [], [0], [Rational(-1, 2)], -z**2)/sqrt(pi)
     assert erfi(z).rewrite('uppergamma') == (sqrt(-z**2)/z*(uppergamma(S.Half,
         -z**2)/sqrt(S.Pi) - S.One))
     assert erfi(z).rewrite('expint') == sqrt(-z**2)/z - z*expint(S.Half, -z**2)/sqrt(S.Pi)
+    assert erfi(z).rewrite('tractable') == -I*(-_erfs(I*z)*exp(z**2) + 1)
     assert expand_func(erfi(I*z)) == I*erf(z)
 
     assert erfi(x).as_real_imag() == \
-        ((erfi(re(x) - I*re(x)*Abs(im(x))/Abs(re(x)))/2 +
-         erfi(re(x) + I*re(x)*Abs(im(x))/Abs(re(x)))/2,
-         I*(erfi(re(x) - I*re(x)*Abs(im(x))/Abs(re(x))) -
-         erfi(re(x) + I*re(x)*Abs(im(x))/Abs(re(x)))) *
-         re(x)*Abs(im(x))/(2*im(x)*Abs(re(x)))))
+        (erfi(re(x) - I*im(x))/2 + erfi(re(x) + I*im(x))/2,
+         -I*(-erfi(re(x) - I*im(x)) + erfi(re(x) + I*im(x)))/2)
+    assert erfi(x).as_real_imag(deep=False) == \
+        (erfi(re(x) - I*im(x))/2 + erfi(re(x) + I*im(x))/2,
+         -I*(-erfi(re(x) - I*im(x)) + erfi(re(x) + I*im(x)))/2)
+
+    assert erfi(w).as_real_imag() == (erfi(w), 0)
+    assert erfi(w).as_real_imag(deep=False) == (erfi(w), 0)
 
     raises(ArgumentIndexError, lambda: erfi(x).fdiff(2))
 
@@ -197,6 +229,9 @@ def test_erfi_series():
     assert erfi(x).series(x, 0, 7) == 2*x/sqrt(pi) + \
         2*x**3/3/sqrt(pi) + x**5/5/sqrt(pi) + O(x**7)
 
+    assert erfi(x).series(x, oo) == \
+        (3/(4*x**5) + 1/(2*x**3) + 1/x + O(x**(-6), (x, oo)))*exp(x**2)/sqrt(pi) - I
+
 
 def test_erfi_evalf():
     assert abs( erfi(Float(2.0)) - 18.5648024145756 ) < 1E-13  # XXX
@@ -204,9 +239,9 @@ def test_erfi_evalf():
 
 def test_erf2():
 
-    assert erf2(0, 0) == S.Zero
-    assert erf2(x, x) == S.Zero
-    assert erf2(nan, 0) == nan
+    assert erf2(0, 0) is S.Zero
+    assert erf2(x, x) is S.Zero
+    assert erf2(nan, 0) is nan
 
     assert erf2(-oo,  y) ==  erf(y) + 1
     assert erf2( oo,  y) ==  erf(y) - 1
@@ -235,20 +270,31 @@ def test_erf2():
     assert erf2(x, y).rewrite('erfc') == erfc(x) - erfc(y)
     assert erf2(x, y).rewrite('erfi') == I*(erfi(I*x) - erfi(I*y))
 
-    raises(ArgumentIndexError, lambda: erfi(x).fdiff(3))
+    assert erf2(x, y).diff(x) == erf2(x, y).fdiff(1)
+    assert erf2(x, y).diff(y) == erf2(x, y).fdiff(2)
+    assert erf2(x, y).diff(x) == -2*exp(-x**2)/sqrt(pi)
+    assert erf2(x, y).diff(y) == 2*exp(-y**2)/sqrt(pi)
+    raises(ArgumentIndexError, lambda: erf2(x, y).fdiff(3))
+
+    assert erf2(x, y).is_extended_real is None
+    xr, yr = symbols('xr yr', extended_real=True)
+    assert erf2(xr, yr).is_extended_real is True
 
 
 def test_erfinv():
     assert erfinv(0) == 0
-    assert erfinv(1) == S.Infinity
-    assert erfinv(nan) == S.NaN
+    assert erfinv(1) is S.Infinity
+    assert erfinv(nan) is S.NaN
+    assert erfinv(-1) is S.NegativeInfinity
 
     assert erfinv(erf(w)) == w
     assert erfinv(erf(-w)) == -w
 
     assert erfinv(x).diff() == sqrt(pi)*exp(erfinv(x)**2)/2
+    raises(ArgumentIndexError, lambda: erfinv(x).fdiff(2))
 
     assert erfinv(z).rewrite('erfcinv') == erfcinv(1-z)
+    assert erfinv(z).inverse() == erf
 
 
 def test_erfinv_evalf():
@@ -257,23 +303,30 @@ def test_erfinv_evalf():
 
 def test_erfcinv():
     assert erfcinv(1) == 0
-    assert erfcinv(0) == S.Infinity
-    assert erfcinv(nan) == S.NaN
+    assert erfcinv(0) is S.Infinity
+    assert erfcinv(nan) is S.NaN
 
     assert erfcinv(x).diff() == -sqrt(pi)*exp(erfcinv(x)**2)/2
+    raises(ArgumentIndexError, lambda: erfcinv(x).fdiff(2))
 
     assert erfcinv(z).rewrite('erfinv') == erfinv(1-z)
+    assert erfcinv(z).inverse() == erfc
 
 
 def test_erf2inv():
-    assert erf2inv(0, 0) == S.Zero
-    assert erf2inv(0, 1) == S.Infinity
-    assert erf2inv(1, 0) == S.One
+    assert erf2inv(0, 0) is S.Zero
+    assert erf2inv(0, 1) is S.Infinity
+    assert erf2inv(1, 0) is S.One
     assert erf2inv(0, y) == erfinv(y)
-    assert erf2inv(oo,y) == erfcinv(-y)
+    assert erf2inv(oo, y) == erfcinv(-y)
+    assert erf2inv(x, 0) == x
+    assert erf2inv(x, oo) == erfinv(x)
+    assert erf2inv(nan, 0) is nan
+    assert erf2inv(0, nan) is nan
 
     assert erf2inv(x, y).diff(x) == exp(-x**2 + erf2inv(x, y)**2)
     assert erf2inv(x, y).diff(y) == sqrt(pi)*exp(erf2inv(x, y)**2)/2
+    raises(ArgumentIndexError, lambda: erf2inv(x, y).fdiff(3))
 
 
 # NOTE we multiply by exp_polar(I*pi) and need this to be on the principal
@@ -281,7 +334,7 @@ def test_erf2inv():
 
 
 def mytn(expr1, expr2, expr3, x, d=0):
-    from sympy.utilities.randtest import verify_numerically, random_complex_number
+    from sympy.testing.randtest import verify_numerically, random_complex_number
     subs = {}
     for a in expr1.free_symbols:
         if a != x:
@@ -291,7 +344,7 @@ def mytn(expr1, expr2, expr3, x, d=0):
 
 
 def mytd(expr1, expr2, x):
-    from sympy.utilities.randtest import test_derivative_numerically, \
+    from sympy.testing.randtest import test_derivative_numerically, \
         random_complex_number
     subs = {}
     for a in expr1.free_symbols:
@@ -301,7 +354,6 @@ def mytd(expr1, expr2, x):
 
 
 def tn_branch(func, s=None):
-    from sympy import I, pi, exp_polar
     from random import uniform
 
     def fn(x):
@@ -316,6 +368,10 @@ def tn_branch(func, s=None):
 
 
 def test_ei():
+    assert Ei(0) is S.NegativeInfinity
+    assert Ei(oo) is S.Infinity
+    assert Ei(-oo) is S.Zero
+
     assert tn_branch(Ei)
     assert mytd(Ei(x), exp(x)/x, x)
     assert mytn(Ei(x), Ei(x).rewrite(uppergamma),
@@ -337,8 +393,13 @@ def test_ei():
 
     assert Ei(x).series(x) == EulerGamma + log(x) + x + x**2/4 + \
         x**3/18 + x**4/96 + x**5/600 + O(x**6)
+    assert Ei(x).series(x, 1, 3) == Ei(1) + E*(x - 1) + O((x - 1)**3, (x, 1))
+    assert Ei(x).series(x, oo) == \
+        (120/x**5 + 24/x**4 + 6/x**3 + 2/x**2 + 1/x + 1 + O(x**(-6), (x, oo)))*exp(x)/x
 
     assert str(Ei(cos(2)).evalf(n=10)) == '-0.6760647401'
+    raises(ArgumentIndexError, lambda: Ei(x).fdiff(2))
+
 
 def test_expint():
     assert mytn(expint(x, y), expint(x, y).rewrite(uppergamma),
@@ -351,7 +412,7 @@ def test_expint():
 
     assert expint(-4, x) == exp(-x)/x + 4*exp(-x)/x**2 + 12*exp(-x)/x**3 \
         + 24*exp(-x)/x**4 + 24*exp(-x)/x**5
-    assert expint(-S(3)/2, x) == \
+    assert expint(Rational(-3, 2), x) == \
         exp(-x)/x + 3*exp(-x)/(2*x**2) + 3*sqrt(pi)*erfc(sqrt(x))/(4*x**S('5/2'))
 
     assert tn_branch(expint, 1)
@@ -367,6 +428,8 @@ def test_expint():
     assert expint(2, x*exp_polar(2*I*pi)) == 2*I*pi*x + expint(2, x)
     assert expint(2, x*exp_polar(-2*I*pi)) == -2*I*pi*x + expint(2, x)
     assert expint(1, x).rewrite(Ei).rewrite(expint) == expint(1, x)
+    assert expint(x, y).rewrite(Ei) == expint(x, y)
+    assert expint(x, y).rewrite(Ci) == expint(x, y)
 
     assert mytn(E1(x), E1(x).rewrite(Shi), Shi(x) - Chi(x), x)
     assert mytn(E1(polar_lift(I)*x), E1(polar_lift(I)*x).rewrite(Si),
@@ -377,17 +440,27 @@ def test_expint():
     assert mytn(expint(3, x), expint(3, x).rewrite(Ei).rewrite(expint),
                 x**2*E1(x)/2 + (1 - x)*exp(-x)/2, x)
 
-    assert expint(S(3)/2, z).nseries(z) == \
+    assert expint(Rational(3, 2), z).nseries(z) == \
         2 + 2*z - z**2/3 + z**3/15 - z**4/84 + z**5/540 - \
         2*sqrt(pi)*sqrt(z) + O(z**6)
 
     assert E1(z).series(z) == -EulerGamma - log(z) + z - \
         z**2/4 + z**3/18 - z**4/96 + z**5/600 + O(z**6)
 
-    assert expint(4, z).series(z) == S(1)/3 - z/2 + z**2/2 + \
-        z**3*(log(z)/6 - S(11)/36 + EulerGamma/6) - z**4/24 + \
+    assert expint(4, z).series(z) == Rational(1, 3) - z/2 + z**2/2 + \
+        z**3*(log(z)/6 - Rational(11, 36) + EulerGamma/6 - I*pi/6) - z**4/24 + \
         z**5/240 + O(z**6)
 
+    assert expint(n, x).series(x, oo, n=3) == \
+        (n*(n + 1)/x**2 - n/x + 1 + O(x**(-3), (x, oo)))*exp(-x)/x
+
+    assert expint(z, y).series(z, 0, 2) == exp(-y)/y - z*meijerg(((), (1, 1)),
+                                  ((0, 0, 1), ()), y)/y + O(z**2)
+    raises(ArgumentIndexError, lambda: expint(x, y).fdiff(3))
+
+    neg = Symbol('neg', negative=True)
+    assert Ei(neg).rewrite(Si) == Shi(neg) + Chi(neg) - I*pi
+
 
 def test__eis():
     assert _eis(z).diff(z) == -_eis(z) + 1/z
@@ -407,7 +480,8 @@ def test__eis():
         == Ei(z).diff(z)
 
     assert _eis(z).series(z, n=3) == EulerGamma + log(z) + z*(-log(z) - \
-        EulerGamma + 1) + z**2*(log(z)/2 - S(3)/4 + EulerGamma/2) + O(z**3*log(z))
+        EulerGamma + 1) + z**2*(log(z)/2 - Rational(3, 4) + EulerGamma/2) + O(z**3*log(z))
+    raises(ArgumentIndexError, lambda: _eis(z).fdiff(2))
 
 
 def tn_arg(func):
@@ -430,17 +504,19 @@ def test_li():
     zn = Symbol("z", negative=True)
 
     assert li(0) == 0
-    assert li(1) == -oo
-    assert li(oo) == oo
+    assert li(1) is -oo
+    assert li(oo) is oo
 
     assert isinstance(li(z), li)
+    assert unchanged(li, -zp)
+    assert unchanged(li, zn)
 
     assert diff(li(z), z) == 1/log(z)
 
     assert conjugate(li(z)) == li(conjugate(z))
     assert conjugate(li(-zr)) == li(-zr)
-    assert conjugate(li(-zp)) == conjugate(li(-zp))
-    assert conjugate(li(zn)) == conjugate(li(zn))
+    assert unchanged(conjugate, li(-zp))
+    assert unchanged(conjugate, li(zn))
 
     assert li(z).rewrite(Li) == Li(z) + li(2)
     assert li(z).rewrite(Ei) == Ei(log(z))
@@ -460,11 +536,14 @@ def test_li():
                                       meijerg(((), (1,)), ((0, 0), ()), -log(z)))
 
     assert gruntz(1/li(z), z, oo) == 0
+    assert li(z).series(z) == log(z)**5/600 + log(z)**4/96 + log(z)**3/18 + log(z)**2/4 + \
+            log(z) + log(log(z)) + EulerGamma
+    raises(ArgumentIndexError, lambda: li(z).fdiff(2))
 
 
 def test_Li():
     assert Li(2) == 0
-    assert Li(oo) == oo
+    assert Li(oo) is oo
 
     assert isinstance(Li(z), Li)
 
@@ -472,6 +551,9 @@ def test_Li():
 
     assert gruntz(1/Li(z), z, oo) == 0
     assert Li(z).rewrite(li) == li(z) - li(2)
+    assert Li(z).series(z) == \
+        log(z)**5/600 + log(z)**4/96 + log(z)**3/18 + log(z)**2/4 + log(z) + log(log(z)) - li(2) + EulerGamma
+    raises(ArgumentIndexError, lambda: Li(z).fdiff(2))
 
 
 def test_si():
@@ -488,8 +570,8 @@ def test_si():
 
     assert Si(oo) == pi/2
     assert Si(-oo) == -pi/2
-    assert Shi(oo) == oo
-    assert Shi(-oo) == -oo
+    assert Shi(oo) is oo
+    assert Shi(-oo) is -oo
 
     assert mytd(Si(x), sin(x)/x, x)
     assert mytd(Shi(x), sinh(x)/x, x)
@@ -516,6 +598,10 @@ def test_si():
     assert Si(x).nseries(x, 1, n=3) == \
         Si(1) + (x - 1)*sin(1) + (x - 1)**2*(-sin(1)/2 + cos(1)/2) + O((x - 1)**3, (x, 1))
 
+    assert Si(x).series(x, oo) == pi/2 - (- 6/x**3 + 1/x \
+        + O(x**(-7), (x, oo)))*sin(x)/x - (24/x**4 - 2/x**2 + 1 \
+        + O(x**(-7), (x, oo)))*cos(x)/x
+
     t = Symbol('t', Dummy=True)
     assert Si(x).rewrite(sinc) == Integral(sinc(t), (t, 0, x))
 
@@ -541,8 +627,8 @@ def test_ci():
 
     assert Ci(oo) == 0
     assert Ci(-oo) == I*pi
-    assert Chi(oo) == oo
-    assert Chi(-oo) == oo
+    assert Chi(oo) is oo
+    assert Chi(-oo) is oo
 
     assert mytd(Ci(x), cos(x)/x, x)
     assert mytd(Chi(x), cosh(x)/x, x)
@@ -555,20 +641,29 @@ def test_ci():
     assert tn_arg(Ci)
     assert tn_arg(Chi)
 
-    from sympy import O, EulerGamma, log, limit
     assert Ci(x).nseries(x, n=4) == \
         EulerGamma + log(x) - x**2/4 + x**4/96 + O(x**5)
     assert Chi(x).nseries(x, n=4) == \
         EulerGamma + log(x) + x**2/4 + x**4/96 + O(x**5)
+
+    assert Ci(x).series(x, oo) == -cos(x)*(-6/x**3 + 1/x \
+        + O(x**(-7), (x, oo)))/x + (24/x**4 - 2/x**2 + 1 \
+        + O(x**(-7), (x, oo)))*sin(x)/x
     assert limit(log(x) - Ci(2*x), x, 0) == -log(2) - EulerGamma
+    assert Ci(x).rewrite(uppergamma) == -expint(1, x*exp_polar(-I*pi/2))/2 -\
+                                        expint(1, x*exp_polar(I*pi/2))/2
+    assert Ci(x).rewrite(expint) == -expint(1, x*exp_polar(-I*pi/2))/2 -\
+                                        expint(1, x*exp_polar(I*pi/2))/2
+    raises(ArgumentIndexError, lambda: Ci(x).fdiff(2))
 
 
 def test_fresnel():
     assert fresnels(0) == 0
     assert fresnels(oo) == S.Half
-    assert fresnels(-oo) == -S.Half
+    assert fresnels(-oo) == Rational(-1, 2)
+    assert fresnels(I*oo) == -I*S.Half
 
-    assert fresnels(z) == fresnels(z)
+    assert unchanged(fresnels, z)
     assert fresnels(-z) == -fresnels(z)
     assert fresnels(I*z) == -I*fresnels(z)
     assert fresnels(-I*z) == I*fresnels(z)
@@ -581,37 +676,46 @@ def test_fresnel():
         erf((S.One + I)/2*sqrt(pi)*z) - I*erf((S.One - I)/2*sqrt(pi)*z))
 
     assert fresnels(z).rewrite(hyper) == \
-        pi*z**3/6 * hyper([S(3)/4], [S(3)/2, S(7)/4], -pi**2*z**4/16)
+        pi*z**3/6 * hyper([Rational(3, 4)], [Rational(3, 2), Rational(7, 4)], -pi**2*z**4/16)
 
     assert fresnels(z).series(z, n=15) == \
         pi*z**3/6 - pi**3*z**7/336 + pi**5*z**11/42240 + O(z**15)
 
-    assert fresnels(w).is_real is True
+    assert fresnels(w).is_extended_real is True
+    assert fresnels(w).is_finite is True
 
-    assert fresnels(z).as_real_imag() == \
-        ((fresnels(re(z) - I*re(z)*Abs(im(z))/Abs(re(z)))/2 +
-          fresnels(re(z) + I*re(z)*Abs(im(z))/Abs(re(z)))/2,
-          I*(fresnels(re(z) - I*re(z)*Abs(im(z))/Abs(re(z))) -
-          fresnels(re(z) + I*re(z)*Abs(im(z))/Abs(re(z)))) *
-          re(z)*Abs(im(z))/(2*im(z)*Abs(re(z)))))
+    assert fresnels(z).is_extended_real is None
+    assert fresnels(z).is_finite is None
+
+    assert fresnels(z).as_real_imag() == (fresnels(re(z) - I*im(z))/2 +
+                    fresnels(re(z) + I*im(z))/2,
+                    -I*(-fresnels(re(z) - I*im(z)) + fresnels(re(z) + I*im(z)))/2)
+
+    assert fresnels(z).as_real_imag(deep=False) == (fresnels(re(z) - I*im(z))/2 +
+                    fresnels(re(z) + I*im(z))/2,
+                    -I*(-fresnels(re(z) - I*im(z)) + fresnels(re(z) + I*im(z)))/2)
+
+    assert fresnels(w).as_real_imag() == (fresnels(w), 0)
+    assert fresnels(w).as_real_imag(deep=True) == (fresnels(w), 0)
 
     assert fresnels(2 + 3*I).as_real_imag() == (
-        fresnels(2 + 3*I)/2 + fresnels(2 - 3*I)/2,
-        I*(fresnels(2 - 3*I) - fresnels(2 + 3*I))/2
-    )
+            fresnels(2 + 3*I)/2 + fresnels(2 - 3*I)/2,
+            -I*(fresnels(2 + 3*I) - fresnels(2 - 3*I))/2
+            )
 
     assert expand_func(integrate(fresnels(z), z)) == \
         z*fresnels(z) + cos(pi*z**2/2)/pi
 
-    assert fresnels(z).rewrite(meijerg) == sqrt(2)*pi*z**(S(9)/4) * \
-        meijerg(((), (1,)), ((S(3)/4,),
-        (S(1)/4, 0)), -pi**2*z**4/16)/(2*(-z)**(S(3)/4)*(z**2)**(S(3)/4))
+    assert fresnels(z).rewrite(meijerg) == sqrt(2)*pi*z**Rational(9, 4) * \
+        meijerg(((), (1,)), ((Rational(3, 4),),
+        (Rational(1, 4), 0)), -pi**2*z**4/16)/(2*(-z)**Rational(3, 4)*(z**2)**Rational(3, 4))
 
     assert fresnelc(0) == 0
     assert fresnelc(oo) == S.Half
-    assert fresnelc(-oo) == -S.Half
+    assert fresnelc(-oo) == Rational(-1, 2)
+    assert fresnelc(I*oo) == I*S.Half
 
-    assert fresnelc(z) == fresnelc(z)
+    assert unchanged(fresnelc, z)
     assert fresnelc(-z) == -fresnelc(z)
     assert fresnelc(I*z) == I*fresnelc(z)
     assert fresnelc(-I*z) == -I*fresnelc(z)
@@ -624,47 +728,31 @@ def test_fresnel():
         erf((S.One + I)/2*sqrt(pi)*z) + I*erf((S.One - I)/2*sqrt(pi)*z))
 
     assert fresnelc(z).rewrite(hyper) == \
-        z * hyper([S.One/4], [S.One/2, S(5)/4], -pi**2*z**4/16)
+        z * hyper([Rational(1, 4)], [S.Half, Rational(5, 4)], -pi**2*z**4/16)
 
-    assert fresnelc(z).series(z, n=15) == \
-        z - pi**2*z**5/40 + pi**4*z**9/3456 - pi**6*z**13/599040 + O(z**15)
-
-    # issue 6510
-    assert fresnels(z).series(z, S.Infinity) == \
-        (-1/(pi**2*z**3) + O(z**(-6), (z, oo)))*sin(pi*z**2/2) + \
-        (3/(pi**3*z**5) - 1/(pi*z) + O(z**(-6), (z, oo)))*cos(pi*z**2/2) + S.Half
-    assert fresnelc(z).series(z, S.Infinity) == \
-        (-1/(pi**2*z**3) + O(z**(-6), (z, oo)))*cos(pi*z**2/2) + \
-        (-3/(pi**3*z**5) + 1/(pi*z) + O(z**(-6), (z, oo)))*sin(pi*z**2/2) + S.Half
-    assert fresnels(1/z).series(z) == \
-        (-z**3/pi**2 + O(z**6))*sin(pi/(2*z**2)) + (-z/pi + 3*z**5/pi**3 + \
-        O(z**6))*cos(pi/(2*z**2)) + S.Half
-    assert fresnelc(1/z).series(z) == \
-        (-z**3/pi**2 + O(z**6))*cos(pi/(2*z**2)) + (z/pi - 3*z**5/pi**3 + \
-        O(z**6))*sin(pi/(2*z**2)) + S.Half
-
-    assert fresnelc(w).is_real is True
+    assert fresnelc(w).is_extended_real is True
 
     assert fresnelc(z).as_real_imag() == \
-        ((fresnelc(re(z) - I*re(z)*Abs(im(z))/Abs(re(z)))/2 +
-          fresnelc(re(z) + I*re(z)*Abs(im(z))/Abs(re(z)))/2,
-          I*(fresnelc(re(z) - I*re(z)*Abs(im(z))/Abs(re(z))) -
-          fresnelc(re(z) + I*re(z)*Abs(im(z))/Abs(re(z)))) *
-          re(z)*Abs(im(z))/(2*im(z)*Abs(re(z)))))
+        (fresnelc(re(z) - I*im(z))/2 + fresnelc(re(z) + I*im(z))/2,
+         -I*(-fresnelc(re(z) - I*im(z)) + fresnelc(re(z) + I*im(z)))/2)
+
+    assert fresnelc(z).as_real_imag(deep=False) == \
+        (fresnelc(re(z) - I*im(z))/2 + fresnelc(re(z) + I*im(z))/2,
+         -I*(-fresnelc(re(z) - I*im(z)) + fresnelc(re(z) + I*im(z)))/2)
 
     assert fresnelc(2 + 3*I).as_real_imag() == (
         fresnelc(2 - 3*I)/2 + fresnelc(2 + 3*I)/2,
-        I*(fresnelc(2 - 3*I) - fresnelc(2 + 3*I))/2
+         -I*(fresnelc(2 + 3*I) - fresnelc(2 - 3*I))/2
     )
 
     assert expand_func(integrate(fresnelc(z), z)) == \
         z*fresnelc(z) - sin(pi*z**2/2)/pi
 
-    assert fresnelc(z).rewrite(meijerg) == sqrt(2)*pi*z**(S(3)/4) * \
-        meijerg(((), (1,)), ((S(1)/4,),
-        (S(3)/4, 0)), -pi**2*z**4/16)/(2*(-z)**(S(1)/4)*(z**2)**(S(1)/4))
+    assert fresnelc(z).rewrite(meijerg) == sqrt(2)*pi*z**Rational(3, 4) * \
+        meijerg(((), (1,)), ((Rational(1, 4),),
+        (Rational(3, 4), 0)), -pi**2*z**4/16)/(2*(-z)**Rational(1, 4)*(z**2)**Rational(1, 4))
 
-    from sympy.utilities.randtest import verify_numerically
+    from sympy.testing.randtest import verify_numerically
 
     verify_numerically(re(fresnels(z)), fresnels(z).as_real_imag()[0], z)
     verify_numerically(im(fresnels(z)), fresnels(z).as_real_imag()[1], z)
@@ -675,3 +763,30 @@ def test_fresnel():
     verify_numerically(im(fresnelc(z)), fresnelc(z).as_real_imag()[1], z)
     verify_numerically(fresnelc(z), fresnelc(z).rewrite(hyper), z)
     verify_numerically(fresnelc(z), fresnelc(z).rewrite(meijerg), z)
+
+    raises(ArgumentIndexError, lambda: fresnels(z).fdiff(2))
+    raises(ArgumentIndexError, lambda: fresnelc(z).fdiff(2))
+
+    assert fresnels(x).taylor_term(-1, x) is S.Zero
+    assert fresnelc(x).taylor_term(-1, x) is S.Zero
+    assert fresnelc(x).taylor_term(1, x) == -pi**2*x**5/40
+
+
+@slow
+def test_fresnel_series():
+    assert fresnelc(z).series(z, n=15) == \
+        z - pi**2*z**5/40 + pi**4*z**9/3456 - pi**6*z**13/599040 + O(z**15)
+
+    # issues 6510, 10102
+    fs = (S.Half - sin(pi*z**2/2)/(pi**2*z**3)
+        + (-1/(pi*z) + 3/(pi**3*z**5))*cos(pi*z**2/2))
+    fc = (S.Half - cos(pi*z**2/2)/(pi**2*z**3)
+        + (1/(pi*z) - 3/(pi**3*z**5))*sin(pi*z**2/2))
+    assert fresnels(z).series(z, oo) == fs + O(z**(-6), (z, oo))
+    assert fresnelc(z).series(z, oo) == fc + O(z**(-6), (z, oo))
+    assert (fresnels(z).series(z, -oo) + fs.subs(z, -z)).expand().is_Order
+    assert (fresnelc(z).series(z, -oo) + fc.subs(z, -z)).expand().is_Order
+    assert (fresnels(1/z).series(z) - fs.subs(z, 1/z)).expand().is_Order
+    assert (fresnelc(1/z).series(z) - fc.subs(z, 1/z)).expand().is_Order
+    assert ((2*fresnels(3*z)).series(z, oo) - 2*fs.subs(z, 3*z)).expand().is_Order
+    assert ((3*fresnelc(2*z)).series(z, oo) - 3*fc.subs(z, 2*z)).expand().is_Order
