@@ -1,15 +1,15 @@
-from sympy import (KroneckerDelta, diff, Piecewise, Sum, Dummy, factor,
+from sympy import (KroneckerDelta, diff, Sum, Dummy, factor,
                    expand, zeros, gcd_terms, Eq, Symbol)
 
-from sympy.core import S, symbols, Add, Mul, SympifyError, Rational
-from sympy.core.expr import unchanged
-from sympy.functions import transpose, sin, cos, sqrt, cbrt, exp
+from sympy.core import (S, symbols, Add, Mul, SympifyError, Rational,
+                    Function)
+from sympy.functions import sin, cos, tan, sqrt, cbrt, exp
 from sympy.simplify import simplify
-from sympy.matrices import (Identity, ImmutableMatrix, Inverse, MatAdd, MatMul,
-        MatPow, Matrix, MatrixExpr, MatrixSymbol, ShapeError, ZeroMatrix,
-        SparseMatrix, Transpose, Adjoint, NonSquareMatrixError)
-from sympy.matrices.expressions.matexpr import (MatrixElement,
-                                                GenericZeroMatrix, GenericIdentity, OneMatrix)
+from sympy.matrices import (ImmutableMatrix, Inverse, MatAdd, MatMul,
+        MatPow, Matrix, MatrixExpr, MatrixSymbol, ShapeError,
+        SparseMatrix, Transpose, Adjoint, NonSquareMatrixError, MatrixSet)
+from sympy.matrices.expressions.matexpr import MatrixElement
+from sympy.matrices.expressions.special import ZeroMatrix, Identity
 from sympy.testing.pytest import raises, XFAIL
 
 
@@ -41,58 +41,6 @@ def test_matrix_symbol_creation():
     raises(ValueError, lambda: MatrixSymbol('A', n, n))
 
 
-def test_zero_matrix_creation():
-    assert unchanged(ZeroMatrix, 2, 2)
-    assert unchanged(ZeroMatrix, 0, 0)
-    raises(ValueError, lambda: ZeroMatrix(-1, 2))
-    raises(ValueError, lambda: ZeroMatrix(2.0, 2))
-    raises(ValueError, lambda: ZeroMatrix(2j, 2))
-    raises(ValueError, lambda: ZeroMatrix(2, -1))
-    raises(ValueError, lambda: ZeroMatrix(2, 2.0))
-    raises(ValueError, lambda: ZeroMatrix(2, 2j))
-
-    n = symbols('n')
-    assert unchanged(ZeroMatrix, n, n)
-    n = symbols('n', integer=False)
-    raises(ValueError, lambda: ZeroMatrix(n, n))
-    n = symbols('n', negative=True)
-    raises(ValueError, lambda: ZeroMatrix(n, n))
-
-
-def test_one_matrix_creation():
-    assert OneMatrix(2, 2)
-    assert OneMatrix(0, 0)
-    assert Eq(OneMatrix(1, 1), Identity(1))
-    raises(ValueError, lambda: OneMatrix(-1, 2))
-    raises(ValueError, lambda: OneMatrix(2.0, 2))
-    raises(ValueError, lambda: OneMatrix(2j, 2))
-    raises(ValueError, lambda: OneMatrix(2, -1))
-    raises(ValueError, lambda: OneMatrix(2, 2.0))
-    raises(ValueError, lambda: OneMatrix(2, 2j))
-
-    n = symbols('n')
-    assert OneMatrix(n, n)
-    n = symbols('n', integer=False)
-    raises(ValueError, lambda: OneMatrix(n, n))
-    n = symbols('n', negative=True)
-    raises(ValueError, lambda: OneMatrix(n, n))
-
-
-def test_identity_matrix_creation():
-    assert Identity(2)
-    assert Identity(0)
-    raises(ValueError, lambda: Identity(-1))
-    raises(ValueError, lambda: Identity(2.0))
-    raises(ValueError, lambda: Identity(2j))
-
-    n = symbols('n')
-    assert Identity(n)
-    n = symbols('n', integer=False)
-    raises(ValueError, lambda: Identity(n))
-    n = symbols('n', negative=True)
-    raises(ValueError, lambda: Identity(n))
-
-
 def test_shape():
     assert A.shape == (n, m)
     assert (A*B).shape == (n, l)
@@ -122,123 +70,6 @@ def test_subs():
     assert (C*D).subs({C: A, D: B}) == MatMul(A, B)
 
 
-def test_ZeroMatrix():
-    A = MatrixSymbol('A', n, m)
-    Z = ZeroMatrix(n, m)
-
-    assert A + Z == A
-    assert A*Z.T == ZeroMatrix(n, n)
-    assert Z*A.T == ZeroMatrix(n, n)
-    assert A - A == ZeroMatrix(*A.shape)
-
-    assert Z
-
-    assert transpose(Z) == ZeroMatrix(m, n)
-    assert Z.conjugate() == Z
-
-    assert ZeroMatrix(n, n)**0 == Identity(n)
-    with raises(NonSquareMatrixError):
-        Z**0
-    with raises(NonSquareMatrixError):
-        Z**1
-    with raises(NonSquareMatrixError):
-        Z**2
-
-    assert ZeroMatrix(3, 3).as_explicit() == ImmutableMatrix.zeros(3, 3)
-
-
-def test_ZeroMatrix_doit():
-    Znn = ZeroMatrix(Add(n, n, evaluate=False), n)
-    assert isinstance(Znn.rows, Add)
-    assert Znn.doit() == ZeroMatrix(2*n, n)
-    assert isinstance(Znn.doit().rows, Mul)
-
-
-def test_OneMatrix():
-    A = MatrixSymbol('A', n, m)
-    a = MatrixSymbol('a', n, 1)
-    U = OneMatrix(n, m)
-
-    assert U.shape == (n, m)
-    assert isinstance(A + U, Add)
-    assert transpose(U) == OneMatrix(m, n)
-    assert U.conjugate() == U
-
-    assert OneMatrix(n, n) ** 0 == Identity(n)
-    with raises(NonSquareMatrixError):
-        U ** 0
-    with raises(NonSquareMatrixError):
-        U ** 1
-    with raises(NonSquareMatrixError):
-        U ** 2
-    with raises(ShapeError):
-        a + U
-
-    U = OneMatrix(n, n)
-    assert U[1, 2] == 1
-
-    U = OneMatrix(2, 3)
-    assert U.as_explicit() == ImmutableMatrix.ones(2, 3)
-
-
-def test_OneMatrix_doit():
-    Unn = OneMatrix(Add(n, n, evaluate=False), n)
-    assert isinstance(Unn.rows, Add)
-    assert Unn.doit() == OneMatrix(2 * n, n)
-    assert isinstance(Unn.doit().rows, Mul)
-
-
-def test_OneMatrix_mul():
-    assert OneMatrix(n, m) * OneMatrix(m, k) == OneMatrix(n, k) * m
-    assert w * OneMatrix(1, 1) == w
-    assert OneMatrix(1, 1) * w.T == w.T
-
-
-def test_Identity():
-    A = MatrixSymbol('A', n, m)
-    i, j = symbols('i j')
-
-    In = Identity(n)
-    Im = Identity(m)
-
-    assert A*Im == A
-    assert In*A == A
-
-    assert transpose(In) == In
-    assert In.inverse() == In
-    assert In.conjugate() == In
-
-    assert In[i, j] != 0
-    assert Sum(In[i, j], (i, 0, n-1), (j, 0, n-1)).subs(n,3).doit() == 3
-    assert Sum(Sum(In[i, j], (i, 0, n-1)), (j, 0, n-1)).subs(n,3).doit() == 3
-
-    # If range exceeds the limit `(0, n-1)`, do not remove `Piecewise`:
-    expr = Sum(In[i, j], (i, 0, n-1))
-    assert expr.doit() == 1
-    expr = Sum(In[i, j], (i, 0, n-2))
-    assert expr.doit().dummy_eq(
-        Piecewise(
-            (1, (j >= 0) & (j <= n-2)),
-            (0, True)
-        )
-    )
-    expr = Sum(In[i, j], (i, 1, n-1))
-    assert expr.doit().dummy_eq(
-        Piecewise(
-            (1, (j >= 1) & (j <= n-1)),
-            (0, True)
-        )
-    )
-    assert Identity(3).as_explicit() == ImmutableMatrix.eye(3)
-
-
-def test_Identity_doit():
-    Inn = Identity(Add(n, n, evaluate=False))
-    assert isinstance(Inn.rows, Add)
-    assert Inn.doit() == Identity(2*n)
-    assert isinstance(Inn.doit().rows, Mul)
-
-
 def test_addition():
     A = MatrixSymbol('A', n, m)
     B = MatrixSymbol('B', n, m)
@@ -510,45 +341,17 @@ def test_issue_7842():
     assert Eq(A, B) == True
 
 
-def test_generic_zero_matrix():
-    z = GenericZeroMatrix()
-    A = MatrixSymbol("A", n, n)
+def test_issue_21195():
+    t = symbols('t')
+    x = Function('x')(t)
+    dx = x.diff(t)
+    exp1 = cos(x) + cos(x)*dx
+    exp2 = sin(x) + tan(x)*(dx.diff(t))
+    exp3 = sin(x)*sin(t)*(dx.diff(t)).diff(t)
+    A = Matrix([[exp1], [exp2], [exp3]])
+    B = Matrix([[exp1.diff(x)], [exp2.diff(x)], [exp3.diff(x)]])
+    assert A.diff(x) == B
 
-    assert z == z
-    assert z != A
-    assert A != z
-
-    assert z.is_ZeroMatrix
-
-    raises(TypeError, lambda: z.shape)
-    raises(TypeError, lambda: z.rows)
-    raises(TypeError, lambda: z.cols)
-
-    assert MatAdd() == z
-    assert MatAdd(z, A) == MatAdd(A)
-    # Make sure it is hashable
-    hash(z)
-
-
-def test_generic_identity():
-    I = GenericIdentity()
-    A = MatrixSymbol("A", n, n)
-
-    assert I == I
-    assert I != A
-    assert A != I
-
-    assert I.is_Identity
-    assert I**-1 == I
-
-    raises(TypeError, lambda: I.shape)
-    raises(TypeError, lambda: I.rows)
-    raises(TypeError, lambda: I.cols)
-
-    assert MatMul() == I
-    assert MatMul(I, A) == MatMul(A)
-    # Make sure it is hashable
-    hash(I)
 
 def test_MatMul_postprocessor():
     z = zeros(2)
@@ -635,8 +438,8 @@ def test_simplify_matrix_expressions():
     # Various simplification functions
     assert type(gcd_terms(C*D + D*C)) == MatAdd
     a = gcd_terms(2*C*D + 4*D*C)
-    assert type(a) == MatMul
-    assert a.args == (2, (C*D + 2*D*C))
+    assert type(a) == MatAdd
+    assert a.args == (2*C*D, 4*D*C)
 
 def test_exp():
     A = MatrixSymbol('A', 2, 2)
@@ -669,3 +472,36 @@ def test_as_explicit():
         [Z[1, 0], Z[1, 1], Z[1, 2]],
     ])
     raises(ValueError, lambda: A.as_explicit())
+
+def test_MatrixSet():
+    M = MatrixSet(2, 2, set=S.Reals)
+    assert M.shape == (2, 2)
+    assert M.set == S.Reals
+    X = Matrix([[1, 2], [3, 4]])
+    assert X in M
+    X = ZeroMatrix(2, 2)
+    assert X in M
+    raises(TypeError, lambda: A in M)
+    raises(TypeError, lambda: 1 in M)
+    M = MatrixSet(n, m, set=S.Reals)
+    assert A in M
+    raises(TypeError, lambda: C in M)
+    raises(TypeError, lambda: X in M)
+    M = MatrixSet(2, 2, set={1, 2, 3})
+    X = Matrix([[1, 2], [3, 4]])
+    Y = Matrix([[1, 2]])
+    assert (X in M) == S.false
+    assert (Y in M) == S.false
+    raises(ValueError, lambda: MatrixSet(2, -2, S.Reals))
+    raises(ValueError, lambda: MatrixSet(2.4, -1, S.Reals))
+    raises(TypeError, lambda: MatrixSet(2, 2, (1, 2, 3)))
+
+def test_matrixsymbol_solving():
+    A = MatrixSymbol('A', 2, 2)
+    B = MatrixSymbol('B', 2, 2)
+    Z = ZeroMatrix(2, 2)
+    assert -(-A + B) - A + B == Z
+    assert (-(-A + B) - A + B).simplify() == Z
+    assert (-(-A + B) - A + B).expand() == Z
+    assert (-(-A + B) - A + B - Z).simplify() == Z
+    assert (-(-A + B) - A + B - Z).expand() == Z
