@@ -2,19 +2,16 @@ from sympy import (
     Rational, Symbol, N, I, Abs, sqrt, exp, Float, sin,
     cos, symbols)
 from sympy.matrices import eye, Matrix
-from sympy.matrices.matrices import MatrixEigen
-from sympy.matrices.common import _MinimalMatrix, _CastableMatrix
 from sympy.core.singleton import S
 from sympy.testing.pytest import raises, XFAIL
 from sympy.matrices.matrices import NonSquareMatrixError, MatrixError
+from sympy.matrices.expressions.fourier import DFT
 from sympy.simplify.simplify import simplify
 from sympy.matrices.immutable import ImmutableMatrix
 from sympy.testing.pytest import slow
+from sympy.testing.matrices import allclose
 
 
-class EigenOnlyMatrix(_MinimalMatrix, _CastableMatrix, MatrixEigen):
-    pass
-
 def test_eigen():
     R = Rational
     M = Matrix.eye(3)
@@ -107,39 +104,52 @@ def test_eigen():
     assert max(i.q for i in M._eigenvects[0][2][0]) > 1
     M._eigenvects = M.eigenvects(simplify=True)
     assert max(i.q for i in M._eigenvects[0][2][0]) == 1
+
     M = Matrix([[Rational(1, 4), 1], [1, 1]])
-    assert M.eigenvects(simplify=True) == [
+    assert M.eigenvects() == [
         (Rational(5, 8) - sqrt(73)/8, 1, [Matrix([[-sqrt(73)/8 - Rational(3, 8)], [1]])]),
         (Rational(5, 8) + sqrt(73)/8, 1, [Matrix([[Rational(-3, 8) + sqrt(73)/8], [1]])])]
-    assert M.eigenvects(simplify=False) == [
-        (Rational(5, 8) - sqrt(73)/8, 1, [Matrix([[-1/(-Rational(3, 8) + sqrt(73)/8)], [1]])]),
-        (Rational(5, 8) + sqrt(73)/8, 1, [Matrix([[8/(3 + sqrt(73))], [1]])])]
 
     # issue 10719
     assert Matrix([]).eigenvals() == {}
+    assert Matrix([]).eigenvals(multiple=True) == []
     assert Matrix([]).eigenvects() == []
 
     # issue 15119
-    raises(NonSquareMatrixError, lambda : Matrix([[1, 2], [0, 4], [0, 0]]).eigenvals())
-    raises(NonSquareMatrixError, lambda : Matrix([[1, 0], [3, 4], [5, 6]]).eigenvals())
-    raises(NonSquareMatrixError, lambda : Matrix([[1, 2, 3], [0, 5, 6]]).eigenvals())
-    raises(NonSquareMatrixError, lambda : Matrix([[1, 0, 0], [4, 5, 0]]).eigenvals())
-    raises(NonSquareMatrixError, lambda : Matrix([[1, 2, 3], [0, 5, 6]]).eigenvals(error_when_incomplete = False))
-    raises(NonSquareMatrixError, lambda : Matrix([[1, 0, 0], [4, 5, 0]]).eigenvals(error_when_incomplete = False))
-
-    # issue 15125
-    from sympy.core.function import count_ops
-    q = Symbol("q", positive = True)
-    m = Matrix([[-2, exp(-q), 1], [exp(q), -2, 1], [1, 1, -2]])
-    assert count_ops(m.eigenvals(simplify=False)) > count_ops(m.eigenvals(simplify=True))
-    assert count_ops(m.eigenvals(simplify=lambda x: x)) > count_ops(m.eigenvals(simplify=True))
+    raises(NonSquareMatrixError,
+           lambda: Matrix([[1, 2], [0, 4], [0, 0]]).eigenvals())
+    raises(NonSquareMatrixError,
+           lambda: Matrix([[1, 0], [3, 4], [5, 6]]).eigenvals())
+    raises(NonSquareMatrixError,
+           lambda: Matrix([[1, 2, 3], [0, 5, 6]]).eigenvals())
+    raises(NonSquareMatrixError,
+           lambda: Matrix([[1, 0, 0], [4, 5, 0]]).eigenvals())
+    raises(NonSquareMatrixError,
+           lambda: Matrix([[1, 2, 3], [0, 5, 6]]).eigenvals(
+               error_when_incomplete = False))
+    raises(NonSquareMatrixError,
+           lambda: Matrix([[1, 0, 0], [4, 5, 0]]).eigenvals(
+               error_when_incomplete = False))
 
+    m = Matrix([[1, 2], [3, 4]])
     assert isinstance(m.eigenvals(simplify=True, multiple=False), dict)
     assert isinstance(m.eigenvals(simplify=True, multiple=True), list)
     assert isinstance(m.eigenvals(simplify=lambda x: x, multiple=False), dict)
     assert isinstance(m.eigenvals(simplify=lambda x: x, multiple=True), list)
 
 
+@slow
+def test_eigen_slow():
+    # issue 15125
+    from sympy.core.function import count_ops
+    q = Symbol("q", positive = True)
+    m = Matrix([[-2, exp(-q), 1], [exp(q), -2, 1], [1, 1, -2]])
+    assert count_ops(m.eigenvals(simplify=False)) > \
+        count_ops(m.eigenvals(simplify=True))
+    assert count_ops(m.eigenvals(simplify=lambda x: x)) > \
+        count_ops(m.eigenvals(simplify=True))
+
+
 def test_float_eigenvals():
     m = Matrix([[1, .6, .6], [.6, .9, .9], [.9, .6, .6]])
     evals = [
@@ -195,25 +205,27 @@ def test_issue_8240():
     assert eigenvals.count(x) == 2
     assert eigenvals.count(y) == 1
 
-# EigenOnlyMatrix tests
+
 def test_eigenvals():
-    M = EigenOnlyMatrix([[0, 1, 1],
+    M = Matrix([[0, 1, 1],
                 [1, 0, 0],
                 [1, 1, 1]])
     assert M.eigenvals() == {2*S.One: 1, -S.One: 1, S.Zero: 1}
 
-    # if we cannot factor the char poly, we raise an error
     m = Matrix([
         [3,  0,  0, 0, -3],
         [0, -3, -3, 0,  3],
         [0,  3,  0, 3,  0],
         [0,  0,  3, 0,  3],
         [3,  0,  0, 3,  0]])
-    raises(MatrixError, lambda: m.eigenvals())
+
+    # XXX Used dry-run test because arbitrary symbol that appears in
+    # CRootOf may not be unique.
+    assert m.eigenvals()
 
 
 def test_eigenvects():
-    M = EigenOnlyMatrix([[0, 1, 1],
+    M = Matrix([[0, 1, 1],
                 [1, 0, 0],
                 [1, 1, 1]])
     vecs = M.eigenvects()
@@ -223,7 +235,7 @@ def test_eigenvects():
 
 
 def test_left_eigenvects():
-    M = EigenOnlyMatrix([[0, 1, 1],
+    M = Matrix([[0, 1, 1],
                 [1, 0, 0],
                 [1, 1, 1]])
     vecs = M.left_eigenvects()
@@ -371,7 +383,7 @@ def test_bidiagonalize():
 
 
 def test_diagonalize():
-    m = EigenOnlyMatrix(2, 2, [0, -1, 1, 0])
+    m = Matrix(2, 2, [0, -1, 1, 0])
     raises(MatrixError, lambda: m.diagonalize(reals_only=True))
     P, D = m.diagonalize()
     assert D.is_diagonal()
@@ -380,7 +392,7 @@ def test_diagonalize():
                  [ 0, I]])
 
     # make sure we use floats out if floats are passed in
-    m = EigenOnlyMatrix(2, 2, [0, .5, .5, 0])
+    m = Matrix(2, 2, [0, .5, .5, 0])
     P, D = m.diagonalize()
     assert all(isinstance(e, Float) for e in D.values())
     assert all(isinstance(e, Float) for e in P.values())
@@ -388,15 +400,20 @@ def test_diagonalize():
     _, D2 = m.diagonalize(reals_only=True)
     assert D == D2
 
+    m = Matrix(
+        [[0, 1, 0, 0], [1, 0, 0, 0.002], [0.002, 0, 0, 1], [0, 0, 1, 0]])
+    P, D = m.diagonalize()
+    assert allclose(P*D, m*P)
+
 
 def test_is_diagonalizable():
     a, b, c = symbols('a b c')
-    m = EigenOnlyMatrix(2, 2, [a, c, c, b])
+    m = Matrix(2, 2, [a, c, c, b])
     assert m.is_symmetric()
     assert m.is_diagonalizable()
-    assert not EigenOnlyMatrix(2, 2, [1, 1, 0, 1]).is_diagonalizable()
+    assert not Matrix(2, 2, [1, 1, 0, 1]).is_diagonalizable()
 
-    m = EigenOnlyMatrix(2, 2, [0, -1, 1, 0])
+    m = Matrix(2, 2, [0, -1, 1, 0])
     assert m.is_diagonalizable()
     assert not m.is_diagonalizable(reals_only=True)
 
@@ -410,7 +427,7 @@ def test_jordan_form():
     # *NOT* be determined  from algebraic and geometric multiplicity alone
     # This can be seen most easily when one lets compute the J.c.f. of a matrix that
     # is in J.c.f already.
-    m = EigenOnlyMatrix(4, 4, [2, 1, 0, 0,
+    m = Matrix(4, 4, [2, 1, 0, 0,
                     0, 2, 1, 0,
                     0, 0, 2, 0,
                     0, 0, 0, 2
@@ -418,7 +435,7 @@ def test_jordan_form():
     P, J = m.jordan_form()
     assert m == J
 
-    m = EigenOnlyMatrix(4, 4, [2, 1, 0, 0,
+    m = Matrix(4, 4, [2, 1, 0, 0,
                     0, 2, 0, 0,
                     0, 0, 2, 1,
                     0, 0, 0, 2
@@ -433,12 +450,10 @@ def test_jordan_form():
     P, J = A.jordan_form()
     assert simplify(P*J*P.inv()) == A
 
-    assert EigenOnlyMatrix(1, 1, [1]).jordan_form() == (
-        Matrix([1]), Matrix([1]))
-    assert EigenOnlyMatrix(1, 1, [1]).jordan_form(
-        calc_transform=False) == Matrix([1])
+    assert Matrix(1, 1, [1]).jordan_form() == (Matrix([1]), Matrix([1]))
+    assert Matrix(1, 1, [1]).jordan_form(calc_transform=False) == Matrix([1])
 
-    # make sure if we cannot factor the characteristic polynomial, we raise an error
+    # If we have eigenvalues in CRootOf form, raise errors
     m = Matrix([[3, 0, 0, 0, -3], [0, -3, -3, 0, 3], [0, 3, 0, 3, 0], [0, 0, 3, 0, 3], [3, 0, 0, 3, 0]])
     raises(MatrixError, lambda: m.jordan_form())
 
@@ -454,7 +469,7 @@ def test_jordan_form():
 def test_singular_values():
     x = Symbol('x', real=True)
 
-    A = EigenOnlyMatrix([[0, 1*I], [2, 0]])
+    A = Matrix([[0, 1*I], [2, 0]])
     # if singular values can be sorted, they should be in decreasing order
     assert A.singular_values() == [2, 1]
 
@@ -465,11 +480,11 @@ def test_singular_values():
     # since Abs(x) cannot be sorted, test set equality
     assert set(vals) == {5, 1, Abs(x)}
 
-    A = EigenOnlyMatrix([[sin(x), cos(x)], [-cos(x), sin(x)]])
+    A = Matrix([[sin(x), cos(x)], [-cos(x), sin(x)]])
     vals = [sv.trigsimp() for sv in A.singular_values()]
     assert vals == [S.One, S.One]
 
-    A = EigenOnlyMatrix([
+    A = Matrix([
         [2, 4],
         [1, 3],
         [0, 0],
@@ -481,7 +496,7 @@ def test_singular_values():
         [sqrt(sqrt(221) + 15), sqrt(15 - sqrt(221)), 0, 0]
 
 def test___eq__():
-    assert (EigenOnlyMatrix(
+    assert (Matrix(
         [[0, 1, 1],
         [1, 0, 0],
         [1, 1, 1]]) == {}) is False
@@ -528,6 +543,7 @@ def test_definite():
     assert m.is_negative_semidefinite == False
     assert m.is_indefinite == False
 
+    # Hermetian matrices
     m = Matrix([[1, 2*I], [-I, 4]])
     assert m.is_positive_definite == True
     assert m.is_positive_semidefinite == True
@@ -572,3 +588,96 @@ def test_definite():
     assert m.is_positive_definite == True
     assert m.is_positive_semidefinite == True
     assert m.is_indefinite == False
+
+    # test for issue 19547: https://github.com/sympy/sympy/issues/19547
+    m = Matrix([
+        [0, 0, 0],
+        [0, 1, 2],
+        [0, 2, 1]
+    ])
+    assert not m.is_positive_definite
+    assert not m.is_positive_semidefinite
+
+
+def test_positive_semidefinite_cholesky():
+    from sympy.matrices.eigen import _is_positive_semidefinite_cholesky
+
+    m = Matrix([[0, 0, 0], [0, 0, 0], [0, 0, 0]])
+    assert _is_positive_semidefinite_cholesky(m) == True
+    m = Matrix([[0, 0, 0], [0, 5, -10*I], [0, 10*I, 5]])
+    assert _is_positive_semidefinite_cholesky(m) == False
+    m = Matrix([[1, 0, 0], [0, 0, 0], [0, 0, -1]])
+    assert _is_positive_semidefinite_cholesky(m) == False
+    m = Matrix([[0, 1], [1, 0]])
+    assert _is_positive_semidefinite_cholesky(m) == False
+
+    # https://www.value-at-risk.net/cholesky-factorization/
+    m = Matrix([[4, -2, -6], [-2, 10, 9], [-6, 9, 14]])
+    assert _is_positive_semidefinite_cholesky(m) == True
+    m = Matrix([[9, -3, 3], [-3, 2, 1], [3, 1, 6]])
+    assert _is_positive_semidefinite_cholesky(m) == True
+    m = Matrix([[4, -2, 2], [-2, 1, -1], [2, -1, 5]])
+    assert _is_positive_semidefinite_cholesky(m) == True
+    m = Matrix([[1, 2, -1], [2, 5, 1], [-1, 1, 9]])
+    assert _is_positive_semidefinite_cholesky(m) == False
+
+
+def test_issue_20582():
+    A = Matrix([
+        [5, -5, -3, 2, -7],
+        [-2, -5, 0, 2, 1],
+        [-2, -7, -5, -2, -6],
+        [7, 10, 3, 9, -2],
+        [4, -10, 3, -8, -4]
+    ])
+    # XXX Used dry-run test because arbitrary symbol that appears in
+    # CRootOf may not be unique.
+    assert A.eigenvects()
+
+
+def test_issue_20275():
+    # XXX We use complex expansions because complex exponentials are not
+    # recognized by polys.domains
+    A = DFT(3).as_explicit().expand(complex=True)
+    eigenvects = A.eigenvects()
+    assert eigenvects[0] == (
+        -1, 1,
+        [Matrix([[1 - sqrt(3)], [1], [1]])]
+    )
+    assert eigenvects[1] == (
+        1, 1,
+        [Matrix([[1 + sqrt(3)], [1], [1]])]
+    )
+    assert eigenvects[2] == (
+        -I, 1,
+        [Matrix([[0], [-1], [1]])]
+    )
+
+    A = DFT(4).as_explicit().expand(complex=True)
+    eigenvects = A.eigenvects()
+    assert eigenvects[0] == (
+        -1, 1,
+        [Matrix([[-1], [1], [1], [1]])]
+    )
+    assert eigenvects[1] == (
+        1, 2,
+        [Matrix([[1], [0], [1], [0]]), Matrix([[2], [1], [0], [1]])]
+    )
+    assert eigenvects[2] == (
+        -I, 1,
+        [Matrix([[0], [-1], [0], [1]])]
+    )
+
+    # XXX We skip test for some parts of eigenvectors which are very
+    # complicated and fragile under expression tree changes
+    A = DFT(5).as_explicit().expand(complex=True)
+    eigenvects = A.eigenvects()
+    assert eigenvects[0] == (
+        -1, 1,
+        [Matrix([[1 - sqrt(5)], [1], [1], [1], [1]])]
+    )
+    assert eigenvects[1] == (
+        1, 2,
+        [Matrix([[S(1)/2 + sqrt(5)/2], [0], [1], [1], [0]]),
+         Matrix([[S(1)/2 + sqrt(5)/2], [1], [0], [0], [1]])]
+    )
