@@ -2,7 +2,7 @@ import os
 
 from sympy import sin, cos
 from sympy.external import import_module
-from sympy.utilities.pytest import skip
+from sympy.testing.pytest import skip
 from sympy.parsing.autolev import parse_autolev
 
 antlr4 = import_module("antlr4")
