@@ -1,11 +1,8 @@
 """Quantum mechanical angular momemtum."""
 
-from __future__ import print_function, division
-
 from sympy import (Add, binomial, cos, exp, Expr, factorial, I, Integer, Mul,
                    pi, Rational, S, sin, simplify, sqrt, Sum, symbols, sympify,
                    Tuple, Dummy)
-from sympy.core.compatibility import unicode
 from sympy.matrices import zeros
 from sympy.printing.pretty.stringpict import prettyForm, stringPict
 from sympy.printing.pretty.pretty_symbology import pretty_symbol
@@ -66,7 +63,7 @@ def m_values(j):
 #-----------------------------------------------------------------------------
 
 
-class SpinOpBase(object):
+class SpinOpBase:
     """Base class for spin operators."""
 
     @classmethod
@@ -79,15 +76,15 @@ class SpinOpBase(object):
         return self.args[0]
 
     def _print_contents(self, printer, *args):
-        return '%s%s' % (unicode(self.name), self._coord)
+        return '%s%s' % (self.name, self._coord)
 
     def _print_contents_pretty(self, printer, *args):
-        a = stringPict(unicode(self.name))
+        a = stringPict(str(self.name))
         b = stringPict(self._coord)
         return self._print_subscript_pretty(a, b)
 
     def _print_contents_latex(self, printer, *args):
-        return r'%s_%s' % ((unicode(self.name), self._coord))
+        return r'%s_%s' % ((self.name, self._coord))
 
     def _represent_base(self, basis, **options):
         j = options.get('j', S.Half)
@@ -405,8 +402,8 @@ class J2Op(SpinOpBase, HermitianOperator):
         return self._represent_base(basis, **options)
 
     def _print_contents_pretty(self, printer, *args):
-        a = prettyForm(unicode(self.name))
-        b = prettyForm(u'2')
+        a = prettyForm(str(self.name))
+        b = prettyForm('2')
         return a**b
 
     def _print_contents_latex(self, printer, *args):
@@ -502,7 +499,7 @@ class Rotation(UnitaryOperator):
 
     def _print_operator_name_pretty(self, printer, *args):
         if printer._use_unicode:
-            return prettyForm(u'\N{SCRIPT CAPITAL R}' + u' ')
+            return prettyForm('\N{SCRIPT CAPITAL R}' + ' ')
         else:
             return prettyForm("R ")
 
@@ -623,7 +620,7 @@ class Rotation(UnitaryOperator):
     def _represent_JzOp(self, basis, **options):
         return self._represent_base(basis, **options)
 
-    def _apply_operator_uncoupled(self, state, ket, **options):
+    def _apply_operator_uncoupled(self, state, ket, *, dummy=True, **options):
         a = self.alpha
         b = self.beta
         g = self.gamma
@@ -639,7 +636,7 @@ class Rotation(UnitaryOperator):
                 s.append(z*state(j, mp))
             return Add(*s)
         else:
-            if options.pop('dummy', True):
+            if dummy:
                 mp = Dummy('mp')
             else:
                 mp = symbols('mp')
@@ -654,7 +651,7 @@ class Rotation(UnitaryOperator):
     def _apply_operator_JzKet(self, ket, **options):
         return self._apply_operator_uncoupled(JzKet, ket, **options)
 
-    def _apply_operator_coupled(self, state, ket, **options):
+    def _apply_operator_coupled(self, state, ket, *, dummy=True, **options):
         a = self.alpha
         b = self.beta
         g = self.gamma
@@ -672,7 +669,7 @@ class Rotation(UnitaryOperator):
                 s.append(z*state(j, mp, jn, coupling))
             return Add(*s)
         else:
-            if options.pop('dummy', True):
+            if dummy:
                 mp = Dummy('mp')
             else:
                 mp = symbols('mp')
@@ -859,6 +856,8 @@ class WignerD(Expr):
         alpha = sympify(self.alpha)
         beta = sympify(self.beta)
         gamma = sympify(self.gamma)
+        if alpha == 0 and beta == 0 and gamma == 0:
+            return KroneckerDelta(m, mp)
         if not j.is_number:
             raise ValueError(
                 'j parameter must be numerical to evaluate, got %s' % j)
@@ -952,7 +951,6 @@ class SpinState(State):
         gamma = sympify(options.get('gamma', 0))
         size, mvals = m_values(j)
         result = zeros(size, 1)
-        # TODO: Use KroneckerDelta if all Euler angles == 0
         # breaks finding angles on L930
         for p, mval in enumerate(mvals):
             if m.is_number:
@@ -1441,15 +1439,16 @@ class CoupledSpinState(SpinState):
         )
 
     def _print_label_latex(self, printer, *args):
-        label = [self.j, self.m]
+        label = [
+            printer._print(self.j, *args),
+            printer._print(self.m, *args)
+        ]
         for i, ji in enumerate(self.jn, start=1):
-            label.append('j_{%d}=%s' % (i, printer._print(ji)) )
+            label.append('j_{%d}=%s' % (i, printer._print(ji, *args)) )
         for jn, (n1, n2) in zip(self.coupled_jn[:-1], self.coupled_n[:-1]):
             n = ','.join(str(i) for i in sorted(n1 + n2))
-            label.append('j_{%s}=%s' % (n, printer._print(jn)) )
-        return self._print_sequence(
-            label, self._label_separator, printer, *args
-        )
+            label.append('j_{%s}=%s' % (n, printer._print(jn, *args)) )
+        return self._label_separator.join(label)
 
     @property
     def jn(self):
