@@ -4,8 +4,6 @@ Second quantization operators and states for bosons.
 This follow the formulation of Fetter and Welecka, "Quantum Theory
 of Many-Particle Systems."
 """
-from __future__ import print_function, division
-
 from collections import defaultdict
 
 from sympy import (Add, Basic, cacheit, Dummy, Expr, Function, I,
@@ -325,6 +323,9 @@ class SqOperator(Expr):
         """
         Returns the state index related to this operator.
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F, Fd, B, Bd
         >>> p = Symbol('p')
@@ -345,6 +346,9 @@ class SqOperator(Expr):
         """
         Returns True if the state is a symbol (as opposed to a number).
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F
         >>> p = Symbol('p')
@@ -487,11 +491,16 @@ class FermionicOperator(SqOperator):
         """
         Is this FermionicOperator restricted with respect to fermi level?
 
-        Return values:
+        Returns
+        =======
+
         1  : restricted to orbits above fermi
         0  : no restriction
         -1 : restricted to orbits below fermi
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F, Fd
         >>> a = Symbol('a', above_fermi=True)
@@ -524,6 +533,8 @@ class FermionicOperator(SqOperator):
         """
         Does the index of this FermionicOperator allow values above fermi?
 
+        Examples
+        ========
 
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F
@@ -538,6 +549,9 @@ class FermionicOperator(SqOperator):
         >>> F(p).is_above_fermi
         True
 
+        Note
+        ====
+
         The same applies to creation operators Fd
 
         """
@@ -548,6 +562,9 @@ class FermionicOperator(SqOperator):
         """
         Does the index of this FermionicOperator allow values below fermi?
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F
         >>> a = Symbol('a', above_fermi=True)
@@ -571,6 +588,9 @@ class FermionicOperator(SqOperator):
         """
         Is the index of this FermionicOperator restricted to values below fermi?
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F
         >>> a = Symbol('a', above_fermi=True)
@@ -593,6 +613,9 @@ class FermionicOperator(SqOperator):
         """
         Is the index of this FermionicOperator restricted to values above fermi?
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F
         >>> a = Symbol('a', above_fermi=True)
@@ -670,6 +693,9 @@ class AnnihilateFermion(FermionicOperator, Annihilator):
         Can we create a quasi-particle?  (create hole or create particle)
         If so, would that be above or below the fermi surface?
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F
         >>> a = Symbol('a', above_fermi=True)
@@ -694,6 +720,9 @@ class AnnihilateFermion(FermionicOperator, Annihilator):
         Can we destroy a quasi-particle?  (annihilate hole or annihilate particle)
         If so, would that be above or below the fermi surface?
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F
         >>> a = Symbol('a', above_fermi=1)
@@ -717,6 +746,8 @@ class AnnihilateFermion(FermionicOperator, Annihilator):
         """
         Always create a quasi-particle?  (create hole or create particle)
 
+        Examples
+        ========
 
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F
@@ -739,6 +770,9 @@ class AnnihilateFermion(FermionicOperator, Annihilator):
         """
         Always destroy a quasi-particle?  (annihilate hole or annihilate particle)
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import F
         >>> a = Symbol('a', above_fermi=True)
@@ -805,6 +839,9 @@ class CreateFermion(FermionicOperator, Creator):
         Can we create a quasi-particle?  (create hole or create particle)
         If so, would that be above or below the fermi surface?
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import Fd
         >>> a = Symbol('a', above_fermi=True)
@@ -829,6 +866,9 @@ class CreateFermion(FermionicOperator, Creator):
         Can we destroy a quasi-particle?  (annihilate hole or annihilate particle)
         If so, would that be above or below the fermi surface?
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import Fd
         >>> a = Symbol('a', above_fermi=1)
@@ -852,6 +892,9 @@ class CreateFermion(FermionicOperator, Creator):
         """
         Always create a quasi-particle?  (create hole or create particle)
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import Fd
         >>> a = Symbol('a', above_fermi=True)
@@ -873,6 +916,9 @@ class CreateFermion(FermionicOperator, Creator):
         """
         Always destroy a quasi-particle?  (annihilate hole or annihilate particle)
 
+        Examples
+        ========
+
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import Fd
         >>> a = Symbol('a', above_fermi=True)
@@ -1021,11 +1067,17 @@ class FermionState(FockState):
         """
         Performs the action of a creation operator.
 
+        Explanation
+        ===========
+
         If below fermi we try to remove a hole,
         if above fermi we try to create a particle.
 
-        if general index p we return Kronecker(p,i)*self
-        where i is a new symbol with restriction above or below.
+        If general index p we return ``Kronecker(p,i)*self``
+        where ``i`` is a new symbol with restriction above or below.
+
+        Examples
+        ========
 
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import FKet
@@ -1067,11 +1119,17 @@ class FermionState(FockState):
         """
         Performs the action of an annihilation operator.
 
-        If below fermi we try to create a hole,
-        if above fermi we try to remove a particle.
+        Explanation
+        ===========
 
-        if general index p we return Kronecker(p,i)*self
-        where i is a new symbol with restriction above or below.
+        If below fermi we try to create a hole,
+        If above fermi we try to remove a particle.
+
+        If general index p we return ``Kronecker(p,i)*self``
+        where ``i`` is a new symbol with restriction above or below.
+
+        Examples
+        ========
 
         >>> from sympy import Symbol
         >>> from sympy.physics.secondquant import FKet
@@ -1161,7 +1219,7 @@ class FermionState(FockState):
     @classmethod
     def _count_holes(cls, list):
         """
-        returns number of identified hole states in list.
+        Returns the number of identified hole states in list.
         """
         return len([i for i in list if cls._only_below_fermi(i)])
 
@@ -1236,6 +1294,9 @@ class FockStateFermionKet(FermionState, FockStateKet):
     """
     Many-particle Fock state with a sequence of occupied orbits.
 
+    Explanation
+    ===========
+
     Each state can only have one particle, so we choose to store a list of
     occupied orbits rather than a tuple with occupation numbers (zeros and ones).
 
@@ -1283,6 +1344,9 @@ def _apply_Mul(m):
     """
     Take a Mul instance with operators and apply them to states.
 
+    Explanation
+    ===========
+
     This method applies all operators with integer state labels
     to the actual states.  For symbolic state labels, nothing is done.
     When inner products of FockStates are encountered (like <a|b>),
@@ -1363,6 +1427,9 @@ class InnerProduct(Basic):
     """
     An unevaluated inner product between a bra and ket.
 
+    Explanation
+    ===========
+
     Currently this class just reduces things to a product of
     Kronecker Deltas.  In the future, we could introduce abstract
     states like ``|a>`` and ``|b>``, and leave the inner product unevaluated as
@@ -1431,14 +1498,14 @@ def matrix_rep(op, basis):
     return a
 
 
-class BosonicBasis(object):
+class BosonicBasis:
     """
     Base class for a basis set of bosonic Fock states.
     """
     pass
 
 
-class VarBosonicBasis(object):
+class VarBosonicBasis:
     """
     A single state, variable particle number basis set.
 
@@ -1593,6 +1660,9 @@ class Commutator(Function):
 
     The arguments are ordered according to .__cmp__()
 
+    Examples
+    ========
+
     >>> from sympy import symbols
     >>> from sympy.physics.secondquant import Commutator
     >>> A, B = symbols('A,B', commutative=False)
@@ -1738,11 +1808,17 @@ class NO(Expr):
 
     i.e.  {abcd}  sometimes written  :abcd:
 
+    Explanation
+    ===========
+
     Applying the function NO(arg) to an argument means that all operators in
     the argument will be assumed to anticommute, and have vanishing
     contractions.  This allows an immediate reordering to canonical form
     upon object creation.
 
+    Examples
+    ========
+
     >>> from sympy import symbols
     >>> from sympy.physics.secondquant import NO, F, Fd
     >>> p,q = symbols('p,q')
@@ -1752,7 +1828,9 @@ class NO(Expr):
     -NO(CreateFermion(p)*AnnihilateFermion(q))
 
 
-    Note:
+    Note
+    ====
+
     If you want to generate a normal ordered equivalent of an expression, you
     should use the function wicks().  This class only indicates that all
     operators inside the brackets anticommute, and have vanishing contractions.
@@ -1765,6 +1843,9 @@ class NO(Expr):
         """
         Use anticommutation to get canonical form of operators.
 
+        Explanation
+        ===========
+
         Employ associativity of normal ordered product: {ab{cd}} = {abcd}
         but note that {ab}{cd} /= {abcd}.
 
@@ -1950,6 +2031,9 @@ class NO(Expr):
         """
         Returns a sum of NO objects that contain no ambiguous q-operators.
 
+        Explanation
+        ===========
+
         If an index q has range both above and below fermi, the operator F(q)
         is ambiguous in the sense that it can be both a q-creator and a q-annihilator.
         If q is dummy, it is assumed to be a summation variable and this method
@@ -2139,6 +2223,9 @@ def _sqkey(sq_operator):
 def _sort_anticommuting_fermions(string1, key=_sqkey):
     """Sort fermionic operators to canonical order, assuming all pairs anticommute.
 
+    Explanation
+    ===========
+
     Uses a bidirectional bubble sort.  Items in string1 are not referenced
     so in principle they may be any comparable objects.   The sorting depends on the
     operators '>' and '=='.
@@ -2193,6 +2280,9 @@ def evaluate_deltas(e):
     """
     We evaluate KroneckerDelta symbols in the expression assuming Einstein summation.
 
+    Explanation
+    ===========
+
     If one index is repeated it is summed over and in effect substituted with
     the other one. If both indices are repeated we substitute according to what
     is the preferred index.  this is determined by
@@ -2301,6 +2391,9 @@ def substitute_dummies(expr, new_indices=False, pretty_indices={}):
     """
     Collect terms by substitution of dummy variables.
 
+    Explanation
+    ===========
+
     This routine allows simplification of Add expressions containing terms
     which differ only due to dummy variables.
 
@@ -2474,7 +2567,10 @@ def __kprint(expr):
 
 
 def _get_ordered_dummies(mul, verbose=False):
-    """Returns all dummies in the mul sorted in canonical order
+    """Returns all dummies in the mul sorted in canonical order.
+
+    Explanation
+    ===========
 
     The purpose of the canonical ordering is that dummies can be substituted
     consistently across terms with the result that equivalent terms can be
@@ -2537,8 +2633,8 @@ def _get_ordered_dummies(mul, verbose=False):
     """
     # setup dicts to avoid repeated calculations in key()
     args = Mul.make_args(mul)
-    fac_dum = dict([ (fac, fac.atoms(Dummy)) for fac in args] )
-    fac_repr = dict([ (fac, __kprint(fac)) for fac in args] )
+    fac_dum = { fac: fac.atoms(Dummy) for fac in args }
+    fac_repr = { fac: __kprint(fac) for fac in args }
     all_dums = set().union(*fac_dum.values())
     mask = {}
     for d in all_dums:
@@ -2674,7 +2770,7 @@ def _determine_ambiguous(term, ordered, ambiguous_groups):
     return ordered
 
 
-class _SymbolFactory(object):
+class _SymbolFactory:
     def __init__(self, label):
         self._counterVar = 0
         self._label = label
@@ -2767,12 +2863,11 @@ def wicks(e, **kw_args):
     """
     Returns the normal ordered equivalent of an expression using Wicks Theorem.
 
-
     Examples
     ========
 
-    >>> from sympy import symbols, Function, Dummy
-    >>> from sympy.physics.secondquant import wicks, F, Fd, NO
+    >>> from sympy import symbols, Dummy
+    >>> from sympy.physics.secondquant import wicks, F, Fd
     >>> p, q, r = symbols('p,q,r')
     >>> wicks(Fd(p)*F(q))
     KroneckerDelta(_i, q)*KroneckerDelta(p, q) + NO(CreateFermion(p)*AnnihilateFermion(q))
@@ -2892,6 +2987,9 @@ class PermutationOperator(Expr):
         """
         Returns -expr with permuted indices.
 
+        Explanation
+        ===========
+
         >>> from sympy import symbols, Function
         >>> from sympy.physics.secondquant import PermutationOperator
         >>> p,q = symbols('p,q')
@@ -2919,6 +3017,9 @@ def simplify_index_permutations(expr, permutation_operators):
     """
     Performs simplification by introducing PermutationOperators where appropriate.
 
+    Explanation
+    ===========
+
     Schematically:
         [abij] - [abji] - [baij] + [baji] ->  P(ab)*P(ij)*[abij]
 
@@ -2969,8 +3070,8 @@ def simplify_index_permutations(expr, permutation_operators):
         terms = set(expr.args)
 
         for P in permutation_operators:
-            new_terms = set([])
-            on_hold = set([])
+            new_terms = set()
+            on_hold = set()
             while terms:
                 term = terms.pop()
                 permuted = P.get_permuted(term)
