@@ -2,8 +2,6 @@
 Physical quantities.
 """
 
-from __future__ import division
-
 from sympy import AtomicExpr, Symbol, sympify
 from sympy.physics.units.dimensions import _QuantityMapper
 from sympy.physics.units.prefixes import Prefix
@@ -210,4 +208,4 @@ class Quantity(AtomicExpr):
     @property
     def free_symbols(self):
         """Return free symbols from quantity."""
-        return set([])
+        return set()
