@@ -1,10 +1,11 @@
 from sympy.core.backend import sympify, Add, ImmutableMatrix as Matrix
+from sympy.core.evalf import EvalfMixin, prec_to_dps
 from sympy.printing.defaults import Printable
 
 __all__ = ['Dyadic']
 
 
-class Dyadic(Printable):
+class Dyadic(Printable, EvalfMixin):
     """A Dyadic object.
 
     See:
@@ -17,6 +18,8 @@ class Dyadic(Printable):
 
     """
 
+    is_number = False
+
     def __init__(self, inlist):
         """
         Just like Vector's init, you shouldn't call this unless creating a
@@ -55,6 +58,11 @@ class Dyadic(Printable):
                 i -= 1
             i += 1
 
+    @property
+    def func(self):
+        """Returns the class Dyadic. """
+        return Dyadic
+
     def __add__(self, other):
         """The add operator for Dyadic. """
         other = _check_dyadic(other)
@@ -187,7 +195,7 @@ class Dyadic(Printable):
     def _pretty(self, printer):
         e = self
 
-        class Fake(object):
+        class Fake:
             baseline = 0
 
             def render(self, *args, **kwargs):
@@ -521,6 +529,61 @@ class Dyadic(Printable):
     dot = __and__
     cross = __xor__
 
+    def _eval_evalf(self, prec):
+        if not self.args:
+            return self
+        new_args = []
+        for inlist in self.args:
+            new_inlist = list(inlist)
+            new_inlist[0] = inlist[0].evalf(n=prec_to_dps(prec))
+            new_args.append(tuple(new_inlist))
+        return Dyadic(new_args)
+
+    def xreplace(self, rule):
+        """
+        Replace occurrences of objects within the measure numbers of the Dyadic.
+
+        Parameters
+        ==========
+
+        rule : dict-like
+            Expresses a replacement rule.
+
+        Returns
+        =======
+
+        Dyadic
+            Result of the replacement.
+
+        Examples
+        ========
+
+        >>> from sympy import symbols, pi
+        >>> from sympy.physics.vector import ReferenceFrame, outer
+        >>> N = ReferenceFrame('N')
+        >>> D = outer(N.x, N.x)
+        >>> x, y, z = symbols('x y z')
+        >>> ((1 + x*y) * D).xreplace({x: pi})
+        (pi*y + 1)*(N.x|N.x)
+        >>> ((1 + x*y) * D).xreplace({x: pi, y: 2})
+        (1 + 2*pi)*(N.x|N.x)
+
+        Replacements occur only if an entire node in the expression tree is
+        matched:
+
+        >>> ((x*y + z) * D).xreplace({x*y: pi})
+        (z + pi)*(N.x|N.x)
+        >>> ((x*y*z) * D).xreplace({x*y: pi})
+        x*y*z*(N.x|N.x)
+
+        """
+
+        new_args = []
+        for inlist in self.args:
+            new_inlist = list(inlist)
+            new_inlist[0] = new_inlist[0].xreplace(rule)
+            new_args.append(tuple(new_inlist))
+        return Dyadic(new_args)
 
 def _check_dyadic(other):
     if not isinstance(other, Dyadic):
