@@ -3,11 +3,12 @@ from sympy.core.backend import (S, sympify, expand, sqrt, Add, zeros,
 from sympy import trigsimp
 from sympy.printing.defaults import Printable
 from sympy.utilities.misc import filldedent
+from sympy.core.evalf import EvalfMixin, prec_to_dps
 
 __all__ = ['Vector']
 
 
-class Vector(Printable):
+class Vector(Printable, EvalfMixin):
     """The class used to define vectors.
 
     It along with ReferenceFrame are the building blocks of describing a
@@ -22,6 +23,7 @@ class Vector(Printable):
     """
 
     simp = False
+    is_number = False
 
     def __init__(self, inlist):
         """This is the constructor for the Vector class.  You shouldn't be
@@ -52,6 +54,11 @@ class Vector(Printable):
             if v != Matrix([0, 0, 0]):
                 self.args.append((v, k))
 
+    @property
+    def func(self):
+        """Returns the class Vector. """
+        return Vector
+
     def __hash__(self):
         return hash(tuple(self.args))
 
@@ -250,7 +257,7 @@ class Vector(Printable):
         from sympy.printing.pretty.stringpict import prettyForm
         e = self
 
-        class Fake(object):
+        class Fake:
 
             def render(self, *args, **kwargs):
                 ar = e.args  # just to shorten things
@@ -704,13 +711,64 @@ class Vector(Printable):
 
         return self.to_matrix(reference_frame).free_symbols
 
+    def _eval_evalf(self, prec):
+        if not self.args:
+            return self
+        new_args = []
+        for mat, frame in self.args:
+            new_args.append([mat.evalf(n=prec_to_dps(prec)), frame])
+        return Vector(new_args)
+
+    def xreplace(self, rule):
+        """
+        Replace occurrences of objects within the measure numbers of the vector.
+
+        Parameters
+        ==========
+
+        rule : dict-like
+            Expresses a replacement rule.
+
+        Returns
+        =======
+
+        Vector
+            Result of the replacement.
+
+        Examples
+        ========
+
+        >>> from sympy import symbols, pi
+        >>> from sympy.physics.vector import ReferenceFrame
+        >>> A = ReferenceFrame('A')
+        >>> x, y, z = symbols('x y z')
+        >>> ((1 + x*y) * A.x).xreplace({x: pi})
+        (pi*y + 1)*A.x
+        >>> ((1 + x*y) * A.x).xreplace({x: pi, y: 2})
+        (1 + 2*pi)*A.x
+
+        Replacements occur only if an entire node in the expression tree is
+        matched:
+
+        >>> ((x*y + z) * A.x).xreplace({x*y: pi})
+        (z + pi)*A.x
+        >>> ((x*y*z) * A.x).xreplace({x*y: pi})
+        x*y*z*A.x
+
+        """
+
+        new_args = []
+        for mat, frame in self.args:
+            mat = mat.xreplace(rule)
+            new_args.append([mat, frame])
+        return Vector(new_args)
 
 class VectorTypeError(TypeError):
 
     def __init__(self, other, want):
         msg = filldedent("Expected an instance of %s, but received object "
                          "'%s' of %s." % (type(want), other, type(other)))
-        super(VectorTypeError, self).__init__(msg)
+        super().__init__(msg)
 
 
 def _check_vector(other):
