@@ -22,14 +22,13 @@ if you care at all about performance. A new backend instance is initialized
 every time you call ``show()`` and the old one is left to the garbage collector.
 """
 
-from __future__ import print_function, division
 
 import warnings
+from collections.abc import Callable
 
 from sympy import sympify, Expr, Tuple, Dummy, Symbol
 from sympy.external import import_module
 from sympy.core.function import arity
-from sympy.core.compatibility import Callable
 from sympy.utilities.iterables import is_sequence
 from .experimental_lambdify import (vectorized_lambdify, lambdify)
 
@@ -57,9 +56,12 @@ def unset_show():
 ##############################################################################
 
 
-class Plot(object):
+class Plot:
     """The central class of the plotting module.
 
+    Explanation
+    ===========
+
     For interactive work the function ``plot`` is better suited.
 
     This class permits the plotting of sympy expressions using numerous
@@ -117,7 +119,8 @@ class Plot(object):
     - aspect_ratio : tuple of two floats or {'auto'}
     - autoscale : bool
     - margin : float in [0, 1]
-    - backend : {'default', 'matplotlib', 'text'}
+    - backend : {'default', 'matplotlib', 'text'} or a subclass of BaseBackend
+    - size : optional tuple of two floats, (width, height); default: None
 
     The per data series options and aesthetics are:
     There are none in the base series. See below for options for subclasses.
@@ -149,8 +152,8 @@ class Plot(object):
         xlim=None, ylim=None, axis_center='auto', axis=True,
         xscale='linear', yscale='linear', legend=False, autoscale=True,
         margin=0, annotations=None, markers=None, rectangles=None,
-        fill=None, backend='default', **kwargs):
-        super(Plot, self).__init__()
+        fill=None, backend='default', size=None, **kwargs):
+        super().__init__()
 
         # Options for the graph as a whole.
         # The possible values for each option are described in the docstring of
@@ -179,31 +182,36 @@ class Plot(object):
         # The backend type. On every show() a new backend instance is created
         # in self._backend which is tightly coupled to the Plot instance
         # (thanks to the parent attribute of the backend).
-        self.backend = plot_backends[backend]
+        if isinstance(backend, str):
+            self.backend = plot_backends[backend]
+        elif (type(backend) == type) and issubclass(backend, BaseBackend):
+            self.backend = backend
+        else:
+            raise TypeError(
+                "backend must be either a string or a subclass of BaseBackend")
 
         is_real = \
             lambda lim: all(getattr(i, 'is_real', True) for i in lim)
         is_finite = \
             lambda lim: all(getattr(i, 'is_finite', True) for i in lim)
 
+        # reduce code repetition
+        def check_and_set(t_name, t):
+            if t:
+                if not is_real(t):
+                    raise ValueError(
+                    "All numbers from {}={} must be real".format(t_name, t))
+                if not is_finite(t):
+                    raise ValueError(
+                    "All numbers from {}={} must be finite".format(t_name, t))
+                setattr(self, t_name, (float(t[0]), float(t[1])))
+
         self.xlim = None
+        check_and_set("xlim", xlim)
         self.ylim = None
-        if xlim:
-            if not is_real(xlim):
-                raise ValueError(
-                "All numbers from xlim={} must be real".format(xlim))
-            if not is_finite(xlim):
-                raise ValueError(
-                "All numbers from xlim={} must be finite".format(xlim))
-            self.xlim = (float(xlim[0]), float(xlim[1]))
-        if ylim:
-            if not is_real(ylim):
-                raise ValueError(
-                "All numbers from ylim={} must be real".format(ylim))
-            if not is_finite(ylim):
-                raise ValueError(
-                "All numbers from ylim={} must be finite".format(ylim))
-            self.ylim = (float(ylim[0]), float(ylim[1]))
+        check_and_set("ylim", ylim)
+        self.size = None
+        check_and_set("size", size)
 
 
     def show(self):
@@ -306,7 +314,7 @@ class Plot(object):
             raise TypeError('Expecting Plot or sequence of BaseSeries')
 
 
-class PlotGrid(object):
+class PlotGrid:
     """This class helps to plot subplots from already created sympy plots
     in a single figure.
 
@@ -382,17 +390,19 @@ class PlotGrid(object):
         [0]: cartesian surface: x*y for x over (-5.0, 5.0) and y over (-5.0, 5.0)
 
     """
-    def __init__(self, nrows, ncolumns, *args, **kwargs):
+    def __init__(self, nrows, ncolumns, *args, show=True, size=None, **kwargs):
         """
         Parameters
         ==========
 
-        nrows : The number of rows that should be in the grid of the
-                required subplot
-        ncolumns : The number of columns that should be in the grid
-                   of the required subplot
+        nrows :
+            The number of rows that should be in the grid of the
+            required subplot.
+        ncolumns :
+            The number of columns that should be in the grid
+            of the required subplot.
 
-        nrows and ncolumns together define the required grid
+        nrows and ncolumns together define the required grid.
 
         Arguments
         =========
@@ -405,11 +415,15 @@ class PlotGrid(object):
         =================
 
         show : Boolean
-               The default value is set to ``True``. Set show to ``False`` and
-               the function will not display the subplot. The returned instance
-               of the ``PlotGrid`` class can then be used to save or display the
-               plot by calling the ``save()`` and ``show()`` methods
-               respectively.
+            The default value is set to ``True``. Set show to ``False`` and
+            the function will not display the subplot. The returned instance
+            of the ``PlotGrid`` class can then be used to save or display the
+            plot by calling the ``save()`` and ``show()`` methods
+            respectively.
+        size : (float, float), optional
+            A tuple in the form (width, height) in inches to specify the size of
+            the overall figure. The default value is set to ``None``, meaning
+            the size will be set by the default backend.
         """
         self.nrows = nrows
         self.ncolumns = ncolumns
@@ -418,7 +432,7 @@ class PlotGrid(object):
         for arg in args:
             self._series.append(arg._series)
         self.backend = DefaultBackend
-        show = kwargs.pop('show', True)
+        self.size = size
         if show:
             self.show()
 
@@ -447,9 +461,12 @@ class PlotGrid(object):
 #TODO more general way to calculate aesthetics (see get_color_array)
 
 ### The base class for all series
-class BaseSeries(object):
+class BaseSeries:
     """Base class for the data objects containing stuff to be plotted.
 
+    Explanation
+    ===========
+
     The backend should check if it supports the data series that it's given.
     (eg TextBackend supports only LineOver1DRange).
     It's the backend responsibility to know how to use the class of
@@ -504,7 +521,7 @@ class BaseSeries(object):
     # used for calculation aesthetics
 
     def __init__(self):
-        super(BaseSeries, self).__init__()
+        super().__init__()
 
     @property
     def is_3D(self):
@@ -537,7 +554,7 @@ class Line2DBaseSeries(BaseSeries):
     _dim = 2
 
     def __init__(self):
-        super(Line2DBaseSeries, self).__init__()
+        super().__init__()
         self.label = None
         self.steps = False
         self.only_integers = False
@@ -579,7 +596,7 @@ class List2DSeries(Line2DBaseSeries):
 
     def __init__(self, list_x, list_y):
         np = import_module('numpy')
-        super(List2DSeries, self).__init__()
+        super().__init__()
         self.list_x = np.array(list_x)
         self.list_y = np.array(list_y)
         self.label = 'list'
@@ -595,7 +612,7 @@ class LineOver1DRangeSeries(Line2DBaseSeries):
     """Representation for a line consisting of a SymPy expression over a range."""
 
     def __init__(self, expr, var_start_end, **kwargs):
-        super(LineOver1DRangeSeries, self).__init__()
+        super().__init__()
         self.expr = sympify(expr)
         self.label = kwargs.get('label', None) or str(self.expr)
         self.var = sympify(var_start_end[0])
@@ -615,6 +632,9 @@ class LineOver1DRangeSeries(Line2DBaseSeries):
         """
         Adaptively gets segments for plotting.
 
+        Explanation
+        ===========
+
         The adaptive sampling is done by recursively checking if three
         points are almost collinear. If they are not collinear, then more
         points are added between those points.
@@ -628,7 +648,7 @@ class LineOver1DRangeSeries(Line2DBaseSeries):
         """
 
         if self.only_integers or not self.adaptive:
-            return super(LineOver1DRangeSeries, self).get_segments()
+            return super().get_segments()
         else:
             f = lambdify([self.var], self.expr)
             list_segments = []
@@ -715,7 +735,7 @@ class Parametric2DLineSeries(Line2DBaseSeries):
     is_parametric = True
 
     def __init__(self, expr_x, expr_y, var_start_end, **kwargs):
-        super(Parametric2DLineSeries, self).__init__()
+        super().__init__()
         self.expr_x = sympify(expr_x)
         self.expr_y = sympify(expr_y)
         self.label = kwargs.get('label', None) or \
@@ -749,18 +769,22 @@ class Parametric2DLineSeries(Line2DBaseSeries):
         """
         Adaptively gets segments for plotting.
 
+        Explanation
+        ===========
+
         The adaptive sampling is done by recursively checking if three
         points are almost collinear. If they are not collinear, then more
         points are added between those points.
 
         References
         ==========
-        [1] Adaptive polygonal approximation of parametric curves,
+
+        .. [1] Adaptive polygonal approximation of parametric curves,
             Luiz Henrique de Figueiredo.
 
         """
         if not self.adaptive:
-            return super(Parametric2DLineSeries, self).get_segments()
+            return super().get_segments()
 
         f_x = lambdify([self.var], self.expr_x)
         f_y = lambdify([self.var], self.expr_y)
@@ -839,7 +863,7 @@ class Line3DBaseSeries(Line2DBaseSeries):
     _dim = 3
 
     def __init__(self):
-        super(Line3DBaseSeries, self).__init__()
+        super().__init__()
 
 
 class Parametric3DLineSeries(Line3DBaseSeries):
@@ -847,7 +871,7 @@ class Parametric3DLineSeries(Line3DBaseSeries):
     expressions and a range."""
 
     def __init__(self, expr_x, expr_y, expr_z, var_start_end, **kwargs):
-        super(Parametric3DLineSeries, self).__init__()
+        super().__init__()
         self.expr_x = sympify(expr_x)
         self.expr_y = sympify(expr_y)
         self.expr_z = sympify(expr_z)
@@ -900,7 +924,7 @@ class SurfaceBaseSeries(BaseSeries):
     is_3Dsurface = True
 
     def __init__(self):
-        super(SurfaceBaseSeries, self).__init__()
+        super().__init__()
         self.surface_color = None
 
     def get_color_array(self):
@@ -930,7 +954,7 @@ class SurfaceOver2DRangeSeries(SurfaceBaseSeries):
     """Representation for a 3D surface consisting of a sympy expression and 2D
     range."""
     def __init__(self, expr, var_start_end_x, var_start_end_y, **kwargs):
-        super(SurfaceOver2DRangeSeries, self).__init__()
+        super().__init__()
         self.expr = sympify(expr)
         self.var_x = sympify(var_start_end_x[0])
         self.start_x = float(var_start_end_x[1])
@@ -977,7 +1001,7 @@ class ParametricSurfaceSeries(SurfaceBaseSeries):
     def __init__(
         self, expr_x, expr_y, expr_z, var_start_end_u, var_start_end_v,
             **kwargs):
-        super(ParametricSurfaceSeries, self).__init__()
+        super().__init__()
         self.expr_x = sympify(expr_x)
         self.expr_y = sympify(expr_y)
         self.expr_z = sympify(expr_z)
@@ -1045,7 +1069,7 @@ class ContourSeries(BaseSeries):
     is_contour = True
 
     def __init__(self, expr, var_start_end_x, var_start_end_y):
-        super(ContourSeries, self).__init__()
+        super().__init__()
         self.nb_of_points_x = 50
         self.nb_of_points_y = 50
         self.expr = sympify(expr)
@@ -1084,17 +1108,83 @@ class ContourSeries(BaseSeries):
 # Backends
 ##############################################################################
 
-class BaseBackend(object):
+class BaseBackend:
+    """Base class for all backends. A backend represents the plotting library,
+    which implements the necessary functionalities in order to use SymPy
+    plotting functions.
+
+    How the plotting module works:
+
+    1. Whenever a plotting function is called, the provided expressions are
+        processed and a list of instances of the `BaseSeries` class is created,
+        containing the necessary information to plot the expressions (eg the
+        expression, ranges, series name, ...). Eventually, these objects will
+        generate the numerical data to be plotted.
+    2. A Plot object is instantiated, which stores the list of series and the
+        main attributes of the plot (eg axis labels, title, ...).
+    3. When the "show" command is executed, a new backend is instantiated,
+        which loops through each series object to generate and plot the
+        numerical data. The backend is also going to set the axis labels, title,
+        ..., according to the values stored in the Plot instance.
+
+    The backend should check if it supports the data series that it's given
+    (eg TextBackend supports only LineOver1DRange).
+
+    It's the backend responsibility to know how to use the class of data series
+    that it's given. Note that the current implementation of the `*Series`
+    classes is "matplotlib-centric": the numerical data returned by the
+    `get_points` and `get_meshes` methods is meant to be used directly by
+    Matplotlib. Therefore, the new backend will have to pre-process the
+    numerical data to make it compatible with the chosen plotting library.
+    Keep in mind that future SymPy versions may improve the `*Series` classes in
+    order to return numerical data "non-matplotlib-centric", hence if you code
+    a new backend you have the responsibility to check if its working on each
+    SymPy release.
+
+    Please, explore the `MatplotlibBackend` source code to understand how a
+    backend should be coded.
+
+    Methods
+    =======
+
+    In order to be used by SymPy plotting functions, a backend must implement
+    the following methods:
+
+    * `show(self)`: used to loop over the data series, generate the numerical
+        data, plot it and set the axis labels, title, ...
+    * save(self, path): used to save the current plot to the specified file
+        path.
+    * close(self): used to close the current plot backend (note: some plotting
+        library doesn't support this functionality. In that case, just raise a
+        warning).
+
+    See also
+    ========
+
+    MatplotlibBackend
+    """
     def __init__(self, parent):
-        super(BaseBackend, self).__init__()
+        super().__init__()
         self.parent = parent
 
+    def show(self):
+        raise NotImplementedError
+
+    def save(self, path):
+        raise NotImplementedError
+
+    def close(self):
+        raise NotImplementedError
+
 
 # Don't have to check for the success of importing matplotlib in each case;
 # we will only be using this backend if we can successfully import matploblib
 class MatplotlibBackend(BaseBackend):
+    """ This class implements the functionalities to use Matplotlib with SymPy
+    plotting functions.
+    """
     def __init__(self, parent):
-        super(MatplotlibBackend, self).__init__(parent)
+        super().__init__(parent)
         self.matplotlib = import_module('matplotlib',
             import_kwargs={'fromlist': ['pyplot', 'cm', 'collections']},
             min_module_version='1.1.0', catch=(RuntimeError,))
@@ -1113,7 +1203,7 @@ class MatplotlibBackend(BaseBackend):
             series_list = self.parent._series
 
         self.ax = []
-        self.fig = self.plt.figure()
+        self.fig = self.plt.figure(figsize=parent.size)
 
         for i, series in enumerate(series_list):
             are_3D = [s.is_3D for s in series]
@@ -1343,7 +1433,7 @@ class MatplotlibBackend(BaseBackend):
 
 class TextBackend(BaseBackend):
     def __init__(self, parent):
-        super(TextBackend, self).__init__(parent)
+        super().__init__(parent)
 
     def show(self):
         if not _show:
@@ -1403,8 +1493,8 @@ def flat(x, y, z, eps=1e-3):
     #   workaround for `lambdify` in `.experimental_lambdify` fails
     #   to return numerical values in some cases. Lower-level fix
     #   in `lambdify` is possible.
-    vector_a = (x - y).astype(np.float)
-    vector_b = (z - y).astype(np.float)
+    vector_a = (x - y).astype(np.float64)
+    vector_b = (z - y).astype(np.float64)
     dot_product = np.dot(vector_a, vector_b)
     vector_a_norm = np.linalg.norm(vector_a)
     vector_b_norm = np.linalg.norm(vector_b)
@@ -1439,13 +1529,13 @@ def _matplotlib_list(interval_list):
 # TODO: Add more plotting options for 3d plots.
 # TODO: Adaptive sampling for 3D plots.
 
-def plot(*args, **kwargs):
+def plot(*args, show=True, **kwargs):
     """Plots a function of a single variable as a curve.
 
     Parameters
     ==========
 
-    args
+    args :
         The first argument is the expression representing the function
         of single variable to be plotted.
 
@@ -1508,10 +1598,10 @@ def plot(*args, **kwargs):
         {'center', 'auto'}
 
     xlim : (float, float), optional
-        Denotes the x-axis limits.
+        Denotes the x-axis limits, ``(min, max)```.
 
     ylim : (float, float), optional
-        Denotes the y-axis limits.
+        Denotes the y-axis limits, ``(min, max)```.
 
     annotations : list, optional
         A list of dictionaries specifying the type of annotation
@@ -1559,6 +1649,11 @@ def plot(*args, **kwargs):
         If the ``adaptive`` flag is set to ``True``, this will be
         ignored.
 
+    size : (float, float), optional
+        A tuple in the form (width, height) in inches to specify the size of
+        the overall figure. The default value is set to ``None``, meaning
+        the size will be set by the default backend.
+
     Examples
     ========
 
@@ -1636,7 +1731,6 @@ def plot(*args, **kwargs):
     x = free.pop() if free else Symbol('x')
     kwargs.setdefault('xlabel', x.name)
     kwargs.setdefault('ylabel', 'f(%s)' % x.name)
-    show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 1, 1)
     series = [LineOver1DRangeSeries(*arg, **kwargs) for arg in plot_expr]
@@ -1647,93 +1741,96 @@ def plot(*args, **kwargs):
     return plots
 
 
-def plot_parametric(*args, **kwargs):
+def plot_parametric(*args, show=True, **kwargs):
     """
-    Plots a 2D parametric plot.
+    Plots a 2D parametric curve.
 
-    The plotting uses an adaptive algorithm which samples recursively to
-    accurately plot the plot. The adaptive algorithm uses a random point near
-    the midpoint of two points that has to be further sampled. Hence the same
-    plots can appear slightly different.
+    Parameters
+    ==========
 
-    Usage
-    =====
+    args
+        Common specifications are:
 
-    Single plot.
+        - Plotting a single parametric curve with a range
+            ``plot_parametric((expr_x, expr_y), range)``
+        - Plotting multiple parametric curves with the same range
+            ``plot_parametric((expr_x, expr_y), ..., range)``
+        - Plotting multiple parametric curves with different ranges
+            ``plot_parametric((expr_x, expr_y, range), ...)``
 
-    ``plot_parametric(expr_x, expr_y, range, **kwargs)``
+        ``expr_x`` is the expression representing $x$ component of the
+        parametric function.
 
-    If the range is not specified, then a default range of (-10, 10) is used.
+        ``expr_y`` is the expression representing $y$ component of the
+        parametric function.
 
-    Multiple plots with same range.
+        ``range`` is a 3-tuple denoting the parameter symbol, start and
+        stop. For example, ``(u, 0, 5)``.
 
-    ``plot_parametric((expr1_x, expr1_y), (expr2_x, expr2_y), range, **kwargs)``
+        If the range is not specified, then a default range of (-10, 10)
+        is used.
 
-    If the range is not specified, then a default range of (-10, 10) is used.
+        However, if the arguments are specified as
+        ``(expr_x, expr_y, range), ...``, you must specify the ranges
+        for each expressions manually.
 
-    Multiple plots with different ranges.
+        Default range may change in the future if a more advanced
+        algorithm is implemented.
 
-    ``plot_parametric((expr_x, expr_y, range), ..., **kwargs)``
+    adaptive : bool, optional
+        Specifies whether to use the adaptive sampling or not.
 
-    Range has to be specified for every expression.
+        The default value is set to ``True``. Set adaptive to ``False``
+        and specify ``nb_of_points`` if uniform sampling is required.
 
-    Default range may change in the future if a more advanced default range
-    detection algorithm is implemented.
+    depth :  int, optional
+        The recursion depth of the adaptive algorithm. A depth of
+        value $n$ samples a maximum of $2^n$ points.
 
-    Arguments
-    =========
+    nb_of_points : int, optional
+        Used when the ``adaptive`` flag is set to ``False``.
 
-    ``expr_x`` : Expression representing the function along x.
+        Specifies the number of the points used for the uniform
+        sampling.
 
-    ``expr_y`` : Expression representing the function along y.
+    line_color : function
+        A function which returns a float.
 
-    ``range``: (u, 0, 5), A 3-tuple denoting the range of the parameter
-    variable.
+        Specifies the color of the plot.
 
-    Keyword Arguments
-    =================
+        See :class:`Plot` for more details.
 
-    Arguments for ``Parametric2DLineSeries`` class:
+    label : str, optional
+        The label of the expression in the plot. It will be used when
+        called with ``legend``. Default is the name of the expression.
+        e.g. ``sin(x)``
 
-    ``adaptive``: Boolean. The default value is set to True. Set adaptive to
-    False and specify ``nb_of_points`` if uniform sampling is required.
+    xlabel : str, optional
+        Label for the x-axis.
 
-    ``depth``: int Recursion depth of the adaptive algorithm. A depth of
-    value ``n`` samples a maximum of `2^{n}` points.
+    ylabel : str, optional
+        Label for the y-axis.
 
-    ``nb_of_points``: int. Used when the ``adaptive`` is set to False. The
-    function is uniformly sampled at ``nb_of_points`` number of points.
+    xscale : 'linear' or 'log', optional
+        Sets the scaling of the x-axis.
 
-    Aesthetics
-    ----------
+    yscale : 'linear' or 'log', optional
+        Sets the scaling of the y-axis.
 
-    ``line_color``: function which returns a float. Specifies the color for the
-    plot. See ``sympy.plotting.Plot`` for more details.
+    axis_center : (float, float), optional
+        Tuple of two floats denoting the coordinates of the center or
+        {'center', 'auto'}
 
-    ``label``: str
-        The label to the plot. It will be used when called with ``legend=True``
-        to denote the function with the given label in the plot.
+    xlim : (float, float), optional
+        Denotes the x-axis limits, ``(min, max)```.
 
-    If there are multiple plots, then the same Series arguments are applied to
-    all the plots. If you want to set these options separately, you can index
-    the returned ``Plot`` object and set it.
+    ylim : (float, float), optional
+        Denotes the y-axis limits, ``(min, max)```.
 
-    Arguments for ``Plot`` class:
-
-    ``xlabel`` : str. Label for the x-axis.
-
-    ``ylabel`` : str. Label for the y-axis.
-
-    ``xscale``: {'linear', 'log'} Sets the scaling of the x-axis.
-
-    ``yscale``: {'linear', 'log'} Sets the scaling if the y-axis.
-
-    ``axis_center``: tuple of two floats denoting the coordinates of the center
-    or {'center', 'auto'}
-
-    ``xlim`` : tuple of two floats, denoting the x-axis limits.
-
-    ``ylim`` : tuple of two floats, denoting the y-axis limits.
+    size : (float, float), optional
+        A tuple in the form (width, height) in inches to specify the size of
+        the overall figure. The default value is set to ``None``, meaning
+        the size will be set by the default backend.
 
     Examples
     ========
@@ -1747,31 +1844,31 @@ def plot_parametric(*args, **kwargs):
        >>> from sympy.plotting import plot_parametric
        >>> u = symbols('u')
 
-    Single Parametric plot
+    A parametric plot with a single expression:
 
     .. plot::
        :context: close-figs
        :format: doctest
        :include-source: True
 
-       >>> plot_parametric(cos(u), sin(u), (u, -5, 5))
+       >>> plot_parametric((cos(u), sin(u)), (u, -5, 5))
        Plot object containing:
        [0]: parametric cartesian line: (cos(u), sin(u)) for u over (-5.0, 5.0)
 
-
-    Multiple parametric plot with single range.
+    A parametric plot with multiple expressions with the same range:
 
     .. plot::
        :context: close-figs
        :format: doctest
        :include-source: True
 
-       >>> plot_parametric((cos(u), sin(u)), (u, cos(u)))
+       >>> plot_parametric((cos(u), sin(u)), (u, cos(u)), (u, -10, 10))
        Plot object containing:
        [0]: parametric cartesian line: (cos(u), sin(u)) for u over (-10.0, 10.0)
        [1]: parametric cartesian line: (u, cos(u)) for u over (-10.0, 10.0)
 
-    Multiple parametric plots.
+    A parametric plot with multiple expressions with different ranges
+    for each curve:
 
     .. plot::
        :context: close-figs
@@ -1784,14 +1881,50 @@ def plot_parametric(*args, **kwargs):
        [0]: parametric cartesian line: (cos(u), sin(u)) for u over (-5.0, 5.0)
        [1]: parametric cartesian line: (cos(u), u) for u over (-5.0, 5.0)
 
+    Notes
+    =====
+
+    The plotting uses an adaptive algorithm which samples recursively to
+    accurately plot the curve. The adaptive algorithm uses a random point
+    near the midpoint of two points that has to be further sampled.
+    Hence, repeating the same plot command can give slightly different
+    results because of the random sampling.
+
+    If there are multiple plots, then the same optional arguments are
+    applied to all the plots drawn in the same canvas. If you want to
+    set these options separately, you can index the returned ``Plot``
+    object and set it.
+
+    For example, when you specify ``line_color`` once, it would be
+    applied simultaneously to both series.
+
+    .. plot::
+       :context: close-figs
+       :format: doctest
+       :include-source: True
+
+        >>> from sympy import pi
+        >>> expr1 = (u, cos(2*pi*u)/2 + 1/2)
+        >>> expr2 = (u, sin(2*pi*u)/2 + 1/2)
+        >>> p = plot_parametric(expr1, expr2, (u, 0, 1), line_color='blue')
+
+    If you want to specify the line color for the specific series, you
+    should index each item and apply the property manually.
+
+    .. plot::
+       :context: close-figs
+       :format: doctest
+       :include-source: True
+
+        >>> p[0].line_color = 'red'
+        >>> p.show()
 
     See Also
     ========
-    Plot, Parametric2DLineSeries
 
+    Plot, Parametric2DLineSeries
     """
     args = list(map(sympify, args))
-    show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 2, 1)
     series = [Parametric2DLineSeries(*arg, **kwargs) for arg in plot_expr]
@@ -1801,7 +1934,7 @@ def plot_parametric(*args, **kwargs):
     return plots
 
 
-def plot3d_parametric_line(*args, **kwargs):
+def plot3d_parametric_line(*args, show=True, **kwargs):
     """
     Plots a 3D parametric line plot.
 
@@ -1860,6 +1993,11 @@ def plot3d_parametric_line(*args, **kwargs):
 
     ``title`` : str. Title of the plot.
 
+    ``size`` : (float, float), optional
+        A tuple in the form (width, height) in inches to specify the size of
+        the overall figure. The default value is set to ``None``, meaning
+        the size will be set by the default backend.
+
     Examples
     ========
 
@@ -1905,7 +2043,6 @@ def plot3d_parametric_line(*args, **kwargs):
 
     """
     args = list(map(sympify, args))
-    show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 3, 1)
     series = [Parametric3DLineSeries(*arg, **kwargs) for arg in plot_expr]
@@ -1915,7 +2052,7 @@ def plot3d_parametric_line(*args, **kwargs):
     return plots
 
 
-def plot3d(*args, **kwargs):
+def plot3d(*args, show=True, **kwargs):
     """
     Plots a 3D surface plot.
 
@@ -1977,6 +2114,10 @@ def plot3d(*args, **kwargs):
     Arguments for ``Plot`` class:
 
     ``title`` : str. Title of the plot.
+    ``size`` : (float, float), optional
+    A tuple in the form (width, height) in inches to specify the size of the
+    overall figure. The default value is set to ``None``, meaning the size will
+    be set by the default backend.
 
     Examples
     ========
@@ -2037,7 +2178,6 @@ def plot3d(*args, **kwargs):
     """
 
     args = list(map(sympify, args))
-    show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 1, 2)
     series = [SurfaceOver2DRangeSeries(*arg, **kwargs) for arg in plot_expr]
@@ -2047,12 +2187,12 @@ def plot3d(*args, **kwargs):
     return plots
 
 
-def plot3d_parametric_surface(*args, **kwargs):
+def plot3d_parametric_surface(*args, show=True, **kwargs):
     """
     Plots a 3D parametric surface plot.
 
-    Usage
-    =====
+    Explanation
+    ===========
 
     Single plot.
 
@@ -2108,6 +2248,10 @@ def plot3d_parametric_surface(*args, **kwargs):
     Arguments for ``Plot`` class:
 
     ``title`` : str. Title of the plot.
+    ``size`` : (float, float), optional
+    A tuple in the form (width, height) in inches to specify the size of the
+    overall figure. The default value is set to ``None``, meaning the size will
+    be set by the default backend.
 
     Examples
     ========
@@ -2142,7 +2286,6 @@ def plot3d_parametric_surface(*args, **kwargs):
     """
 
     args = list(map(sympify, args))
-    show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 3, 2)
     series = [ParametricSurfaceSeries(*arg, **kwargs) for arg in plot_expr]
@@ -2151,7 +2294,7 @@ def plot3d_parametric_surface(*args, **kwargs):
         plots.show()
     return plots
 
-def plot_contour(*args, **kwargs):
+def plot_contour(*args, show=True, **kwargs):
     """
     Draws contour plot of a function
 
@@ -2213,6 +2356,10 @@ def plot_contour(*args, **kwargs):
     Arguments for ``Plot`` class:
 
     ``title`` : str. Title of the plot.
+    ``size`` : (float, float), optional
+        A tuple in the form (width, height) in inches to specify the size of
+        the overall figure. The default value is set to ``None``, meaning
+        the size will be set by the default backend.
 
     See Also
     ========
@@ -2222,7 +2369,6 @@ def plot_contour(*args, **kwargs):
     """
 
     args = list(map(sympify, args))
-    show = kwargs.pop('show', True)
     plot_expr = check_arguments(args, 1, 2)
     series = [ContourSeries(*arg) for arg in plot_expr]
     plot_contours = Plot(*series, **kwargs)
@@ -2235,7 +2381,7 @@ def plot_contour(*args, **kwargs):
 def check_arguments(args, expr_len, nb_of_free_symbols):
     """
     Checks the arguments and converts into tuples of the
-    form (exprs, ranges)
+    form (exprs, ranges).
 
     Examples
     ========
@@ -2245,7 +2391,7 @@ def check_arguments(args, expr_len, nb_of_free_symbols):
        :format: doctest
        :include-source: True
 
-       >>> from sympy import plot, cos, sin, symbols
+       >>> from sympy import cos, sin, symbols
        >>> from sympy.plotting.plot import check_arguments
        >>> x = symbols('x')
        >>> check_arguments([cos(x), sin(x)], 2, 1)
