@@ -1,6 +1,5 @@
 """Basic tools for dense recursive polynomials in ``K[x]`` or ``K[X]``. """
 
-from __future__ import print_function, division
 
 from sympy import oo
 from sympy.core import igcd
@@ -316,11 +315,11 @@ def _rec_validate(f, g, i, K):
         if K is not None and not K.of_type(g):
             raise TypeError("%s in %s in not of type %s" % (g, f, K.dtype))
 
-        return set([i - 1])
+        return {i - 1}
     elif not g:
-        return set([i])
+        return {i}
     else:
-        levels = set([])
+        levels = set()
 
         for c in g:
             levels |= _rec_validate(f, c, i + 1, K)
