@@ -1,6 +1,5 @@
 """Advanced tools for dense recursive polynomials in ``K[x]`` or ``K[X]``. """
 
-from __future__ import print_function, division
 
 from sympy.polys.densearith import (
     dup_add_term, dmp_add_term,
@@ -1112,6 +1111,11 @@ def dmp_lift(f, u, K):
     x**8 + 2*x**6 + 9*x**4 - 8*x**2 + 16
 
     """
+    if K.is_GaussianField:
+        K1 = K.as_AlgebraicField()
+        f = dmp_convert(f, u, K, K1)
+        K = K1
+
     if not K.is_Algebraic:
         raise DomainError(
             'computation can be done only in an algebraic domain')
