@@ -1,11 +1,12 @@
 """Implementation of RootOf class and related tools. """
 
-from __future__ import print_function, division
 
+from sympy import Basic
 from sympy.core import (S, Expr, Integer, Float, I, oo, Add, Lambda,
     symbols, sympify, Rational, Dummy)
 from sympy.core.cache import cacheit
 from sympy.core.compatibility import ordered
+from sympy.core.relational import is_le
 from sympy.polys.domains import QQ
 from sympy.polys.polyerrors import (
     MultivariatePolynomialError,
@@ -25,7 +26,7 @@ from sympy.utilities import lambdify, public, sift, numbered_symbols
 
 from mpmath import mpf, mpc, findroot, workprec
 from mpmath.libmp.libmpf import dps_to_prec, prec_to_dps
-
+from sympy.multipledispatch import dispatch
 from itertools import chain
 
 
@@ -33,7 +34,7 @@ __all__ = ['CRootOf']
 
 
 
-class _pure_key_dict(object):
+class _pure_key_dict:
     """A minimal dictionary that makes sure that the key is a
     univariate PurePoly instance.
 
@@ -43,7 +44,7 @@ class _pure_key_dict(object):
     Only the following actions are guaranteed:
 
     >>> from sympy.polys.rootoftools import _pure_key_dict
-    >>> from sympy import S, PurePoly
+    >>> from sympy import PurePoly
     >>> from sympy.abc import x, y
 
     1) creation
@@ -560,7 +561,7 @@ class ComplexRootOf(RootOf):
         # with the indexing method but assert that the desired state
         # is not broken
         C, F = 0, 1  # location of ComplexInterval and factor
-        fs = set([i[F] for i in complexes])
+        fs = {i[F] for i in complexes}
         for i in range(1, len(complexes)):
             if complexes[i][F] != complexes[i - 1][F]:
                 # if this fails the factors of a root were not
@@ -907,7 +908,7 @@ class ComplexRootOf(RootOf):
         ensure the decimal representation of the approximation will be
         correct (including rounding) to 6 digits:
 
-        >>> from sympy import S, legendre_poly, Symbol
+        >>> from sympy import legendre_poly, Symbol
         >>> x = Symbol("x")
         >>> p = legendre_poly(4, x, polys=True)
         >>> r = p.real_roots()[-1]
@@ -970,42 +971,48 @@ class ComplexRootOf(RootOf):
         self._set_interval(interval)
         return real + I*imag
 
-    def _eval_Eq(self, other):
-        # CRootOf represents a Root, so if other is that root, it should set
-        # the expression to zero *and* it should be in the interval of the
-        # CRootOf instance. It must also be a number that agrees with the
-        # is_real value of the CRootOf instance.
-        if type(self) == type(other):
-            return sympify(self == other)
-        if not other.is_number:
-            return None
-        if not other.is_finite:
-            return S.false
-        z = self.expr.subs(self.expr.free_symbols.pop(), other).is_zero
-        if z is False:  # all roots will make z True but we don't know
-                        # whether this is the right root if z is True
-            return S.false
-        o = other.is_real, other.is_imaginary
-        s = self.is_real, self.is_imaginary
-        assert None not in s  # this is part of initial refinement
-        if o != s and None not in o:
-            return S.false
-        re, im = other.as_real_imag()
-        if self.is_real:
-            if im:
-                return S.false
-            i = self._get_interval()
-            a, b = [Rational(str(_)) for _ in (i.a, i.b)]
-            return sympify(a <= other and other <= b)
-        i = self._get_interval()
-        r1, r2, i1, i2 = [Rational(str(j)) for j in (
-            i.ax, i.bx, i.ay, i.by)]
-        return sympify((
-            r1 <= re and re <= r2) and (
-            i1 <= im and im <= i2))
 
 CRootOf = ComplexRootOf
 
+
+@dispatch(ComplexRootOf, ComplexRootOf)
+def _eval_is_eq(lhs, rhs): # noqa:F811
+    # if we use is_eq to check here, we get infinite recurion
+    return lhs == rhs
+
+
+@dispatch(ComplexRootOf, Basic)  # type:ignore
+def _eval_is_eq(lhs, rhs): # noqa:F811
+    # CRootOf represents a Root, so if rhs is that root, it should set
+    # the expression to zero *and* it should be in the interval of the
+    # CRootOf instance. It must also be a number that agrees with the
+    # is_real value of the CRootOf instance.
+    if not rhs.is_number:
+        return None
+    if not rhs.is_finite:
+        return False
+    z = lhs.expr.subs(lhs.expr.free_symbols.pop(), rhs).is_zero
+    if z is False:  # all roots will make z True but we don't know
+        # whether this is the right root if z is True
+        return False
+    o = rhs.is_real, rhs.is_imaginary
+    s = lhs.is_real, lhs.is_imaginary
+    assert None not in s  # this is part of initial refinement
+    if o != s and None not in o:
+        return False
+    re, im = rhs.as_real_imag()
+    if lhs.is_real:
+        if im:
+            return False
+        i = lhs._get_interval()
+        a, b = [Rational(str(_)) for _ in (i.a, i.b)]
+        return sympify(a <= rhs and rhs <= b)
+    i = lhs._get_interval()
+    r1, r2, i1, i2 = [Rational(str(j)) for j in (
+        i.ax, i.bx, i.ay, i.by)]
+    return is_le(r1, re) and is_le(re,r2) and is_le(i1,im) and is_le(im,i2)
+
+
 @public
 class RootSum(Expr):
     """Represents a sum of all roots of a univariate polynomial. """
