@@ -1,4 +1,3 @@
-
 from sympy.polys.galoistools import (
     gf_crt, gf_crt1, gf_crt2, gf_int,
     gf_degree, gf_strip, gf_trunc, gf_normal,
@@ -35,7 +34,7 @@ from sympy.polys import polyconfig as config
 
 from sympy.polys.domains import ZZ
 from sympy import pi, nextprime
-from sympy.utilities.pytest import raises
+from sympy.testing.pytest import raises
 
 
 def test_gf_crt():
@@ -521,7 +520,6 @@ def test_gf_frobenius_map():
     f = ZZ.map([2, 0, 1, 0, 2, 2, 0, 2, 2, 2])
     g = ZZ.map([1,1,0,2,0,1,0,2,0,1])
     p = 3
-    n = 4
     b = gf_frobenius_monomial_base(g, p, ZZ)
     h = gf_frobenius_map(f, g, b, p, ZZ)
     h1 = gf_pow_mod(f, p, g, p, ZZ)
