@@ -1,5 +1,6 @@
 """Test sparse polynomials. """
 
+from functools import reduce
 from operator import add, mul
 
 from sympy.polys.rings import ring, xring, sring, PolyRing, PolyElement
@@ -11,7 +12,7 @@ from sympy.polys.polyerrors import GeneratorsError, \
 
 from sympy.testing.pytest import raises
 from sympy.core import Symbol, symbols
-from sympy.core.compatibility import reduce
+
 from sympy import sqrt, pi, oo
 
 def test_PolyRing___init__():
@@ -171,7 +172,7 @@ def test_sring():
 
     r = sqrt(2) - sqrt(3)
     R, a = sring(r, extension=True)
-    assert R.domain == QQ.algebraic_field(r)
+    assert R.domain == QQ.algebraic_field(sqrt(2) + sqrt(3))
     assert R.gens == ()
     assert a == R.domain.from_sympy(r)
 
@@ -207,6 +208,9 @@ def test_PolyElement___eq__():
     assert ((x*y - x*y) != 1) == True
     assert (1 != (x*y - x*y)) == True
 
+    assert R.one == QQ(1, 1) == R.one
+    assert R.one == 1 == R.one
+
     Rt, t = ring("t", ZZ)
     R, x, y = ring("x,y", Rt)
 
@@ -570,7 +574,7 @@ def test_PolyElement___mul__():
 
     assert dict(EX(pi)*x*y*z) == dict(x*y*z*EX(pi)) == {(1, 1, 1): EX(pi)}
 
-def test_PolyElement___div__():
+def test_PolyElement___truediv__():
     R, x,y,z = ring("x,y,z", ZZ)
 
     assert (2*x**2 - 4)/2 == x**2 - 2
