@@ -12,13 +12,10 @@ TODO:
       top/center/bottom alignment options for left/right
 """
 
-from __future__ import print_function, division
+from .pretty_symbology import hobj, vobj, xsym, xobj, pretty_use_unicode, line_width
+from sympy.utilities.exceptions import SymPyDeprecationWarning
 
-from .pretty_symbology import hobj, vobj, xsym, xobj, pretty_use_unicode
-from sympy.core.compatibility import string_types, range
-
-
-class stringPict(object):
+class stringPict:
     """An ASCII picture.
     The pictures are represented as a list of equal length strings.
     """
@@ -42,7 +39,7 @@ class stringPict(object):
         if not lines:
             return ['']
 
-        width = max(len(line) for line in lines)
+        width = max(line_width(line) for line in lines)
         return [line.center(width) for line in lines]
 
     def height(self):
@@ -51,7 +48,7 @@ class stringPict(object):
 
     def width(self):
         """The width of the picture in characters."""
-        return len(self.picture[0])
+        return line_width(self.picture[0])
 
     @staticmethod
     def next(*args):
@@ -61,7 +58,7 @@ class stringPict(object):
         #convert everything to stringPicts
         objects = []
         for arg in args:
-            if isinstance(arg, string_types):
+            if isinstance(arg, str):
                 arg = stringPict(arg)
             objects.append(arg)
 
@@ -122,7 +119,7 @@ class stringPict(object):
         #convert everything to stringPicts; keep LINE
         objects = []
         for arg in args:
-            if arg is not stringPict.LINE and isinstance(arg, string_types):
+            if arg is not stringPict.LINE and isinstance(arg, str):
                 arg = stringPict(arg)
             objects.append(arg)
 
@@ -340,13 +337,10 @@ class stringPict(object):
         return False
 
     def __hash__(self):
-        return super(stringPict, self).__hash__()
+        return super().__hash__()
 
     def __str__(self):
-        return str.join('\n', self.picture)
-
-    def __unicode__(self):
-        return unicode.join(u'\n', self.picture)
+        return '\n'.join(self.picture)
 
     def __repr__(self):
         return "stringPict(%r,%d)" % ('\n'.join(self.picture), self.baseline)
@@ -382,7 +376,20 @@ class prettyForm(stringPict):
         """Initialize from stringPict and binding power."""
         stringPict.__init__(self, s, baseline)
         self.binding = binding
-        self.unicode = unicode or s
+        if unicode is not None:
+            SymPyDeprecationWarning(
+                feature="``unicode`` argument to ``prettyForm``",
+                useinstead="the ``s`` argument",
+                deprecated_since_version="1.7").warn()
+        self._unicode = unicode or s
+
+    @property
+    def unicode(self):
+        SymPyDeprecationWarning(
+            feature="``prettyForm.unicode`` attribute",
+            useinstead="``stringPrict.s`` attribute",
+            deprecated_since_version="1.7").warn()
+        return self._unicode
 
     # Note: code to handle subtraction is in _print_Add
 
@@ -404,7 +411,7 @@ class prettyForm(stringPict):
             result.append(arg)
         return prettyForm(binding=prettyForm.ADD, *stringPict.next(*result))
 
-    def __div__(self, den, slashed=False):
+    def __truediv__(self, den, slashed=False):
         """Make a pretty division; stacked or slashed.
         """
         if slashed:
@@ -423,15 +430,12 @@ class prettyForm(stringPict):
             stringPict.LINE,
             den))
 
-    def __truediv__(self, o):
-        return self.__div__(o)
-
     def __mul__(self, *others):
         """Make a pretty multiplication.
         Parentheses are needed around +, - and neg.
         """
         quantity = {
-            'degree': u"\N{DEGREE SIGN}"
+            'degree': "\N{DEGREE SIGN}"
         }
 
         if len(others) == 0:
