@@ -1,7 +1,5 @@
 """Limits of sequences"""
 
-from __future__ import print_function, division
-
 from sympy.core.add import Add
 from sympy.core.function import PoleError
 from sympy.core.power import Pow
@@ -9,6 +7,8 @@ from sympy.core.singleton import S
 from sympy.core.symbol import Dummy
 from sympy.core.sympify import sympify
 from sympy.functions.combinatorial.numbers import fibonacci
+from sympy.functions.combinatorial.factorials import factorial, subfactorial
+from sympy.functions.special.gamma_functions import gamma
 from sympy.functions.elementary.complexes import Abs
 from sympy.functions.elementary.miscellaneous import Max, Min
 from sympy.functions.elementary.trigonometric import cos, sin
@@ -18,6 +18,9 @@ from sympy.series.limits import Limit
 def difference_delta(expr, n=None, step=1):
     """Difference Operator.
 
+    Explanation
+    ===========
+
     Discrete analog of differential operator. Given a sequence x[n],
     returns the sequence x[n + step] - x[n].
 
@@ -64,6 +67,9 @@ def dominant(expr, n):
     """Finds the dominant term in a sum, that is a term that dominates
     every other term.
 
+    Explanation
+    ===========
+
     If limit(a/b, n, oo) is oo then a dominates b.
     If limit(a/b, n, oo) is 0 then b dominates a.
     Otherwise, a and b are comparable.
@@ -149,13 +155,13 @@ def _limit_seq(expr, n, trials):
 
 
 def limit_seq(expr, n=None, trials=5):
-    """Finds the limit of a sequence as index n tends to infinity.
+    """Finds the limit of a sequence as index ``n`` tends to infinity.
 
     Parameters
     ==========
 
     expr : Expr
-        SymPy expression for the n-th term of the sequence
+        SymPy expression for the ``n-th`` term of the sequence
     n : Symbol, optional
         The index of the sequence, an integer that tends to positive
         infinity. If None, inferred from the expression unless it has
@@ -211,6 +217,7 @@ def limit_seq(expr, n=None, trials=5):
         return expr
 
     expr = expr.rewrite(fibonacci, S.GoldenRatio)
+    expr = expr.rewrite(factorial, subfactorial, gamma)
     n_ = Dummy("n", integer=True, positive=True)
     n1 = Dummy("n", odd=True, positive=True)
     n2 = Dummy("n", even=True, positive=True)
