@@ -1,11 +1,9 @@
 """Tools for manipulation of expressions using paths. """
 
-from __future__ import print_function, division
-
 from sympy.core import Basic
 
 
-class EPath(object):
+class EPath:
     r"""
     Manipulate expressions using paths.
 
@@ -284,6 +282,9 @@ def epath(path, expr=None, func=None, args=None, kwargs=None):
     r"""
     Manipulate parts of an expression selected by a path.
 
+    Explanation
+    ===========
+
     This function allows to manipulate large nested expressions in single
     line of code, utilizing techniques to those applied in XML processing
     standards (e.g. XPath).
