@@ -2,13 +2,13 @@
 # This is the module for ODE solver classes for single ODEs.
 #
 
-#import typing
-#
-#if typing.TYPE_CHECKING:
-#    from typing import ClassVar
-#from typing import Dict, Type
+import typing
 
-from typing import Iterable, List, Optional
+if typing.TYPE_CHECKING:
+    from typing import ClassVar
+from typing import Dict, Type
+
+from typing import Iterator, List, Optional
 
 from sympy.core import S
 from sympy.core.exprtools import factor_terms
@@ -17,10 +17,11 @@ from sympy.core.function import AppliedUndef, Derivative, Function, expand
 from sympy.core.numbers import Float
 from sympy.core.relational import Equality, Eq
 from sympy.core.symbol import Symbol, Dummy, Wild
+from sympy.core.mul import Mul
 from sympy.functions import exp, sqrt, tan, log
 from sympy.integrals import Integral
-from sympy.polys.polytools import cancel, factor, factor_list
-from sympy.simplify import simplify
+from sympy.polys.polytools import cancel, factor
+from sympy.simplify.simplify import simplify
 from sympy.simplify.radsimp import fraction
 from sympy.utilities import numbered_symbols
 
@@ -58,7 +59,7 @@ class SingleODEProblem:
     This class is used internally by dsolve. To instantiate an instance
     directly first define an ODE problem:
 
-    >>> from sympy import Eq, Function, Symbol
+    >>> from sympy import Function, Symbol
     >>> x = Symbol('x')
     >>> f = Function('f')
     >>> eq = f(x).diff(x, 2)
@@ -123,7 +124,7 @@ class SingleODEProblem:
         Cs = [next(ncs) for i in range(num)]
         return Cs
 
-    def iter_numbered_constants(self, start=1, prefix='C') -> Iterable[Symbol]:
+    def iter_numbered_constants(self, start=1, prefix='C') -> Iterator[Symbol]:
         """
         Returns an iterator of constants that do not occur
         in eq already.
@@ -134,6 +135,13 @@ class SingleODEProblem:
             atom_set |= {Symbol(str(f.func)) for f in func_set}
         return numbered_symbols(start=start, prefix=prefix, exclude=atom_set)
 
+    @cached_property
+    def is_autonomous(self):
+        u = Dummy('u')
+        x = self.sym
+        syms = self.eq.subs(self.func, u).free_symbols
+        return x not in syms
+
     # TODO: Add methods that can be used by many ODE solvers:
     # order
     # is_linear()
@@ -155,7 +163,7 @@ class SingleODESolver:
     You can use a subclass of SingleODEProblem to solve a particular type of
     ODE. We first define a particular ODE problem:
 
-    >>> from sympy import Eq, Function, Symbol
+    >>> from sympy import Function, Symbol
     >>> x = Symbol('x')
     >>> f = Function('f')
     >>> eq = f(x).diff(x, 2)
@@ -181,11 +189,11 @@ class SingleODESolver:
 
     # Subclasses should store the hint name (the argument to dsolve) in this
     # attribute
-    hint = None  ## type: ClassVar[str]
+    hint = None  # type: ClassVar[str]
 
     # Subclasses should define this to indicate if they support an _Integral
     # hint.
-    has_integral = None  ## type: ClassVar[bool]
+    has_integral = None  # type: ClassVar[bool]
 
     # The ODE to be solved
     ode_problem = None  # type: SingleODEProblem
@@ -195,7 +203,7 @@ class SingleODESolver:
 
     # Subclasses should store in this attribute the list of order(s) of ODE
     # that subclass can solve or leave it to None if not specific to any order
-    order = None  # type: None or list
+    order = None  # type: Optional[list]
 
     def __init__(self, ode_problem):
         self.ode_problem = ode_problem
@@ -243,12 +251,12 @@ class SinglePatternODESolver(SingleODESolver):
         f = self.ode_problem.func.func
         x = self.ode_problem.sym
         order = self.ode_problem.order
-        df = f(x).diff(x)
+        df = f(x).diff(x, order)
 
-        if order != 1:
+        if order not in [1, 2]:
             return False
 
-        pattern = self._equation(f(x), x, 1)
+        pattern = self._equation(f(x), x, order)
 
         if not pattern.coeff(df).has(Wild):
             eq = expand(eq / eq.coeff(df))
@@ -354,7 +362,7 @@ class NthAlgebraic(SingleODESolver):
     # be stored in cached results we need to ensure that we always get the
     # same class back for each particular integration variable so we store these
     # classes in a global dict:
-    _diffx_stored = {}  ## type: Dict[Symbol, Type[Function]]
+    _diffx_stored = {}  # type: Dict[Symbol, Type[Function]]
 
     @staticmethod
     def _get_diffx(var):
@@ -443,8 +451,8 @@ class FirstLinear(SinglePatternODESolver):
         fx = self.ode_problem.func
         x = self.ode_problem.sym
         (C1,)  = self.ode_problem.get_numbered_constants(num=1)
-        gensol = Eq(fx, (((C1 + Integral(Q*exp(Integral(P, x)),x))
-            * exp(-Integral(P, x)))))
+        gensol = Eq(fx, ((C1 + Integral(Q*exp(Integral(P, x)),x))
+            * exp(-Integral(P, x))))
         return [gensol]
 
 
@@ -469,8 +477,8 @@ class AlmostLinear(SinglePatternODESolver):
     Examples
     ========
 
-    >>> from sympy import Function, Derivative, pprint, sin, cos
-    >>> from sympy.solvers.ode import dsolve, classify_ode
+    >>> from sympy import Function, pprint, sin, cos
+    >>> from sympy.solvers.ode import dsolve
     >>> from sympy.abc import x
     >>> f = Function('f')
     >>> d = f(x).diff(x)
@@ -529,8 +537,8 @@ class AlmostLinear(SinglePatternODESolver):
     def _get_general_solution(self, *, simplify: bool = True):
         x = self.ode_problem.sym
         (C1,)  = self.ode_problem.get_numbered_constants(num=1)
-        gensol = Eq(self.ly, (((C1 + Integral((self.cx/self.ax)*exp(Integral(self.bx/self.ax, x)),x))
-                * exp(-Integral(self.bx/self.ax, x)))))
+        gensol = Eq(self.ly, ((C1 + Integral((self.cx/self.ax)*exp(Integral(self.bx/self.ax, x)),x))
+                * exp(-Integral(self.bx/self.ax, x))))
 
         return [gensol]
 
@@ -631,8 +639,8 @@ class Bernoulli(SinglePatternODESolver):
         (C1,) = self.ode_problem.get_numbered_constants(num=1)
         if n==1:
             gensol = Eq(log(fx), (
-            (C1 + Integral((-P + Q),x)
-        )))
+            C1 + Integral((-P + Q),x)
+        ))
         else:
             gensol = Eq(fx**(1-n), (
                 (C1 - (n - 1) * Integral(Q*exp(-n*Integral(P, x))
@@ -653,7 +661,7 @@ class Factorable(SingleODESolver):
         Examples
         ========
 
-        >>> from sympy import Function, dsolve, Eq, pprint, Derivative
+        >>> from sympy import Function, dsolve, pprint
         >>> from sympy.abc import x
         >>> f = Function('f')
         >>> eq = (f(x)**2-4)*(f(x).diff(x)+f(x))
@@ -675,7 +683,8 @@ class Factorable(SingleODESolver):
         self.eqs = []
         eq = eq.collect(f(x), func = cancel)
         eq = fraction(factor(eq))[0]
-        roots = factor_list(eq)[1]
+        factors = Mul.make_args(factor(eq))
+        roots = [fac.as_base_exp() for fac in factors if len(fac.args)!=0]
         if len(roots)>1 or roots[0][1]>1:
             for base,expo in roots:
                 if base.has(f(x)):
@@ -730,7 +739,7 @@ class RiccatiSpecial(SinglePatternODESolver):
     and is valid when neither `a` nor `b` are zero and either `c` or `d` is
     zero.
 
-    >>> from sympy.abc import x, y, a, b, c, d
+    >>> from sympy.abc import x, a, b, c, d
     >>> from sympy.solvers.ode import dsolve, checkodesol
     >>> from sympy import pprint, Function
     >>> f = Function('f')
@@ -782,5 +791,66 @@ class RiccatiSpecial(SinglePatternODESolver):
         return [gensol]
 
 
+class SecondNonlinearAutonomousConserved(SinglePatternODESolver):
+    r"""
+    Gives solution for the autonomous second order nonlinear
+    differential equation of the form
+
+    .. math :: f''(x) = g(f(x))
+
+    The solution for this differential equation can be computed
+    by multiplying by `f'(x)` and integrating on both sides,
+    converting it into a first order differential equation.
+
+    Examples
+    ========
+
+    >>> from sympy import Function, symbols, dsolve
+    >>> f, g = symbols('f g', cls=Function)
+    >>> x = symbols('x')
+
+    >>> eq = f(x).diff(x, 2) - g(f(x))
+    >>> dsolve(eq, simplify=False)
+    [Eq(Integral(1/sqrt(C1 + 2*Integral(g(_u), _u)), (_u, f(x))), C2 + x),
+    Eq(Integral(1/sqrt(C1 + 2*Integral(g(_u), _u)), (_u, f(x))), C2 - x)]
+
+    >>> from sympy import exp, log
+    >>> eq = f(x).diff(x, 2) - exp(f(x)) + log(f(x))
+    >>> dsolve(eq, simplify=False)
+    [Eq(Integral(1/sqrt(-2*_u*log(_u) + 2*_u + C1 + 2*exp(_u)), (_u, f(x))), C2 + x),
+    Eq(Integral(1/sqrt(-2*_u*log(_u) + 2*_u + C1 + 2*exp(_u)), (_u, f(x))), C2 - x)]
+
+    References
+    ==========
+
+    http://eqworld.ipmnet.ru/en/solutions/ode/ode0301.pdf
+    """
+    hint = "2nd_nonlinear_autonomous_conserved"
+    has_integral = True
+    order = [2]
+
+    def _wilds(self, f, x, order):
+        fy = Wild('fy', exclude=[0, f(x).diff(x), f(x).diff(x, 2)])
+        return (fy,)
+
+    def _equation(self, fx, x, order):
+        fy = self.wilds()[0]
+        return fx.diff(x, 2) + fy
+
+    def _verify(self, fx):
+        return self.ode_problem.is_autonomous
+
+    def _get_general_solution(self, *, simplify: bool = True):
+        g = self.wilds_match()[0]
+        fx = self.ode_problem.func
+        x = self.ode_problem.sym
+        u = Dummy('u')
+        g = g.subs(fx, u)
+        C1, C2 = self.ode_problem.get_numbered_constants(num=2)
+        inside = -2*Integral(g, u) + C1
+        lhs = Integral(1/sqrt(inside), (u, fx))
+        return [Eq(lhs, C2 + x), Eq(lhs, C2 - x)]
+
+
 # Avoid circular import:
 from .ode import dsolve
