@@ -9,7 +9,6 @@ sympy.stats.crv
 sympy.stats.drv
 """
 
-from __future__ import print_function, division
 
 from sympy import (Basic, Lambda, sympify, Indexed, Symbol, ProductSet, S,
                    Dummy)
@@ -18,19 +17,17 @@ from sympy.concrete.summations import Sum, summation
 from sympy.core.compatibility import iterable
 from sympy.core.containers import Tuple
 from sympy.integrals.integrals import Integral, integrate
-from sympy.matrices import ImmutableMatrix
-from sympy.stats.crv import (ContinuousDistribution,
-                             SingleContinuousDistribution, SingleContinuousPSpace)
-from sympy.stats.drv import (DiscreteDistribution,
-                             SingleDiscreteDistribution, SingleDiscretePSpace)
-from sympy.stats.rv import (ProductPSpace, NamedArgsMixin,
-                            ProductDomain, RandomSymbol, random_symbols, SingleDomain)
+from sympy.matrices import ImmutableMatrix, matrix2numpy, list2numpy
+from sympy.stats.crv import SingleContinuousDistribution, SingleContinuousPSpace
+from sympy.stats.drv import SingleDiscreteDistribution, SingleDiscretePSpace
+from sympy.stats.rv import (ProductPSpace, NamedArgsMixin, Distribution,
+                            ProductDomain, RandomSymbol, random_symbols,
+                            SingleDomain, _symbol_converter)
 from sympy.utilities.misc import filldedent
-
+from sympy.external import import_module
 
 # __all__ = ['marginal_distribution']
 
-
 class JointPSpace(ProductPSpace):
     """
     Represents a joint probability space. Represented using symbols for
@@ -41,10 +38,7 @@ class JointPSpace(ProductPSpace):
             return SingleContinuousPSpace(sym, dist)
         if isinstance(dist, SingleDiscreteDistribution):
             return SingleDiscretePSpace(sym, dist)
-        if isinstance(sym, str):
-            sym = Symbol(sym)
-        if not isinstance(sym, Symbol):
-            raise TypeError("s should have been string or Symbol")
+        sym = _symbol_converter(sym)
         return Basic.__new__(cls, sym, dist)
 
     @property
@@ -134,13 +128,126 @@ class JointPSpace(ProductPSpace):
     def compute_density(self, expr):
         raise NotImplementedError()
 
-    def sample(self):
-        raise NotImplementedError()
+    def sample(self, size=(), library='scipy', seed=None):
+        """
+        Internal sample method
+
+        Returns dictionary mapping RandomSymbol to realization value.
+        """
+        return {RandomSymbol(self.symbol, self): self.distribution.sample(size,
+                    library=library, seed=seed)}
 
     def probability(self, condition):
         raise NotImplementedError()
 
-class JointDistribution(Basic, NamedArgsMixin):
+
+class SampleJointScipy:
+    """Returns the sample from scipy of the given distribution"""
+    def __new__(cls, dist, size, seed=None):
+        return cls._sample_scipy(dist, size, seed)
+
+    @classmethod
+    def _sample_scipy(cls, dist, size, seed):
+        """Sample from SciPy."""
+
+        from scipy import stats as scipy_stats
+        scipy_rv_map = {
+            'MultivariateNormalDistribution': lambda dist, size: scipy_stats.multivariate_normal.rvs(
+                mean=matrix2numpy(dist.mu).flatten(),
+                cov=matrix2numpy(dist.sigma), size=size, random_state=seed),
+            'MultivariateBetaDistribution': lambda dist, size: scipy_stats.dirichlet.rvs(
+                alpha=list2numpy(dist.alpha, float).flatten(), size=size, random_state=seed),
+            'MultinomialDistribution': lambda dist, size: scipy_stats.multinomial.rvs(
+                n=int(dist.n), p=list2numpy(dist.p, float).flatten(), size=size, random_state=seed)
+        }
+
+        dist_list = scipy_rv_map.keys()
+
+        if dist.__class__.__name__ not in dist_list:
+            return None
+
+        samples = scipy_rv_map[dist.__class__.__name__](dist, size)
+        if samples.shape[0:len(size)] != size:
+            samples = samples.reshape((size[0],) + samples.shape)
+        return samples
+
+class SampleJointNumpy:
+    """Returns the sample from numpy of the given distribution"""
+
+    def __new__(cls, dist, size, seed=None):
+        return cls._sample_numpy(dist, size, seed)
+
+    @classmethod
+    def _sample_numpy(cls, dist, size, seed):
+        """Sample from NumPy."""
+
+        import numpy
+        if seed is None or isinstance(seed, int):
+            rand_state = numpy.random.default_rng(seed=seed)
+        else:
+            rand_state = seed
+        numpy_rv_map = {
+            'MultivariateNormalDistribution': lambda dist, size: rand_state.multivariate_normal(
+                mean=matrix2numpy(dist.mu, float).flatten(),
+                cov=matrix2numpy(dist.sigma, float), size=size),
+            'MultivariateBetaDistribution': lambda dist, size: rand_state.dirichlet(
+                alpha=list2numpy(dist.alpha, float).flatten(), size=size),
+            'MultinomialDistribution': lambda dist, size: rand_state.multinomial(
+                n=int(dist.n), pvals=list2numpy(dist.p, float).flatten(), size=size)
+        }
+
+        dist_list = numpy_rv_map.keys()
+
+        if dist.__class__.__name__ not in dist_list:
+            return None
+
+        samples = numpy_rv_map[dist.__class__.__name__](dist, size)
+        if samples.shape[0:len(size)] != size:
+            samples = samples.reshape((size[0],) + samples.shape)
+        return samples
+
+class SampleJointPymc:
+    """Returns the sample from pymc3 of the given distribution"""
+
+    def __new__(cls, dist, size, seed=None):
+        return cls._sample_pymc3(dist, size, seed)
+
+    @classmethod
+    def _sample_pymc3(cls, dist, size, seed):
+        """Sample from PyMC3."""
+
+        import pymc3
+        pymc3_rv_map = {
+            'MultivariateNormalDistribution': lambda dist:
+                pymc3.MvNormal('X', mu=matrix2numpy(dist.mu, float).flatten(),
+                cov=matrix2numpy(dist.sigma, float), shape=(1, dist.mu.shape[0])),
+            'MultivariateBetaDistribution': lambda dist:
+                pymc3.Dirichlet('X', a=list2numpy(dist.alpha, float).flatten()),
+            'MultinomialDistribution': lambda dist:
+                pymc3.Multinomial('X', n=int(dist.n),
+                p=list2numpy(dist.p, float).flatten(), shape=(1, len(dist.p)))
+        }
+
+        dist_list = pymc3_rv_map.keys()
+
+        if dist.__class__.__name__ not in dist_list:
+            return None
+
+        with pymc3.Model():
+            pymc3_rv_map[dist.__class__.__name__](dist)
+            samples = pymc3.sample(size, chains=1, progressbar=False, random_seed=seed)[:]['X']
+            if samples.shape[0:len(size)] != size:
+                samples = samples.reshape((size[0],) + samples.shape)
+            return samples
+
+
+_get_sample_class_jrv = {
+    'scipy': SampleJointScipy,
+    'pymc3': SampleJointPymc,
+    'numpy': SampleJointNumpy
+}
+
+class JointDistribution(Distribution, NamedArgsMixin):
     """
     Represented by the random variables part of the joint distribution.
     Contains methods for PDF, CDF, sampling, marginal densities, etc.
@@ -178,6 +285,25 @@ class JointDistribution(Basic, NamedArgsMixin):
                     other[rvs[i]]))
         return density
 
+    def sample(self, size=(), library='scipy', seed=None):
+        """ A random realization from the distribution """
+
+        libraries = ['scipy', 'numpy', 'pymc3']
+        if library not in libraries:
+            raise NotImplementedError("Sampling from %s is not supported yet."
+                                        % str(library))
+        if not import_module(library):
+            raise ValueError("Failed to import %s" % library)
+
+        samps = _get_sample_class_jrv[library](self, size, seed=seed)
+
+        if samps is not None:
+            return samps
+        raise NotImplementedError(
+                "Sampling for %s is not currently implemented from %s"
+                % (self.__class__.__name__, library)
+                )
+
     def __call__(self, *args):
         return self.pdf(*args)
 
@@ -193,92 +319,9 @@ class JointRandomSymbol(RandomSymbol):
                     (self.name, self.pspace.component_count - 1))
             return Indexed(self, key)
 
-class JointDistributionHandmade(JointDistribution, NamedArgsMixin):
-
-    _argnames = ('pdf',)
-    is_Continuous = True
-
-    @property
-    def set(self):
-        return self.args[1]
-
-def marginal_distribution(rv, *indices):
-    """
-    Marginal distribution function of a joint random variable.
-
-    Parameters
-    ==========
-
-    rv: A random variable with a joint probability distribution.
-    indices: component indices or the indexed random symbol
-        for whom the joint distribution is to be calculated
-
-    Returns
-    =======
-
-    A Lambda expression n `sym`.
-
-    Examples
-    ========
-
-    >>> from sympy.stats.crv_types import Normal
-    >>> from sympy.stats.joint_rv import marginal_distribution
-    >>> m = Normal('X', [1, 2], [[2, 1], [1, 2]])
-    >>> marginal_distribution(m, m[0])(1)
-    1/(2*sqrt(pi))
-
-    """
-    indices = list(indices)
-    for i in range(len(indices)):
-        if isinstance(indices[i], Indexed):
-            indices[i] = indices[i].args[1]
-    prob_space = rv.pspace
-    if not indices:
-        raise ValueError(
-            "At least one component for marginal density is needed.")
-    if hasattr(prob_space.distribution, 'marginal_distribution'):
-        return prob_space.distribution.marginal_distribution(indices, rv.symbol)
-    return prob_space.marginal_distribution(*indices)
 
 
-class CompoundDistribution(Basic, NamedArgsMixin):
-    """
-    Represents a compound probability distribution.
-
-    Constructed using a single probability distribution with a parameter
-    distributed according to some given distribution.
-    """
-    def __new__(cls, dist):
-        if not isinstance(dist, (ContinuousDistribution, DiscreteDistribution)):
-            raise ValueError(filldedent('''CompoundDistribution can only be
-             initialized from ContinuousDistribution or DiscreteDistribution
-             '''))
-        _args = dist.args
-        if not any([isinstance(i, RandomSymbol) for i in _args]):
-            return dist
-        return Basic.__new__(cls, dist)
-
-    @property
-    def latent_distributions(self):
-        return random_symbols(self.args[0])
-
-    def pdf(self, *x):
-        dist = self.args[0]
-        z = Dummy('z')
-        if isinstance(dist, ContinuousDistribution):
-            rv = SingleContinuousPSpace(z, dist).value
-        elif isinstance(dist, DiscreteDistribution):
-            rv = SingleDiscretePSpace(z, dist).value
-        return MarginalDistribution(self, (rv,)).pdf(*x)
-
-    def set(self):
-        return self.args[0].set
-
-    def __call__(self, *args):
-        return self.pdf(*args)
-
-
-class MarginalDistribution(Basic):
+class MarginalDistribution(Distribution):
     """
     Represents the marginal distribution of a joint probability space.
 
@@ -310,15 +353,12 @@ class MarginalDistribution(Basic):
     @property
     def symbols(self):
         rvs = self.args[1]
-        return set([rv.pspace.symbol for rv in rvs])
+        return {rv.pspace.symbol for rv in rvs}
 
     def pdf(self, *x):
         expr, rvs = self.args[0], self.args[1]
         marginalise_out = [i for i in random_symbols(expr) if i not in rvs]
-        if isinstance(expr, CompoundDistribution):
-            syms = Dummy('x', real=True)
-            expr = expr.args[0].pdf(syms)
-        elif isinstance(expr, JointDistribution):
+        if isinstance(expr, JointDistribution):
             count = len(expr.domain.args)
             x = Dummy('x', real=True, finite=True)
             syms = tuple(Indexed(x, i) for i in count)
