@@ -13,24 +13,36 @@ sympy.stats.frv
 sympy.stats.rv_interface
 """
 
-from __future__ import print_function, division
 
+from functools import singledispatch
 from typing import Tuple as tTuple
 
-from sympy import (Basic, S, Expr, Symbol, Tuple, And, Add, Eq, lambdify,
+from sympy import (Basic, S, Expr, Symbol, Tuple, And, Add, Eq, lambdify, Or,
                    Equality, Lambda, sympify, Dummy, Ne, KroneckerDelta,
                    DiracDelta, Mul, Indexed, MatrixSymbol, Function)
 from sympy.core.relational import Relational
 from sympy.core.sympify import _sympify
-from sympy.logic.boolalg import Boolean
 from sympy.sets.sets import FiniteSet, ProductSet, Intersection
 from sympy.solvers.solveset import solveset
+from sympy.external import import_module
+from sympy.utilities.misc import filldedent
+import warnings
+
 
 x = Symbol('x')
 
+@singledispatch
+def is_random(x):
+    return False
+
+@is_random.register(Basic)
+def _(x):
+    atoms = x.free_symbols
+    return any([is_random(i) for i in atoms])
+
 class RandomDomain(Basic):
     """
-    Represents a set of variables and the values which they can take
+    Represents a set of variables and the values which they can take.
 
     See Also
     ========
@@ -65,7 +77,7 @@ class RandomDomain(Basic):
 
 class SingleDomain(RandomDomain):
     """
-    A single variable and its domain
+    A single variable and its domain.
 
     See Also
     ========
@@ -92,9 +104,27 @@ class SingleDomain(RandomDomain):
         return self.symbol == sym and val in self.set
 
 
+class MatrixDomain(RandomDomain):
+    """
+    A Random Matrix variable and its domain.
+
+    """
+    def __new__(cls, symbol, set):
+        symbol, set = _symbol_converter(symbol), _sympify(set)
+        return Basic.__new__(cls, symbol, set)
+
+    @property
+    def symbol(self):
+        return self.args[0]
+
+    @property
+    def symbols(self):
+        return FiniteSet(self.symbol)
+
+
 class ConditionalDomain(RandomDomain):
     """
-    A RandomDomain with an attached condition
+    A RandomDomain with an attached condition.
 
     See Also
     ========
@@ -103,8 +133,8 @@ class ConditionalDomain(RandomDomain):
     sympy.stats.frv.ConditionalFiniteDomain
     """
     def __new__(cls, fulldomain, condition):
-        condition = condition.xreplace(dict((rs, rs.symbol)
-            for rs in random_symbols(condition)))
+        condition = condition.xreplace({rs: rs.symbol
+            for rs in random_symbols(condition)})
         return Basic.__new__(cls, fulldomain, condition)
 
     @property
@@ -129,7 +159,10 @@ class ConditionalDomain(RandomDomain):
 
 class PSpace(Basic):
     """
-    A Probability Space
+    A Probability Space.
+
+    Explanation
+    ===========
 
     Probability Spaces encode processes that equal different values
     probabilistically. These underly Random Symbols which occur in SymPy
@@ -185,10 +218,7 @@ class SinglePSpace(PSpace):
     attributed to a single variable/symbol.
     """
     def __new__(cls, s, distribution):
-        if isinstance(s, str):
-            s = Symbol(s)
-        if not isinstance(s, Symbol):
-            raise TypeError("s should have been string or Symbol")
+        s = _symbol_converter(s)
         return Basic.__new__(cls, s, distribution)
 
     @property
@@ -210,11 +240,14 @@ class SinglePSpace(PSpace):
 
 class RandomSymbol(Expr):
     """
-    Random Symbols represent ProbabilitySpaces in SymPy Expressions
+    Random Symbols represent ProbabilitySpaces in SymPy Expressions.
     In principle they can take on any value that their symbol can take on
     within the associated PSpace with probability determined by the PSpace
     Density.
 
+    Explanation
+    ===========
+
     Random Symbols contain pspace and symbol properties.
     The pspace property points to the represented Probability Space
     The symbol is a standard SymPy Symbol that is used in that probability space
@@ -239,8 +272,7 @@ class RandomSymbol(Expr):
         if pspace is None:
             # Allow single arg, representing pspace == PSpace()
             pspace = PSpace()
-        if not isinstance(symbol, Symbol):
-            raise TypeError("symbol should be of type Symbol")
+        symbol = _symbol_converter(symbol)
         if not isinstance(pspace, PSpace):
             raise TypeError("pspace variable should be of type PSpace")
         if cls == JointRandomSymbol and isinstance(pspace, SinglePSpace):
@@ -277,6 +309,9 @@ class RandomSymbol(Expr):
 class RandomIndexedSymbol(RandomSymbol):
 
     def __new__(cls, idx_obj, pspace=None):
+        if pspace is None:
+            # Allow single arg, representing pspace == PSpace()
+            pspace = PSpace()
         if not isinstance(idx_obj, (Indexed, Function)):
             raise TypeError("An Function or Indexed object is expected not %s"%(idx_obj))
         return Basic.__new__(cls, idx_obj, pspace)
@@ -291,15 +326,31 @@ class RandomIndexedSymbol(RandomSymbol):
         elif isinstance(self.symbol, Function):
             return self.symbol.args[0]
 
-class RandomMatrixSymbol(MatrixSymbol):
+    @property
+    def free_symbols(self):
+        if self.key.free_symbols:
+            free_syms = self.key.free_symbols
+            free_syms.add(self)
+            return free_syms
+        return {self}
+
+    @property
+    def pspace(self):
+        return self.args[1]
+
+class RandomMatrixSymbol(RandomSymbol, MatrixSymbol): # type: ignore
     def __new__(cls, symbol, n, m, pspace=None):
         n, m = _sympify(n), _sympify(m)
         symbol = _symbol_converter(symbol)
+        if pspace is None:
+            # Allow single arg, representing pspace == PSpace()
+            pspace = PSpace()
         return Basic.__new__(cls, symbol, n, m, pspace)
 
     symbol = property(lambda self: self.args[0])
     pspace = property(lambda self: self.args[3])
 
+
 class ProductPSpace(PSpace):
     """
     Abstract class for representing probability spaces with multiple random
@@ -318,7 +369,7 @@ class IndependentProductPSpace(ProductPSpace):
     A probability space resulting from the merger of two independent probability
     spaces.
 
-    Often created using the function, pspace
+    Often created using the function, pspace.
     """
 
     def __new__(cls, *spaces):
@@ -330,7 +381,8 @@ class IndependentProductPSpace(ProductPSpace):
         symbols = FiniteSet(*[val.symbol for val in rs_space_dict.keys()])
 
         # Overlapping symbols
-        from sympy.stats.joint_rv import MarginalDistribution, CompoundDistribution
+        from sympy.stats.joint_rv import MarginalDistribution
+        from sympy.stats.compound_rv import CompoundDistribution
         if len(symbols) < sum(len(space.symbols) for space in spaces if not
          isinstance(space.distribution, (
             CompoundDistribution, MarginalDistribution))):
@@ -347,7 +399,7 @@ class IndependentProductPSpace(ProductPSpace):
     @property
     def pdf(self):
         p = Mul(*[space.pdf for space in self.spaces])
-        return p.subs(dict((rv, rv.symbol) for rv in self.values))
+        return p.subs({rv: rv.symbol for rv in self.values})
 
     @property
     def rs_space_dict(self):
@@ -386,15 +438,20 @@ class IndependentProductPSpace(ProductPSpace):
     def density(self):
         raise NotImplementedError("Density not available for ProductSpaces")
 
-    def sample(self, size=()):
+    def sample(self, size=(), library='scipy', seed=None):
         return {k: v for space in self.spaces
-            for k, v in space.sample().items()}
+            for k, v in space.sample(size=size, library=library, seed=seed).items()}
+
 
     def probability(self, condition, **kwargs):
         cond_inv = False
         if isinstance(condition, Ne):
             condition = Eq(condition.args[0], condition.args[1])
             cond_inv = True
+        elif isinstance(condition, And): # they are independent
+            return Mul(*[self.probability(arg) for arg in condition.args])
+        elif isinstance(condition, Or): # they are independent
+            return Add(*[self.probability(arg) for arg in condition.args])
         expr = condition.lhs - condition.rhs
         rvs = random_symbols(expr)
         dens = self.compute_density(expr)
@@ -437,7 +494,7 @@ class IndependentProductPSpace(ProductPSpace):
 
     def conditional_space(self, condition, normalize=True, **kwargs):
         rvs = random_symbols(condition)
-        condition = condition.xreplace(dict((rv, rv.symbol) for rv in self.values))
+        condition = condition.xreplace({rv: rv.symbol for rv in self.values})
         if any([pspace(rv).is_Continuous for rv in rvs]):
             from sympy.stats.crv import (ConditionalContinuousDomain,
                 ContinuousPSpace)
@@ -463,7 +520,7 @@ class IndependentProductPSpace(ProductPSpace):
 
 class ProductDomain(RandomDomain):
     """
-    A domain resulting from the merger of two independent domains
+    A domain resulting from the merger of two independent domains.
 
     See Also
     ========
@@ -496,8 +553,8 @@ class ProductDomain(RandomDomain):
 
     @property
     def sym_domain_dict(self):
-        return dict((symbol, domain) for domain in self.domains
-                                     for symbol in domain.symbols)
+        return {symbol: domain for domain in self.domains
+                                     for symbol in domain.symbols}
 
     @property
     def symbols(self):
@@ -552,7 +609,6 @@ def pspace(expr):
     ========
 
     >>> from sympy.stats import pspace, Normal
-    >>> from sympy.stats.rv import IndependentProductPSpace
     >>> X = Normal('X', 0, 1)
     >>> pspace(2*X + 1) == X.pspace
     True
@@ -570,6 +626,11 @@ def pspace(expr):
     # If only one space present
     if all(rv.pspace == rvs[0].pspace for rv in rvs):
         return rvs[0].pspace
+    from sympy.stats.compound_rv import CompoundPSpace
+    from sympy.stats.stochastic_process import StochasticPSpace
+    for rv in rvs:
+        if isinstance(rv.pspace, (CompoundPSpace, StochasticPSpace)):
+            return rv.pspace
     # Otherwise make a product space
     return IndependentProductPSpace(*[rv.pspace for rv in rvs])
 
@@ -601,7 +662,11 @@ def rs_swap(a, b):
 
 
 def given(expr, condition=None, **kwargs):
-    r""" Conditional Random Expression
+    r""" Conditional Random Expression.
+
+    Explanation
+    ===========
+
     From a random expression and a condition on that expression creates a new
     probability space from the condition and returns the same expression on that
     conditional probability space.
@@ -635,7 +700,7 @@ def given(expr, condition=None, **kwargs):
          2*\/ pi
     """
 
-    if not random_symbols(condition) or pspace_independent(expr, condition):
+    if not is_random(condition) or pspace_independent(expr, condition):
         return expr
 
     if isinstance(condition, RandomSymbol):
@@ -683,7 +748,7 @@ def given(expr, condition=None, **kwargs):
 
 def expectation(expr, condition=None, numsamples=None, evaluate=True, **kwargs):
     """
-    Returns the expected value of a random expression
+    Returns the expected value of a random expression.
 
     Parameters
     ==========
@@ -713,36 +778,19 @@ def expectation(expr, condition=None, numsamples=None, evaluate=True, **kwargs):
     5
     """
 
-    if not random_symbols(expr):  # expr isn't random?
+    if not is_random(expr):  # expr isn't random?
         return expr
-    if numsamples:  # Computing by monte carlo sampling?
-        return sampling_E(expr, condition, numsamples=numsamples)
-
-    if expr.has(RandomIndexedSymbol):
-        return pspace(expr).compute_expectation(expr, condition, evaluate, **kwargs)
-
-    # Create new expr and recompute E
-    if condition is not None:  # If there is a condition
-        return expectation(given(expr, condition), evaluate=evaluate)
-
-    # A few known statements for efficiency
-
-    if expr.is_Add:  # We know that E is Linear
-        return Add(*[expectation(arg, evaluate=evaluate)
-                     for arg in expr.args])
-
-    # Otherwise case is simple, pass work off to the ProbabilitySpace
-    result = pspace(expr).compute_expectation(expr, evaluate=evaluate, **kwargs)
-    if evaluate and hasattr(result, 'doit'):
-        return result.doit(**kwargs)
-    else:
-        return result
+    kwargs['numsamples'] = numsamples
+    from sympy.stats.symbolic_probability import Expectation
+    if evaluate:
+        return Expectation(expr, condition).doit(**kwargs)
+    return Expectation(expr, condition)
 
 
 def probability(condition, given_condition=None, numsamples=None,
                 evaluate=True, **kwargs):
     """
-    Probability that a condition is true, optionally given a second condition
+    Probability that a condition is true, optionally given a second condition.
 
     Parameters
     ==========
@@ -771,50 +819,16 @@ def probability(condition, given_condition=None, numsamples=None,
     5/12
     """
 
-    condition = sympify(condition)
-    given_condition = sympify(given_condition)
-
-    if condition.has(RandomIndexedSymbol):
-        return pspace(condition).probability(condition, given_condition, evaluate, **kwargs)
-
-    if isinstance(given_condition, RandomSymbol):
-        condrv = random_symbols(condition)
-        if len(condrv) == 1 and condrv[0] == given_condition:
-            from sympy.stats.frv_types import BernoulliDistribution
-            return BernoulliDistribution(probability(condition), 0, 1)
-        if any([dependent(rv, given_condition) for rv in condrv]):
-            from sympy.stats.symbolic_probability import Probability
-            return Probability(condition, given_condition)
-        else:
-            return probability(condition)
-
-    if given_condition is not None and \
-            not isinstance(given_condition, (Relational, Boolean)):
-        raise ValueError("%s is not a relational or combination of relationals"
-                % (given_condition))
-    if given_condition == False:
-        return S.Zero
-    if not isinstance(condition, (Relational, Boolean)):
-        raise ValueError("%s is not a relational or combination of relationals"
-                % (condition))
-    if condition is S.true:
-        return S.One
-    if condition is S.false:
-        return S.Zero
-
-    if numsamples:
-        return sampling_P(condition, given_condition, numsamples=numsamples,
-                **kwargs)
-    if given_condition is not None:  # If there is a condition
-        # Recompute on new conditional expr
-        return probability(given(condition, given_condition, **kwargs), **kwargs)
-
-    # Otherwise pass work off to the ProbabilitySpace
-    result = pspace(condition).probability(condition, **kwargs)
-    if evaluate and hasattr(result, 'doit'):
-        return result.doit()
-    else:
-        return result
+    kwargs['numsamples'] = numsamples
+    from sympy.stats.symbolic_probability import Probability
+    if evaluate:
+        return Probability(condition, given_condition).doit(**kwargs)
+    ### TODO: Remove the user warnings in the future releases
+    message = ("Since version 1.7, using `evaluate=False` returns `Probability` "
+              "object. If you want unevaluated Integral/Sum use "
+              "`P(condition, given_condition, evaluate=False).rewrite(Integral)`")
+    warnings.warn(filldedent(message))
+    return Probability(condition, given_condition)
 
 
 class Density(Basic):
@@ -828,25 +842,28 @@ class Density(Basic):
             return None
 
     def doit(self, evaluate=True, **kwargs):
+        from sympy.stats.random_matrix import RandomMatrixPSpace
         from sympy.stats.joint_rv import JointPSpace
+        from sympy.stats.matrix_distributions import MatrixPSpace
+        from sympy.stats.compound_rv import CompoundPSpace
         from sympy.stats.frv import SingleFiniteDistribution
         expr, condition = self.expr, self.condition
-        if _sympify(expr).has(RandomMatrixSymbol):
-            return pspace(expr).compute_density(expr)
+
         if isinstance(expr, SingleFiniteDistribution):
             return expr.dict
         if condition is not None:
             # Recompute on new conditional expr
             expr = given(expr, condition, **kwargs)
-        if isinstance(expr, RandomSymbol) and \
-            isinstance(expr.pspace, JointPSpace):
-            return expr.pspace.distribution
         if not random_symbols(expr):
             return Lambda(x, DiracDelta(x - expr))
-        if (isinstance(expr, RandomSymbol) and
-            hasattr(expr.pspace, 'distribution') and
-            isinstance(pspace(expr), (SinglePSpace))):
-            return expr.pspace.distribution
+        if isinstance(expr, RandomSymbol):
+            if isinstance(expr.pspace, (SinglePSpace, JointPSpace, MatrixPSpace)) and \
+                hasattr(expr.pspace, 'distribution'):
+                return expr.pspace.distribution
+            elif isinstance(expr.pspace, RandomMatrixPSpace):
+                return expr.pspace.model
+        if isinstance(pspace(expr), CompoundPSpace):
+            kwargs['compound_evaluate'] = evaluate
         result = pspace(expr).compute_density(expr, **kwargs)
 
         if evaluate and hasattr(result, 'doit'):
@@ -860,6 +877,9 @@ def density(expr, condition=None, evaluate=True, numsamples=None, **kwargs):
     Probability density of a random expression, optionally given a second
     condition.
 
+    Explanation
+    ===========
+
     This density will take on different forms for different types of
     probability spaces. Discrete variables produce Dicts. Continuous
     variables produce Lambdas.
@@ -904,7 +924,10 @@ def cdf(expr, condition=None, evaluate=True, **kwargs):
     """
     Cumulative Distribution Function of a random expression.
 
-    optionally given a second condition
+    optionally given a second condition.
+
+    Explanation
+    ===========
 
     This density will take on different forms for different types of
     probability spaces.
@@ -944,9 +967,9 @@ def cdf(expr, condition=None, evaluate=True, **kwargs):
 
 def characteristic_function(expr, condition=None, evaluate=True, **kwargs):
     """
-    Characteristic function of a random expression, optionally given a second condition
+    Characteristic function of a random expression, optionally given a second condition.
 
-    Returns a Lambda
+    Returns a Lambda.
 
     Examples
     ========
@@ -994,7 +1017,7 @@ def where(condition, given_condition=None, **kwargs):
     ========
 
     >>> from sympy.stats import where, Die, Normal
-    >>> from sympy import symbols, And
+    >>> from sympy import And
 
     >>> D1, D2 = Die('a', 6), Die('b', 6)
     >>> a, b = D1.symbol, D2.symbol
@@ -1017,74 +1040,101 @@ def where(condition, given_condition=None, **kwargs):
     return pspace(condition).where(condition, **kwargs)
 
 
-def sample(expr, condition=None, size=(), **kwargs):
+def sample(expr, condition=None, size=(), library='scipy',
+           numsamples=1, seed=None, **kwargs):
     """
-    A realization of the random expression
-
-    Examples
-    ========
-
-    >>> from sympy.stats import Die, sample, Normal
-    >>> X, Y, Z = Die('X', 6), Die('Y', 6), Die('Z', 6)
-
-    >>> die_roll = sample(X + Y + Z) # A random realization of three dice
-    >>> N = Normal('N', 3, 4)
-    >>> samp = sample(N)
-    >>> samp in N.pspace.domain.set
-    True
-    >>> samp_list = sample(N, size=4)
-    >>> [sam in N.pspace.domain.set for sam in samp_list]
-    [True, True, True, True]
-
-    """
-    return next(sample_iter(expr, condition, size=size, numsamples=1))
-
-
-def sample_iter(expr, condition=None, size=(), numsamples=S.Infinity, **kwargs):
-    """
-    Returns an iterator of realizations from the expression given a condition
+    A realization of the random expression.
 
     Parameters
     ==========
 
-    expr: Expr
-        Random expression to be realized
-    condition: Expr, optional
+    expr : Expression of random variables
+        Expression from which sample is extracted
+    condition : Expr containing RandomSymbols
         A conditional expression
-    numsamples: integer, optional
-        Length of the iterator (defaults to infinity)
+    size : int, tuple
+        Represents size of each sample in numsamples
+    library : str
+        - 'scipy' : Sample using scipy
+        - 'numpy' : Sample using numpy
+        - 'pymc3' : Sample using PyMC3
+
+        Choose any of the available options to sample from as string,
+        by default is 'scipy'
+    numsamples : int
+        Number of samples, each with size as ``size``
+    seed :
+        An object to be used as seed by the given external library for sampling `expr`.
+        Following is the list of possible types of object for the supported libraries,
+
+        - 'scipy': int, numpy.random.RandomState, numpy.random.Generator
+        - 'numpy': int, numpy.random.RandomState, numpy.random.Generator
+        - 'pymc3': int
+
+        Optional, by default None, in which case seed settings
+        related to the given library will be used.
+        No modifications to environment's global seed settings
+        are done by this argument.
 
     Examples
     ========
 
-    >>> from sympy.stats import Normal, sample_iter
-    >>> X = Normal('X', 0, 1)
-    >>> expr = X*X + 3
-    >>> iterator = sample_iter(expr, numsamples=3)
-    >>> list(iterator) # doctest: +SKIP
-    [12, 4, 7]
+    >>> from sympy.stats import Die, sample, Normal, Geometric
+    >>> X, Y, Z = Die('X', 6), Die('Y', 6), Die('Z', 6) # Finite Random Variable
 
-    See Also
-    ========
+    >>> die_roll = sample(X + Y + Z) # doctest: +SKIP
+    >>> next(die_roll) # doctest: +SKIP
+    6
+    >>> N = Normal('N', 3, 4) # Continuous Random Variable
+    >>> samp = next(sample(N)) # doctest: +SKIP
+    >>> samp in N.pspace.domain.set # doctest: +SKIP
+    True
+    >>> samp = next(sample(N, N>0)) # doctest: +SKIP
+    >>> samp > 0 # doctest: +SKIP
+    True
+    >>> samp_list = next(sample(N, size=4)) # doctest: +SKIP
+    >>> [sam in N.pspace.domain.set for sam in samp_list] # doctest: +SKIP
+    [True, True, True, True]
+    >>> G = Geometric('G', 0.5) # Discrete Random Variable
+    >>> samp_list = next(sample(G, size=3)) # doctest: +SKIP
+    >>> samp_list # doctest: +SKIP
+    array([10,  4,  1])
+    >>> [sam in G.pspace.domain.set for sam in samp_list] # doctest: +SKIP
+    [True, True, True]
+    >>> MN = Normal("MN", [3, 4], [[2, 1], [1, 2]]) # Joint Random Variable
+    >>> samp_list = next(sample(MN, size=4)) # doctest: +SKIP
+    >>> samp_list # doctest: +SKIP
+    array([[4.22564264, 3.23364418],
+           [3.41002011, 4.60090908],
+           [3.76151866, 4.77617143],
+           [4.71440865, 2.65714157]])
+    >>> [tuple(sam) in MN.pspace.domain.set for sam in samp_list] # doctest: +SKIP
+    [True, True, True, True]
 
-    sample
-    sampling_P
-    sampling_E
-    sample_iter_lambdify
-    sample_iter_subs
+
+    Returns
+    =======
+
+    sample: iterator object
+        iterator object containing the sample/samples of given expr
 
     """
-    # lambdify is much faster but not as robust
-    try:
-        return sample_iter_lambdify(expr, condition, size=size, numsamples=numsamples, **kwargs)
-    # use subs when lambdify fails
-    except TypeError:
-        return sample_iter_subs(expr, condition, size=size, numsamples=numsamples, **kwargs)
+    ### TODO: Remove the user warnings in the future releases
+    message = ("The return type of sample has been changed to return an "
+                  "iterator object since version 1.7. For more information see "
+                  "https://github.com/sympy/sympy/issues/19061")
+    warnings.warn(filldedent(message))
+    return sample_iter(expr, condition, size=size, library=library,
+                                                        numsamples=numsamples, seed=seed)
+
 
 def quantile(expr, evaluate=True, **kwargs):
     r"""
     Return the :math:`p^{th}` order quantile of a probability distribution.
 
+    Explanation
+    ===========
+
     Quantile is defined as the value at which the probability of the random
     variable is less than or equal to the given probability.
 
@@ -1127,42 +1177,115 @@ def quantile(expr, evaluate=True, **kwargs):
     else:
         return result
 
-def sample_iter_lambdify(expr, condition=None, size=(), numsamples=S.Infinity, **kwargs):
+def sample_iter(expr, condition=None, size=(), library='scipy',
+                    numsamples=S.Infinity, seed=None, **kwargs):
+
     """
-    Uses lambdify for computation. This is fast but does not always work.
+    Returns an iterator of realizations from the expression given a condition.
+
+    Parameters
+    ==========
+
+    expr: Expr
+        Random expression to be realized
+    condition: Expr, optional
+        A conditional expression
+    size : int, tuple
+        Represents size of each sample in numsamples
+    numsamples: integer, optional
+        Length of the iterator (defaults to infinity)
+    seed :
+        An object to be used as seed by the given external library for sampling `expr`.
+        Following is the list of possible types of object for the supported libraries,
+
+        - 'scipy': int, numpy.random.RandomState, numpy.random.Generator
+        - 'numpy': int, numpy.random.RandomState, numpy.random.Generator
+        - 'pymc3': int
+
+        Optional, by default None, in which case seed settings
+        related to the given library will be used.
+        No modifications to environment's global seed settings
+        are done by this argument.
+
+    Examples
+    ========
+
+    >>> from sympy.stats import Normal, sample_iter
+    >>> X = Normal('X', 0, 1)
+    >>> expr = X*X + 3
+    >>> iterator = sample_iter(expr, numsamples=3) # doctest: +SKIP
+    >>> list(iterator) # doctest: +SKIP
+    [12, 4, 7]
+
+    Returns
+    =======
+
+    sample_iter: iterator object
+        iterator object containing the sample/samples of given expr
 
     See Also
     ========
 
-    sample_iter
+    sample
+    sampling_P
+    sampling_E
 
     """
-    if condition:
+    from sympy.stats.joint_rv import JointRandomSymbol
+    if not import_module(library):
+        raise ValueError("Failed to import %s" % library)
+
+    if condition is not None:
         ps = pspace(Tuple(expr, condition))
     else:
         ps = pspace(expr)
 
     rvs = list(ps.values)
-    fn = lambdify(rvs, expr, **kwargs)
-    if condition:
+    if isinstance(expr, JointRandomSymbol):
+        expr = expr.subs({expr: RandomSymbol(expr.symbol, expr.pspace)})
+    else:
+        sub = {}
+        for arg in expr.args:
+            if isinstance(arg, JointRandomSymbol):
+                sub[arg] = RandomSymbol(arg.symbol, arg.pspace)
+        expr = expr.subs(sub)
+
+    def fn_subs(*args):
+        return expr.subs({rv: arg for rv, arg in zip(rvs, args)})
+
+    def given_fn_subs(*args):
+        if condition is not None:
+            return condition.subs({rv: arg for rv, arg in zip(rvs, args)})
+        return False
+
+    if library == 'pymc3':
+        # Currently unable to lambdify in pymc3
+        # TODO : Remove 'pymc3' when lambdify accepts 'pymc3' as module
+        fn = lambdify(rvs, expr, **kwargs)
+    else:
+        fn = lambdify(rvs, expr, modules=library, **kwargs)
+
+    if condition is not None:
         given_fn = lambdify(rvs, condition, **kwargs)
 
-    # Check that lambdify can handle the expression
-    # Some operations like Sum can prove difficult
-    d = ps.sample(size)  # a dictionary that maps RVs to values
-    args = [d[rv] for rv in rvs]
-    fn(*args)
-    if condition:
-        given_fn(*args)
-
-    def return_generator():
+    def return_generator_infinite():
         count = 0
+        np = import_module('numpy')
+        if np:
+            rand_state = np.random.default_rng(seed=seed)
+        else:
+            rand_state = None
         while count < numsamples:
-            d = ps.sample(size)  # a dictionary that maps RVs to values
+            d = ps.sample(size=size, library=library, seed=rand_state)  # a dictionary that maps RVs to values
             args = [d[rv] for rv in rvs]
 
-            if condition:  # Check that these values satisfy the condition
-                gd = given_fn(*args)
+            if condition is not None:  # Check that these values satisfy the condition
+                # TODO: Replace the try-except block with only given_fn(*args)
+                # once lambdify works with unevaluated SymPy objects.
+                try:
+                    gd = given_fn(*args)
+                except (NameError, TypeError):
+                    gd = given_fn_subs(*args)
                 if gd != True and gd != False:
                     raise ValueError(
                         "Conditions must not contain free symbols")
@@ -1171,43 +1294,65 @@ def sample_iter_lambdify(expr, condition=None, size=(), numsamples=S.Infinity, *
 
             yield fn(*args)
             count += 1
-    return return_generator()
+
+    def return_generator_finite():
+        faulty = True
+        while faulty:
+            d = ps.sample(size=(numsamples,) + ((size,) if isinstance(size, int) else size),
+                          library=library, seed=seed) # a dictionary that maps RVs to values
+            faulty = False
+            count = 0
+            while count < numsamples and not faulty:
+                args = [d[rv][count] for rv in rvs]
+
+                if condition is not None:  # Check that these values satisfy the condition
+                    # TODO: Replace the try-except block with only given_fn(*args)
+                    # once lambdify works with unevaluated SymPy objects.
+                    try:
+                        gd = given_fn(*args)
+                    except (NameError, TypeError):
+                        gd = given_fn_subs(*args)
+                    if gd != True and gd != False:
+                        raise ValueError(
+                            "Conditions must not contain free symbols")
+                    if not gd:  # If the values don't satisfy then try again
+                        faulty = True
+
+                count += 1
+
+        count = 0
+        while count < numsamples:
+            args = [d[rv][count] for rv in rvs]
+            # TODO: Replace the try-except block with only fn(*args)
+            # once lambdify works with unevaluated SymPy objects.
+            try:
+                yield fn(*args)
+            except (NameError, TypeError):
+                yield fn_subs(*args)
+            count += 1
+
+    if numsamples is S.Infinity:
+        return return_generator_infinite()
+
+    return return_generator_finite()
+
+def sample_iter_lambdify(expr, condition=None, size=(),
+                         numsamples=S.Infinity, seed=None, **kwargs):
+
+    return sample_iter(expr, condition=condition, size=size,
+                       numsamples=numsamples, seed=seed, **kwargs)
+
+def sample_iter_subs(expr, condition=None, size=(),
+                     numsamples=S.Infinity, seed=None, **kwargs):
+
+    return sample_iter(expr, condition=condition, size=size,
+                       numsamples=numsamples, seed=seed, **kwargs)
 
 
-def sample_iter_subs(expr, condition=None, size=(), numsamples=S.Infinity, **kwargs):
+def sampling_P(condition, given_condition=None, library='scipy', numsamples=1,
+               evalf=True, seed=None, **kwargs):
     """
-    Uses subs for computation. This is slow but almost always works.
-
-    See Also
-    ========
-
-    sample_iter
-
-    """
-    if condition is not None:
-        ps = pspace(Tuple(expr, condition))
-    else:
-        ps = pspace(expr)
-
-    count = 0
-    while count < numsamples:
-        d = ps.sample(size)  # a dictionary that maps RVs to values
-
-        if condition is not None:  # Check that these values satisfy the condition
-            gd = condition.xreplace(d)
-            if gd != True and gd != False:
-                raise ValueError("Conditions must not contain free symbols")
-            if not gd:  # If the values don't satisfy then try again
-                continue
-
-        yield expr.xreplace(d)
-        count += 1
-
-
-def sampling_P(condition, given_condition=None, numsamples=1,
-               evalf=True, **kwargs):
-    """
-    Sampling version of P
+    Sampling version of P.
 
     See Also
     ========
@@ -1220,14 +1365,10 @@ def sampling_P(condition, given_condition=None, numsamples=1,
 
     count_true = 0
     count_false = 0
-
-    samples = sample_iter(condition, given_condition,
-                          numsamples=numsamples, **kwargs)
+    samples = sample_iter(condition, given_condition, library=library,
+                          numsamples=numsamples, seed=seed, **kwargs)
 
     for sample in samples:
-        if sample != True and sample != False:
-            raise ValueError("Conditions must not contain free symbols")
-
         if sample:
             count_true += 1
         else:
@@ -1240,10 +1381,10 @@ def sampling_P(condition, given_condition=None, numsamples=1,
         return result
 
 
-def sampling_E(expr, given_condition=None, numsamples=1,
-               evalf=True, **kwargs):
+def sampling_E(expr, given_condition=None, library='scipy', numsamples=1,
+               evalf=True, seed=None, **kwargs):
     """
-    Sampling version of E
+    Sampling version of E.
 
     See Also
     ========
@@ -1252,19 +1393,19 @@ def sampling_E(expr, given_condition=None, numsamples=1,
     sampling_P
     sampling_density
     """
+    samples = list(sample_iter(expr, given_condition, library=library,
+                          numsamples=numsamples, seed=seed, **kwargs))
+    result = Add(*[samp for samp in samples]) / numsamples
 
-    samples = sample_iter(expr, given_condition,
-                          numsamples=numsamples, **kwargs)
-
-    result = Add(*list(samples)) / numsamples
     if evalf:
         return result.evalf()
     else:
         return result
 
-def sampling_density(expr, given_condition=None, numsamples=1, **kwargs):
+def sampling_density(expr, given_condition=None, library='scipy',
+                    numsamples=1, seed=None, **kwargs):
     """
-    Sampling version of density
+    Sampling version of density.
 
     See Also
     ========
@@ -1274,15 +1415,16 @@ def sampling_density(expr, given_condition=None, numsamples=1, **kwargs):
     """
 
     results = {}
-    for result in sample_iter(expr, given_condition,
-                              numsamples=numsamples, **kwargs):
+    for result in sample_iter(expr, given_condition, library=library,
+                              numsamples=numsamples, seed=seed, **kwargs):
         results[result] = results.get(result, 0) + 1
+
     return results
 
 
 def dependent(a, b):
     """
-    Dependence of two random expressions
+    Dependence of two random expressions.
 
     Two expressions are independent if knowledge of one does not change
     computations on the other.
@@ -1319,7 +1461,7 @@ def dependent(a, b):
 
 def independent(a, b):
     """
-    Independence of two random expressions
+    Independence of two random expressions.
 
     Two expressions are independent if knowledge of one does not change
     computations on the other.
@@ -1383,7 +1525,8 @@ def rv_subs(expr, symbols=None):
     swapdict = {rv: rv.symbol for rv in symbols}
     return expr.subs(swapdict)
 
-class NamedArgsMixin(object):
+
+class NamedArgsMixin:
     _argnames = ()  # type: tTuple[str, ...]
 
     def __getattr__(self, attr):
@@ -1393,6 +1536,54 @@ class NamedArgsMixin(object):
             raise AttributeError("'%s' object has no attribute '%s'" % (
                 type(self).__name__, attr))
 
+
+class Distribution(Basic):
+
+    def sample(self, size=(), library='scipy', seed=None):
+        """ A random realization from the distribution """
+
+        module = import_module(library)
+        if library in {'scipy', 'numpy', 'pymc3'} and module is None:
+            raise ValueError("Failed to import %s" % library)
+
+        if library == 'scipy':
+            # scipy does not require map as it can handle using custom distributions.
+            # However, we will still use a map where we can.
+
+            # TODO: do this for drv.py and frv.py if necessary.
+            # TODO: add more distributions here if there are more
+            # See links below referring to sections beginning with "A common parametrization..."
+            # I will remove all these comments if everything is ok.
+
+            from sympy.stats.sampling.sample_scipy import do_sample_scipy
+            samps = do_sample_scipy(self, size, seed)
+        elif library == 'numpy':
+            from sympy.stats.sampling.sample_numpy import do_sample_numpy
+            import numpy
+            if seed is None or isinstance(seed, int):
+                rand_state = numpy.random.default_rng(seed=seed)
+            else:
+                rand_state = seed
+            samps = do_sample_numpy(self, size, rand_state)
+        elif library == 'pymc3':
+            from sympy.stats.sampling.sample_pymc3 import do_sample_pymc3
+            import pymc3
+            with pymc3.Model():
+                if do_sample_pymc3(self):
+                    samps = pymc3.sample(size, chains=1, progressbar=False, random_seed=seed)[:]['X']
+                else:
+                    samps = None
+        else:
+            raise NotImplementedError("Sampling from %s is not supported yet."
+                                      % str(library))
+
+        if samps is not None:
+            return samps
+        raise NotImplementedError(
+            "Sampling for %s is not currently implemented from %s"
+            % (self, library))
+
+
 def _value_check(condition, message):
     """
     Raise a ValueError with message if condition is False, else
@@ -1493,7 +1684,44 @@ def _symbol_converter(sym):
     True
     """
     if isinstance(sym, str):
-            sym = Symbol(sym)
+        sym = Symbol(sym)
     if not isinstance(sym, Symbol):
         raise TypeError("%s is neither a Symbol nor a string"%(sym))
     return sym
+
+def sample_stochastic_process(process):
+    """
+    This function is used to sample from stochastic process.
+
+    Parameters
+    ==========
+
+    process: StochasticProcess
+        Process used to extract the samples. It must be an instance of
+        StochasticProcess
+
+    Examples
+    ========
+
+    >>> from sympy.stats import sample_stochastic_process, DiscreteMarkovChain
+    >>> from sympy import Matrix
+    >>> T = Matrix([[0.5, 0.2, 0.3],[0.2, 0.5, 0.3],[0.2, 0.3, 0.5]])
+    >>> Y = DiscreteMarkovChain("Y", [0, 1, 2], T)
+    >>> next(sample_stochastic_process(Y)) in Y.state_space # doctest: +SKIP
+    True
+    >>> next(sample_stochastic_process(Y))  # doctest: +SKIP
+    0
+    >>> next(sample_stochastic_process(Y)) # doctest: +SKIP
+    2
+
+    Returns
+    =======
+
+    sample: iterator object
+        iterator object containing the sample of given process
+
+    """
+    from sympy.stats.stochastic_process_types import StochasticProcess
+    if not isinstance(process, StochasticProcess):
+        raise ValueError("Process must be an instance of Stochastic Process")
+    return process.sample()
