@@ -1,21 +1,20 @@
 from sympy import E as e
 from sympy import (Symbol, Abs, exp, expint, S, pi, simplify, Interval, erf, erfc, Ne,
                    EulerGamma, Eq, log, lowergamma, uppergamma, symbols, sqrt, And,
-                   gamma, beta, Piecewise, Integral, sin, cos, tan, sinh, cosh,
+                   gamma, beta, Piecewise, Integral, sin, cos, tan, atan, sinh, cosh,
                    besseli, floor, expand_func, Rational, I, re, Lambda, asin,
                    im, lambdify, hyper, diff, Or, Mul, sign, Dummy, Sum,
                    factorial, binomial, erfi, besselj, besselk)
-from sympy.external import import_module
 from sympy.functions.special.error_functions import erfinv
 from sympy.functions.special.hyper import meijerg
 from sympy.sets.sets import Intersection, FiniteSet
 from sympy.stats import (P, E, where, density, variance, covariance, skewness, kurtosis, median,
                          given, pspace, cdf, characteristic_function, moment_generating_function,
-                         ContinuousRV, sample, Arcsin, Benini, Beta, BetaNoncentral, BetaPrime,
+                         ContinuousRV, Arcsin, Benini, Beta, BetaNoncentral, BetaPrime,
                          Cauchy, Chi, ChiSquared, ChiNoncentral, Dagum, Erlang, ExGaussian,
                          Exponential, ExponentialPower, FDistribution, FisherZ, Frechet, Gamma,
-                         GammaInverse, Gompertz, Gumbel, Kumaraswamy, Laplace, Levy, Logistic,
-                         LogLogistic, LogNormal, Maxwell, Moyal, Nakagami, Normal, GaussianInverse,
+                         GammaInverse, Gompertz, Gumbel, Kumaraswamy, Laplace, Levy, Logistic, LogCauchy,
+                         LogLogistic, LogitNormal, LogNormal, Maxwell, Moyal, Nakagami, Normal, GaussianInverse,
                          Pareto, PowerFunction, QuadraticU, RaisedCosine, Rayleigh, Reciprocal, ShiftedGompertz, StudentT,
                          Trapezoidal, Triangular, Uniform, UniformSum, VonMises, Weibull, coskewness,
                          WignerSemicircle, Wald, correlation, moment, cmoment, smoment, quantile,
@@ -26,7 +25,7 @@ from sympy.stats.joint_rv_types import MultivariateLaplaceDistribution, Multivar
 from sympy.stats.crv import SingleContinuousPSpace, SingleContinuousDomain
 from sympy.stats.compound_rv import CompoundPSpace
 from sympy.stats.symbolic_probability import Probability
-from sympy.testing.pytest import raises, XFAIL, slow, skip, ignore_warnings
+from sympy.testing.pytest import raises, XFAIL, slow, ignore_warnings
 from sympy.testing.randtest import verify_numerically as tn
 
 oo = S.Infinity
@@ -321,33 +320,6 @@ def test_moment_generating_function():
         besseli(0, 1)
 
 
-def test_sample_continuous():
-    Z = ContinuousRV(z, exp(-z), set=Interval(0, oo))
-    assert density(Z)(-1) == 0
-
-    scipy = import_module('scipy')
-    if not scipy:
-        skip('Scipy is not installed. Abort tests')
-    with ignore_warnings(UserWarning): ### TODO: Restore tests once warnings are removed
-        assert next(sample(Z)) in Z.pspace.domain.set
-    sym, val = list(Z.pspace.sample().items())[0]
-    assert sym == Z and val in Interval(0, oo)
-
-    libraries = ['scipy', 'numpy', 'pymc3']
-    for lib in libraries:
-        try:
-            imported_lib = import_module(lib)
-            if imported_lib:
-                s0, s1, s2 = [], [], []
-                s0 = list(sample(Z, numsamples=10, library=lib, seed=0))
-                s1 = list(sample(Z, numsamples=10, library=lib, seed=0))
-                s2 = list(sample(Z, numsamples=10, library=lib, seed=1))
-                assert s0 == s1
-                assert s1 != s2
-        except NotImplementedError:
-            continue
-
-
 def test_ContinuousRV():
     pdf = sqrt(2)*exp(-x**2/2)/(2*sqrt(pi))  # Normal distribution
     # X and Y should be equivalent
@@ -777,14 +749,6 @@ def test_gamma_inverse():
             * besselk(a, 2*sqrt(b)*sqrt(-I*x))/gamma(a)
     raises(NotImplementedError, lambda: moment_generating_function(X))
 
-def test_sampling_gamma_inverse():
-    scipy = import_module('scipy')
-    if not scipy:
-        skip('Scipy not installed. Abort tests for sampling of gamma inverse.')
-    X = GammaInverse("x", 1, 1)
-    with ignore_warnings(UserWarning): ### TODO: Restore tests once warnings are removed
-        assert next(sample(X)) in X.pspace.domain.set
-
 def test_gompertz():
     b = Symbol("b", positive=True)
     eta = Symbol("eta", positive=True)
@@ -856,6 +820,16 @@ def test_levy():
     mu = Symbol("mu", real=True)
     raises(ValueError, lambda: Levy('x',mu,c))
 
+def test_logcauchy():
+    mu = Symbol("mu" , positive=True)
+    sigma = Symbol("sigma" , positive=True)
+
+    X = LogCauchy("x", mu, sigma)
+
+    assert density(X)(x) == sigma/(x*pi*(sigma**2 + (-mu + log(x))**2))
+    assert cdf(X)(x) == atan((log(x) - mu)/sigma)/pi + S.Half
+
+
 def test_logistic():
     mu = Symbol("mu", real=True)
     s = Symbol("s", positive=True)
@@ -897,6 +871,15 @@ def test_loglogistic():
     X = LogLogistic('x', 1, 2)
     assert median(X) == FiniteSet(1)
 
+def test_logitnormal():
+    mu = Symbol('mu', real=True)
+    s = Symbol('s', positive=True)
+    X = LogitNormal('x', mu, s)
+    x = Symbol('x')
+
+    assert density(X)(x) == sqrt(2)*exp(-(-mu + log(x/(1 - x)))**2/(2*s**2))/(2*sqrt(pi)*s*x*(1 - x))
+    assert cdf(X)(x) == erf(sqrt(2)*(-mu + log(x/(1 - x)))/(2*s))/2 + S(1)/2
+
 def test_lognormal():
     mean = Symbol('mu', real=True)
     std = Symbol('sigma', positive=True)
@@ -905,20 +888,6 @@ def test_lognormal():
     #assert E(X) == exp(mean+std**2/2)
     #assert variance(X) == (exp(std**2)-1) * exp(2*mean + std**2)
 
-    # Right now, only density function and sampling works
-    scipy = import_module('scipy')
-    if not scipy:
-        skip('Scipy is not installed. Abort tests')
-    with ignore_warnings(UserWarning): ### TODO: Restore tests once warnings are removed
-        for i in range(3):
-            X = LogNormal('x', i, 1)
-            assert next(sample(X)) in X.pspace.domain.set
-
-    size = 5
-    with ignore_warnings(UserWarning): ### TODO: Restore tests once warnings are removed
-        samps = next(sample(X, size=size))
-        for samp in samps:
-            assert samp in X.pspace.domain.set
     # The sympy integrator can't do this too well
     #assert E(X) ==
     raises(NotImplementedError, lambda: moment_generating_function(X))
@@ -1033,14 +1002,6 @@ def test_gaussian_inverse():
     b = symbols('b', nonpositive=True)
     raises(ValueError, lambda: GaussianInverse('x', a, b))
 
-def test_sampling_gaussian_inverse():
-    scipy = import_module('scipy')
-    if not scipy:
-        skip('Scipy not installed. Abort tests for sampling of Gaussian inverse.')
-    X = GaussianInverse("x", 1, 1)
-    with ignore_warnings(UserWarning): ### TODO: Restore tests once warnings are removed
-        assert next(sample(X, library='scipy')) in X.pspace.domain.set
-
 def test_pareto():
     xm, beta = symbols('xm beta', positive=True)
     alpha = beta + 5
@@ -1324,30 +1285,6 @@ def test_wignersemicircle():
            Piecewise((2*besselj(1, R*x)/(R*x), Ne(x, 0)), (1, True))
 
 
-def test_prefab_sampling():
-    scipy = import_module('scipy')
-    if not scipy:
-        skip('Scipy is not installed. Abort tests')
-    N = Normal('X', 0, 1)
-    L = LogNormal('L', 0, 1)
-    E = Exponential('Ex', 1)
-    P = Pareto('P', 1, 3)
-    W = Weibull('W', 1, 1)
-    U = Uniform('U', 0, 1)
-    B = Beta('B', 2, 5)
-    G = Gamma('G', 1, 3)
-
-    variables = [N, L, E, P, W, U, B, G]
-    niter = 10
-    size = 5
-    with ignore_warnings(UserWarning): ### TODO: Restore tests once warnings are removed
-        for var in variables:
-            for _ in range(niter):
-                assert next(sample(var)) in var.pspace.domain.set
-                samps = next(sample(var, size=size))
-                for samp in samps:
-                    assert samp in var.pspace.domain.set
-
 def test_input_value_assertions():
     a, b = symbols('a b')
     p, q = symbols('p q', positive=True)
@@ -1539,6 +1476,13 @@ def test_issue_13324():
     assert E(X, X > S.Half) == Rational(3, 4)
     assert E(X, X > 0) == S.Half
 
+def test_issue_20756():
+    X = Uniform('X', -1, +1)
+    Y = Uniform('Y', -1, +1)
+    assert E(X * Y) == S.Zero
+    assert E(X * ((Y + 1) - 1)) == S.Zero
+    assert E(Y * (X*(X + 1) - X*X)) == S.Zero
+
 def test_FiniteSet_prob():
     E = Exponential('E', 3)
     N = Normal('N', 5, 7)
@@ -1591,95 +1535,7 @@ def test_ContinuousDistributionHandmade():
     assert variance(space.value) == Rational(13, 12)
 
 
-def test_sample_numpy():
-    distribs_numpy = [
-        Beta("B", 1, 1),
-        Normal("N", 0, 1),
-        Gamma("G", 2, 7),
-        Exponential("E", 2),
-        LogNormal("LN", 0, 1),
-        Pareto("P", 1, 1),
-        ChiSquared("CS", 2),
-        Uniform("U", 0, 1)
-    ]
-    size = 3
-    numpy = import_module('numpy')
-    if not numpy:
-        skip('Numpy is not installed. Abort tests for _sample_numpy.')
-    else:
-        with ignore_warnings(UserWarning): ### TODO: Restore tests once warnings are removed
-            for X in distribs_numpy:
-                samps = next(sample(X, size=size, library='numpy'))
-                for sam in samps:
-                    assert sam in X.pspace.domain.set
-            raises(NotImplementedError,
-                lambda: next(sample(Chi("C", 1), library='numpy')))
-    raises(NotImplementedError,
-            lambda: Chi("C", 1).pspace.distribution.sample(library='tensorflow'))
-
-def test_sample_scipy():
-    distribs_scipy = [
-        Beta("B", 1, 1),
-        BetaPrime("BP", 1, 1),
-        Cauchy("C", 1, 1),
-        Chi("C", 1),
-        Normal("N", 0, 1),
-        Gamma("G", 2, 7),
-        GammaInverse("GI", 1, 1),
-        GaussianInverse("GUI", 1, 1),
-        Exponential("E", 2),
-        LogNormal("LN", 0, 1),
-        Pareto("P", 1, 1),
-        StudentT("S", 2),
-        ChiSquared("CS", 2),
-        Uniform("U", 0, 1)
-    ]
-    size = 3
-    numsamples = 5
-    scipy = import_module('scipy')
-    if not scipy:
-        skip('Scipy is not installed. Abort tests for _sample_scipy.')
-    else:
-        with ignore_warnings(UserWarning): ### TODO: Restore tests once warnings are removed
-            g_sample = list(sample(Gamma("G", 2, 7), size=size, numsamples=numsamples))
-            assert len(g_sample) == numsamples
-            for X in distribs_scipy:
-                samps = next(sample(X, size=size, library='scipy'))
-                samps2 = next(sample(X, size=(2, 2), library='scipy'))
-                for sam in samps:
-                    assert sam in X.pspace.domain.set
-                for i in range(2):
-                    for j in range(2):
-                        assert samps2[i][j] in X.pspace.domain.set
-
-def test_sample_pymc3():
-    distribs_pymc3 = [
-        Beta("B", 1, 1),
-        Cauchy("C", 1, 1),
-        Normal("N", 0, 1),
-        Gamma("G", 2, 7),
-        GaussianInverse("GI", 1, 1),
-        Exponential("E", 2),
-        LogNormal("LN", 0, 1),
-        Pareto("P", 1, 1),
-        ChiSquared("CS", 2),
-        Uniform("U", 0, 1)
-    ]
-    size = 3
-    pymc3 = import_module('pymc3')
-    if not pymc3:
-        skip('PyMC3 is not installed. Abort tests for _sample_pymc3.')
-    else:
-        with ignore_warnings(UserWarning): ### TODO: Restore tests once warnings are removed
-            for X in distribs_pymc3:
-                samps = next(sample(X, size=size, library='pymc3'))
-                for sam in samps:
-                    assert sam in X.pspace.domain.set
-            raises(NotImplementedError,
-                lambda: next(sample(Chi("C", 1), library='pymc3')))
-
-
 def test_issue_16318():
-    #test compute_expectation function of the SingleContinuousDomain
+    # test compute_expectation function of the SingleContinuousDomain
     N = SingleContinuousDomain(x, Interval(0, 1))
-    raises (ValueError, lambda: SingleContinuousDomain.compute_expectation(N, x+1, {x, y}))
+    raises(ValueError, lambda: SingleContinuousDomain.compute_expectation(N, x+1, {x, y}))
