@@ -18,73 +18,72 @@ matrix element ``M[i, j]`` as in the following diagram::
            The stem used by itself is usually taken to represent the entire
            array.
 
-    There can be any number of indices on an Indexed object.  No
-    transformation properties are implemented in these Base objects, but
-    implicit contraction of repeated indices is supported.
+There can be any number of indices on an Indexed object.  No
+transformation properties are implemented in these Base objects, but
+implicit contraction of repeated indices is supported.
 
-    Note that the support for complicated (i.e. non-atomic) integer
-    expressions as indices is limited.  (This should be improved in
-    future releases.)
+Note that the support for complicated (i.e. non-atomic) integer
+expressions as indices is limited.  (This should be improved in
+future releases.)
 
-    Examples
-    ========
+Examples
+========
 
-    To express the above matrix element example you would write:
+To express the above matrix element example you would write:
 
-    >>> from sympy import symbols, IndexedBase, Idx
-    >>> M = IndexedBase('M')
-    >>> i, j = symbols('i j', cls=Idx)
-    >>> M[i, j]
-    M[i, j]
+>>> from sympy import symbols, IndexedBase, Idx
+>>> M = IndexedBase('M')
+>>> i, j = symbols('i j', cls=Idx)
+>>> M[i, j]
+M[i, j]
 
-    Repeated indices in a product implies a summation, so to express a
-    matrix-vector product in terms of Indexed objects:
+Repeated indices in a product implies a summation, so to express a
+matrix-vector product in terms of Indexed objects:
 
-    >>> x = IndexedBase('x')
-    >>> M[i, j]*x[j]
-    M[i, j]*x[j]
+>>> x = IndexedBase('x')
+>>> M[i, j]*x[j]
+M[i, j]*x[j]
 
-    If the indexed objects will be converted to component based arrays, e.g.
-    with the code printers or the autowrap framework, you also need to provide
-    (symbolic or numerical) dimensions.  This can be done by passing an
-    optional shape parameter to IndexedBase upon construction:
+If the indexed objects will be converted to component based arrays, e.g.
+with the code printers or the autowrap framework, you also need to provide
+(symbolic or numerical) dimensions.  This can be done by passing an
+optional shape parameter to IndexedBase upon construction:
 
-    >>> dim1, dim2 = symbols('dim1 dim2', integer=True)
-    >>> A = IndexedBase('A', shape=(dim1, 2*dim1, dim2))
-    >>> A.shape
-    (dim1, 2*dim1, dim2)
-    >>> A[i, j, 3].shape
-    (dim1, 2*dim1, dim2)
+>>> dim1, dim2 = symbols('dim1 dim2', integer=True)
+>>> A = IndexedBase('A', shape=(dim1, 2*dim1, dim2))
+>>> A.shape
+(dim1, 2*dim1, dim2)
+>>> A[i, j, 3].shape
+(dim1, 2*dim1, dim2)
 
-    If an IndexedBase object has no shape information, it is assumed that the
-    array is as large as the ranges of its indices:
+If an IndexedBase object has no shape information, it is assumed that the
+array is as large as the ranges of its indices:
 
-    >>> n, m = symbols('n m', integer=True)
-    >>> i = Idx('i', m)
-    >>> j = Idx('j', n)
-    >>> M[i, j].shape
-    (m, n)
-    >>> M[i, j].ranges
-    [(0, m - 1), (0, n - 1)]
+>>> n, m = symbols('n m', integer=True)
+>>> i = Idx('i', m)
+>>> j = Idx('j', n)
+>>> M[i, j].shape
+(m, n)
+>>> M[i, j].ranges
+[(0, m - 1), (0, n - 1)]
 
-    The above can be compared with the following:
+The above can be compared with the following:
 
-    >>> A[i, 2, j].shape
-    (dim1, 2*dim1, dim2)
-    >>> A[i, 2, j].ranges
-    [(0, m - 1), None, (0, n - 1)]
+>>> A[i, 2, j].shape
+(dim1, 2*dim1, dim2)
+>>> A[i, 2, j].ranges
+[(0, m - 1), None, (0, n - 1)]
 
-    To analyze the structure of indexed expressions, you can use the methods
-    get_indices() and get_contraction_structure():
+To analyze the structure of indexed expressions, you can use the methods
+get_indices() and get_contraction_structure():
 
-    >>> from sympy.tensor import get_indices, get_contraction_structure
-    >>> get_indices(A[i, j, j])
-    ({i}, {})
-    >>> get_contraction_structure(A[i, j, j])
-    {(j,): {A[i, j, j]}}
-
-    See the appropriate docstrings for a detailed explanation of the output.
+>>> from sympy.tensor import get_indices, get_contraction_structure
+>>> get_indices(A[i, j, j])
+({i}, {})
+>>> get_contraction_structure(A[i, j, j])
+{(j,): {A[i, j, j]}}
 
+See the appropriate docstrings for a detailed explanation of the output.
 """
 
 #   TODO:  (some ideas for improvement)
@@ -104,14 +103,17 @@ matrix element ``M[i, j]`` as in the following diagram::
 #      - Idx with range smaller than dimension of Indexed
 #      - Idx with stepsize != 1
 #      - Idx with step determined by function call
+from collections.abc import Iterable
 
-from __future__ import print_function, division
-
+from sympy import Number
+from sympy.core.assumptions import StdFactKB
+from sympy.core import Expr, Tuple, sympify, S
+from sympy.core.symbol import _filter_assumptions, Symbol
+from sympy.core.compatibility import (is_sequence, NotIterable)
+from sympy.core.logic import fuzzy_bool, fuzzy_not
 from sympy.core.sympify import _sympify
 from sympy.functions.special.tensor_functions import KroneckerDelta
-from sympy.core import Expr, Tuple, Symbol, sympify, S
-from sympy.core.compatibility import (is_sequence, string_types, NotIterable,
-    range, Iterable)
+from sympy.multipledispatch import dispatch
 
 
 class IndexException(Exception):
@@ -126,10 +128,13 @@ class Indexed(Expr):
     >>> Indexed('A', i, j)
     A[i, j]
 
-    It is recommended that ``Indexed`` objects be created via ``IndexedBase``:
+    It is recommended that ``Indexed`` objects be created by indexing ``IndexedBase``:
+    ``IndexedBase('A')[i, j]`` instead of ``Indexed(IndexedBase('A'), i, j)``.
 
     >>> A = IndexedBase('A')
-    >>> Indexed('A', i, j) == A[i, j]
+    >>> a_ij = A[i, j]           # Prefer this,
+    >>> b_ij = Indexed(A, i, j)  # over this.
+    >>> a_ij == b_ij
     True
 
     """
@@ -145,11 +150,14 @@ class Indexed(Expr):
 
         if not args:
             raise IndexException("Indexed needs at least one index.")
-        if isinstance(base, (string_types, Symbol)):
+        if isinstance(base, (str, Symbol)):
             base = IndexedBase(base)
         elif not hasattr(base, '__getitem__') and not isinstance(base, IndexedBase):
             raise TypeError(filldedent("""
-                Indexed expects string, Symbol, or IndexedBase as base."""))
+                The base can only be replaced with a string, Symbol,
+                IndexedBase or an object with a method for getting
+                items (i.e. an object with a `__getitem__` method).
+                """))
         args = list(map(sympify, args))
         if isinstance(base, (NDimArray, Iterable, Tuple, MatrixBase)) and all([i.is_number for i in args]):
             if len(args) == 1:
@@ -157,7 +165,20 @@ class Indexed(Expr):
             else:
                 return base[args]
 
-        return Expr.__new__(cls, base, *args, **kw_args)
+        obj = Expr.__new__(cls, base, *args, **kw_args)
+
+        try:
+            IndexedBase._set_assumptions(obj, base.assumptions0)
+        except AttributeError:
+            IndexedBase._set_assumptions(obj, {})
+        return obj
+
+    def _hashable_content(self):
+        return super()._hashable_content() + tuple(sorted(self.assumptions0.items()))
+
+    @property
+    def name(self):
+        return str(self)
 
     @property
     def _diff_wrt(self):
@@ -184,6 +205,10 @@ class Indexed(Expr):
                 return S.NaN
             return S.Zero
 
+    @property
+    def assumptions0(self):
+        return {k: v for k, v in self._assumptions.items() if v is not None}
+
     @property
     def base(self):
         """Returns the ``IndexedBase`` of the ``Indexed`` object.
@@ -262,15 +287,21 @@ class Indexed(Expr):
 
         if self.base.shape:
             return self.base.shape
-        try:
-            return Tuple(*[i.upper - i.lower + 1 for i in self.indices])
-        except AttributeError:
-            raise IndexException(filldedent("""
-                Range is not defined for all indices in: %s""" % self))
-        except TypeError:
-            raise IndexException(filldedent("""
-                Shape cannot be inferred from Idx with
-                undefined range: %s""" % self))
+        sizes = []
+        for i in self.indices:
+            upper = getattr(i, 'upper', None)
+            lower = getattr(i, 'lower', None)
+            if None in (upper, lower):
+                raise IndexException(filldedent("""
+                    Range is not defined for all indices in: %s""" % self))
+            try:
+                size = upper - lower + 1
+            except TypeError:
+                raise IndexException(filldedent("""
+                    Shape cannot be inferred from Idx with
+                    undefined range: %s""" % self))
+            sizes.append(size)
+        return Tuple(*sizes)
 
     @property
     def ranges(self):
@@ -294,9 +325,12 @@ class Indexed(Expr):
         """
         ranges = []
         for i in self.indices:
-            try:
-                ranges.append(Tuple(i.lower, i.upper))
-            except AttributeError:
+            sentinel = object()
+            upper = getattr(i, 'upper', sentinel)
+            lower = getattr(i, 'lower', sentinel)
+            if sentinel not in (upper, lower):
+                ranges.append(Tuple(lower, upper))
+            else:
                 ranges.append(None)
         return ranges
 
@@ -369,18 +403,43 @@ class IndexedBase(Expr, NotIterable):
     >>> B[i, j].shape
     (o, p)
 
+    Assumptions can be specified with keyword arguments the same way as for Symbol:
+
+    >>> A_real = IndexedBase('A', real=True)
+    >>> A_real.is_real
+    True
+    >>> A != A_real
+    True
+
+    Assumptions can also be inherited if a Symbol is used to initialize the IndexedBase:
+
+    >>> I = symbols('I', integer=True)
+    >>> C_inherit = IndexedBase(I)
+    >>> C_explicit = IndexedBase('I', integer=True)
+    >>> C_inherit == C_explicit
+    True
     """
     is_commutative = True
     is_symbol = True
     is_Atom = True
 
-    def __new__(cls, label, shape=None, **kw_args):
+    @staticmethod
+    def _set_assumptions(obj, assumptions):
+        """Set assumptions on obj, making sure to apply consistent values."""
+        tmp_asm_copy = assumptions.copy()
+        is_commutative = fuzzy_bool(assumptions.get('commutative', True))
+        assumptions['commutative'] = is_commutative
+        obj._assumptions = StdFactKB(assumptions)
+        obj._assumptions._generator = tmp_asm_copy  # Issue #8873
+
+    def __new__(cls, label, shape=None, *, offset=S.Zero, strides=None, **kw_args):
         from sympy import MatrixBase, NDimArray
 
-        if isinstance(label, string_types):
-            label = Symbol(label)
+        assumptions, kw_args = _filter_assumptions(kw_args)
+        if isinstance(label, str):
+            label = Symbol(label, **assumptions)
         elif isinstance(label, Symbol):
-            pass
+            assumptions = label._merge(assumptions)
         elif isinstance(label, (MatrixBase, NDimArray)):
             return label
         elif isinstance(label, Iterable):
@@ -393,18 +452,29 @@ class IndexedBase(Expr, NotIterable):
         elif shape is not None:
             shape = Tuple(shape)
 
-        offset = kw_args.pop('offset', S.Zero)
-        strides = kw_args.pop('strides', None)
-
         if shape is not None:
-            obj = Expr.__new__(cls, label, shape, **kw_args)
+            obj = Expr.__new__(cls, label, shape)
         else:
-            obj = Expr.__new__(cls, label, **kw_args)
+            obj = Expr.__new__(cls, label)
         obj._shape = shape
         obj._offset = offset
         obj._strides = strides
+        obj._name = str(label)
+
+        IndexedBase._set_assumptions(obj, assumptions)
         return obj
 
+    @property
+    def name(self):
+        return self._name
+
+    def _hashable_content(self):
+        return super()._hashable_content() + tuple(sorted(self.assumptions0.items()))
+
+    @property
+    def assumptions0(self):
+        return {k: v for k, v in self._assumptions.items() if v is not None}
+
     def __getitem__(self, indices, **kw_args):
         if is_sequence(indices):
             # Special case needed because M[*my_tuple] is a syntax error.
@@ -423,7 +493,7 @@ class IndexedBase(Expr, NotIterable):
         Examples
         ========
 
-        >>> from sympy import IndexedBase, Idx, Symbol
+        >>> from sympy import IndexedBase, Idx
         >>> from sympy.abc import x, y
         >>> IndexedBase('A', shape=(x, y)).shape
         (x, y)
@@ -528,7 +598,7 @@ class Idx(Expr):
     Examples
     ========
 
-    >>> from sympy import IndexedBase, Idx, symbols, oo
+    >>> from sympy import Idx, symbols, oo
     >>> n, i, L, U = symbols('n i L U', integer=True)
 
     If a string is given for the label an integer ``Symbol`` is created and the
@@ -568,7 +638,7 @@ class Idx(Expr):
     def __new__(cls, label, range=None, **kw_args):
         from sympy.utilities.misc import filldedent
 
-        if isinstance(label, string_types):
+        if isinstance(label, str):
             label = Symbol(label, integer=True)
         label, range = list(map(sympify, (label, range)))
 
@@ -585,11 +655,12 @@ class Idx(Expr):
                 raise ValueError(filldedent("""
                     Idx range tuple must have length 2, but got %s""" % len(range)))
             for bound in range:
-                if bound.is_integer is False:
+                if (bound.is_integer is False and bound is not S.Infinity
+                        and bound is not S.NegativeInfinity):
                     raise TypeError("Idx object requires integer bounds.")
             args = label, Tuple(*range)
         elif isinstance(range, Expr):
-            if not (range.is_integer or range is S.Infinity):
+            if range is not S.Infinity and fuzzy_not(range.is_integer):
                 raise TypeError("Idx object requires an integer dimension.")
             args = label, Tuple(0, range - 1)
         elif range:
@@ -669,62 +740,49 @@ class Idx(Expr):
     def _sympystr(self, p):
         return p.doprint(self.label)
 
+    @property
+    def name(self):
+        return self.label.name if self.label.is_Symbol else str(self.label)
+
     @property
     def free_symbols(self):
         return {self}
 
-    def __le__(self, other):
-        if isinstance(other, Idx):
-            other_upper = other if other.upper is None else other.upper
-            other_lower = other if other.lower is None else other.lower
-        else:
-            other_upper = other
-            other_lower = other
 
-        if self.upper is not None and (self.upper <= other_lower) == True:
-            return True
-        if self.lower is not None and (self.lower > other_upper) == True:
-            return False
-        return super(Idx, self).__le__(other)
+@dispatch(Idx, Idx)
+def _eval_is_ge(lhs, rhs): # noqa:F811
 
-    def __ge__(self, other):
-        if isinstance(other, Idx):
-            other_upper = other if other.upper is None else other.upper
-            other_lower = other if other.lower is None else other.lower
-        else:
-            other_upper = other
-            other_lower = other
+    other_upper = rhs if rhs.upper is None else rhs.upper
+    other_lower = rhs if rhs.lower is None else rhs.lower
 
-        if self.lower is not None and (self.lower >= other_upper) == True:
-            return True
-        if self.upper is not None and (self.upper < other_lower) == True:
-            return False
-        return super(Idx, self).__ge__(other)
+    if lhs.lower is not None and (lhs.lower >= other_upper) == True:
+        return True
+    if lhs.upper is not None and (lhs.upper < other_lower) == True:
+        return False
+    return None
 
-    def __lt__(self, other):
-        if isinstance(other, Idx):
-            other_upper = other if other.upper is None else other.upper
-            other_lower = other if other.lower is None else other.lower
-        else:
-            other_upper = other
-            other_lower = other
 
-        if self.upper is not None and (self.upper < other_lower) == True:
-            return True
-        if self.lower is not None and (self.lower >= other_upper) == True:
-            return False
-        return super(Idx, self).__lt__(other)
+@dispatch(Idx, Number)  # type:ignore
+def _eval_is_ge(lhs, rhs): # noqa:F811
 
-    def __gt__(self, other):
-        if isinstance(other, Idx):
-            other_upper = other if other.upper is None else other.upper
-            other_lower = other if other.lower is None else other.lower
-        else:
-            other_upper = other
-            other_lower = other
+    other_upper = rhs
+    other_lower = rhs
 
-        if self.lower is not None and (self.lower > other_upper) == True:
-            return True
-        if self.upper is not None and (self.upper <= other_lower) == True:
-            return False
-        return super(Idx, self).__gt__(other)
+    if lhs.lower is not None and (lhs.lower >= other_upper) == True:
+        return True
+    if lhs.upper is not None and (lhs.upper < other_lower) == True:
+        return False
+    return None
+
+
+@dispatch(Number, Idx)  # type:ignore
+def _eval_is_ge(lhs, rhs): # noqa:F811
+
+    other_upper = lhs
+    other_lower = lhs
+
+    if rhs.upper is not None and (rhs.upper <= other_lower) == True:
+        return True
+    if rhs.lower is not None and (rhs.lower > other_upper) == True:
+        return False
+    return None
