@@ -9,24 +9,24 @@ from sympy import get_contraction_structure, get_indices
 
 def test_trivial_indices():
     x, y = symbols('x y')
-    assert get_indices(x) == (set([]), {})
-    assert get_indices(x*y) == (set([]), {})
-    assert get_indices(x + y) == (set([]), {})
-    assert get_indices(x**y) == (set([]), {})
+    assert get_indices(x) == (set(), {})
+    assert get_indices(x*y) == (set(), {})
+    assert get_indices(x + y) == (set(), {})
+    assert get_indices(x**y) == (set(), {})
 
 
 def test_get_indices_Indexed():
     x = IndexedBase('x')
     i, j = Idx('i'), Idx('j')
-    assert get_indices(x[i, j]) == (set([i, j]), {})
-    assert get_indices(x[j, i]) == (set([j, i]), {})
+    assert get_indices(x[i, j]) == ({i, j}, {})
+    assert get_indices(x[j, i]) == ({j, i}, {})
 
 
 def test_get_indices_Idx():
     f = Function('f')
     i, j = Idx('i'), Idx('j')
-    assert get_indices(f(i)*j) == (set([i, j]), {})
-    assert get_indices(f(j, i)) == (set([j, i]), {})
+    assert get_indices(f(i)*j) == ({i, j}, {})
+    assert get_indices(f(j, i)) == ({j, i}, {})
     assert get_indices(f(i)*i) == (set(), {})
 
 
@@ -34,8 +34,8 @@ def test_get_indices_mul():
     x = IndexedBase('x')
     y = IndexedBase('y')
     i, j = Idx('i'), Idx('j')
-    assert get_indices(x[j]*y[i]) == (set([i, j]), {})
-    assert get_indices(x[i]*y[j]) == (set([i, j]), {})
+    assert get_indices(x[j]*y[i]) == ({i, j}, {})
+    assert get_indices(x[i]*y[j]) == ({i, j}, {})
 
 
 def test_get_indices_exceptions():
@@ -49,8 +49,8 @@ def test_scalar_broadcast():
     x = IndexedBase('x')
     y = IndexedBase('y')
     i, j = Idx('i'), Idx('j')
-    assert get_indices(x[i] + y[i, i]) == (set([i]), {})
-    assert get_indices(x[i] + y[j, j]) == (set([i]), {})
+    assert get_indices(x[i] + y[i, i]) == ({i}, {})
+    assert get_indices(x[i] + y[j, j]) == ({i}, {})
 
 
 def test_get_indices_add():
@@ -58,12 +58,12 @@ def test_get_indices_add():
     y = IndexedBase('y')
     A = IndexedBase('A')
     i, j, k = Idx('i'), Idx('j'), Idx('k')
-    assert get_indices(x[i] + 2*y[i]) == (set([i, ]), {})
-    assert get_indices(y[i] + 2*A[i, j]*x[j]) == (set([i, ]), {})
-    assert get_indices(y[i] + 2*(x[i] + A[i, j]*x[j])) == (set([i, ]), {})
-    assert get_indices(y[i] + x[i]*(A[j, j] + 1)) == (set([i, ]), {})
+    assert get_indices(x[i] + 2*y[i]) == ({i}, {})
+    assert get_indices(y[i] + 2*A[i, j]*x[j]) == ({i}, {})
+    assert get_indices(y[i] + 2*(x[i] + A[i, j]*x[j])) == ({i}, {})
+    assert get_indices(y[i] + x[i]*(A[j, j] + 1)) == ({i}, {})
     assert get_indices(
-        y[i] + x[i]*x[j]*(y[j] + A[j, k]*x[k])) == (set([i, ]), {})
+        y[i] + x[i]*x[j]*(y[j] + A[j, k]*x[k])) == ({i}, {})
 
 
 def test_get_indices_Pow():
@@ -71,25 +71,25 @@ def test_get_indices_Pow():
     y = IndexedBase('y')
     A = IndexedBase('A')
     i, j, k = Idx('i'), Idx('j'), Idx('k')
-    assert get_indices(Pow(x[i], y[j])) == (set([i, j]), {})
-    assert get_indices(Pow(x[i, k], y[j, k])) == (set([i, j, k]), {})
-    assert get_indices(Pow(A[i, k], y[k] + A[k, j]*x[j])) == (set([i, k]), {})
+    assert get_indices(Pow(x[i], y[j])) == ({i, j}, {})
+    assert get_indices(Pow(x[i, k], y[j, k])) == ({i, j, k}, {})
+    assert get_indices(Pow(A[i, k], y[k] + A[k, j]*x[j])) == ({i, k}, {})
     assert get_indices(Pow(2, x[i])) == get_indices(exp(x[i]))
 
     # test of a design decision, this may change:
-    assert get_indices(Pow(x[i], 2)) == (set([i, ]), {})
+    assert get_indices(Pow(x[i], 2)) == ({i}, {})
 
 
 def test_get_contraction_structure_basic():
     x = IndexedBase('x')
     y = IndexedBase('y')
     i, j = Idx('i'), Idx('j')
-    assert get_contraction_structure(x[i]*y[j]) == {None: set([x[i]*y[j]])}
-    assert get_contraction_structure(x[i] + y[j]) == {None: set([x[i], y[j]])}
-    assert get_contraction_structure(x[i]*y[i]) == {(i,): set([x[i]*y[i]])}
+    assert get_contraction_structure(x[i]*y[j]) == {None: {x[i]*y[j]}}
+    assert get_contraction_structure(x[i] + y[j]) == {None: {x[i], y[j]}}
+    assert get_contraction_structure(x[i]*y[i]) == {(i,): {x[i]*y[i]}}
     assert get_contraction_structure(
-        1 + x[i]*y[i]) == {None: set([S.One]), (i,): set([x[i]*y[i]])}
-    assert get_contraction_structure(x[i]**y[i]) == {None: set([x[i]**y[i]])}
+        1 + x[i]*y[i]) == {None: {S.One}, (i,): {x[i]*y[i]}}
+    assert get_contraction_structure(x[i]**y[i]) == {None: {x[i]**y[i]}}
 
 
 def test_get_contraction_structure_complex():
@@ -98,10 +98,10 @@ def test_get_contraction_structure_complex():
     A = IndexedBase('A')
     i, j, k = Idx('i'), Idx('j'), Idx('k')
     expr1 = y[i] + A[i, j]*x[j]
-    d1 = {None: set([y[i]]), (j,): set([A[i, j]*x[j]])}
+    d1 = {None: {y[i]}, (j,): {A[i, j]*x[j]}}
     assert get_contraction_structure(expr1) == d1
     expr2 = expr1*A[k, i] + x[k]
-    d2 = {None: set([x[k]]), (i,): set([expr1*A[k, i]]), expr1*A[k, i]: [d1]}
+    d2 = {None: {x[k]}, (i,): {expr1*A[k, i]}, expr1*A[k, i]: [d1]}
     assert get_contraction_structure(expr2) == d2
 
 
@@ -111,18 +111,18 @@ def test_contraction_structure_simple_Pow():
     i, j, k = Idx('i'), Idx('j'), Idx('k')
     ii_jj = x[i, i]**y[j, j]
     assert get_contraction_structure(ii_jj) == {
-        None: set([ii_jj]),
+        None: {ii_jj},
         ii_jj: [
-            {(i,): set([x[i, i]])},
-            {(j,): set([y[j, j]])}
+            {(i,): {x[i, i]}},
+            {(j,): {y[j, j]}}
         ]
     }
 
     ii_jk = x[i, i]**y[j, k]
     assert get_contraction_structure(ii_jk) == {
-        None: set([x[i, i]**y[j, k]]),
+        None: {x[i, i]**y[j, k]},
         x[i, i]**y[j, k]: [
-            {(i,): set([x[i, i]])}
+            {(i,): {x[i, i]}}
         ]
     }
 
@@ -133,22 +133,22 @@ def test_contraction_structure_Mul_and_Pow():
     i, j, k = Idx('i'), Idx('j'), Idx('k')
 
     i_ji = x[i]**(y[j]*x[i])
-    assert get_contraction_structure(i_ji) == {None: set([i_ji])}
+    assert get_contraction_structure(i_ji) == {None: {i_ji}}
     ij_i = (x[i]*y[j])**(y[i])
-    assert get_contraction_structure(ij_i) == {None: set([ij_i])}
+    assert get_contraction_structure(ij_i) == {None: {ij_i}}
     j_ij_i = x[j]*(x[i]*y[j])**(y[i])
-    assert get_contraction_structure(j_ij_i) == {(j,): set([j_ij_i])}
+    assert get_contraction_structure(j_ij_i) == {(j,): {j_ij_i}}
     j_i_ji = x[j]*x[i]**(y[j]*x[i])
-    assert get_contraction_structure(j_i_ji) == {(j,): set([j_i_ji])}
+    assert get_contraction_structure(j_i_ji) == {(j,): {j_i_ji}}
     ij_exp_kki = x[i]*y[j]*exp(y[i]*y[k, k])
     result = get_contraction_structure(ij_exp_kki)
     expected = {
-        (i,): set([ij_exp_kki]),
+        (i,): {ij_exp_kki},
         ij_exp_kki: [{
-                     None: set([exp(y[i]*y[k, k])]),
+                     None: {exp(y[i]*y[k, k])},
                 exp(y[i]*y[k, k]): [{
-                    None: set([y[i]*y[k, k]]),
-                    y[i]*y[k, k]: [{(k,): set([y[k, k]])}]
+                    None: {y[i]*y[k, k]},
+                    y[i]*y[k, k]: [{(k,): {y[k, k]}}]
                 }]}
         ]
     }
@@ -161,10 +161,10 @@ def test_contraction_structure_Add_in_Pow():
     i, j, k = Idx('i'), Idx('j'), Idx('k')
     s_ii_jj_s = (1 + x[i, i])**(1 + y[j, j])
     expected = {
-        None: set([s_ii_jj_s]),
+        None: {s_ii_jj_s},
         s_ii_jj_s: [
-            {None: set([S.One]), (i,): set([x[i, i]])},
-            {None: set([S.One]), (j,): set([y[j, j]])}
+            {None: {S.One}, (i,): {x[i, i]}},
+            {None: {S.One}, (j,): {y[j, j]}}
         ]
     }
     result = get_contraction_structure(s_ii_jj_s)
@@ -172,9 +172,9 @@ def test_contraction_structure_Add_in_Pow():
 
     s_ii_jk_s = (1 + x[i, i]) ** (1 + y[j, k])
     expected_2 = {
-        None: set([(x[i, i] + 1)**(y[j, k] + 1)]),
+        None: {(x[i, i] + 1)**(y[j, k] + 1)},
         s_ii_jk_s: [
-            {None: set([S.One]), (i,): set([x[i, i]])}
+            {None: {S.One}, (i,): {x[i, i]}}
         ]
     }
     result_2 = get_contraction_structure(s_ii_jk_s)
@@ -188,14 +188,14 @@ def test_contraction_structure_Pow_in_Pow():
     i, j, k = Idx('i'), Idx('j'), Idx('k')
     ii_jj_kk = x[i, i]**y[j, j]**z[k, k]
     expected = {
-        None: set([ii_jj_kk]),
+        None: {ii_jj_kk},
         ii_jj_kk: [
-            {(i,): set([x[i, i]])},
+            {(i,): {x[i, i]}},
             {
-                None: set([y[j, j]**z[k, k]]),
+                None: {y[j, j]**z[k, k]},
                 y[j, j]**z[k, k]: [
-                    {(j,): set([y[j, j]])},
-                    {(k,): set([z[k, k]])}
+                    {(j,): {y[j, j]}},
+                    {(k,): {z[k, k]}}
                 ]
             }
         ]
@@ -211,17 +211,17 @@ def test_ufunc_support():
     i, j = Idx('i'), Idx('j')
     a = symbols('a')
 
-    assert get_indices(f(x[i])) == (set([i]), {})
-    assert get_indices(f(x[i], y[j])) == (set([i, j]), {})
+    assert get_indices(f(x[i])) == ({i}, {})
+    assert get_indices(f(x[i], y[j])) == ({i, j}, {})
     assert get_indices(f(y[i])*g(x[i])) == (set(), {})
-    assert get_indices(f(a, x[i])) == (set([i]), {})
-    assert get_indices(f(a, y[i], x[j])*g(x[i])) == (set([j]), {})
-    assert get_indices(g(f(x[i]))) == (set([i]), {})
+    assert get_indices(f(a, x[i])) == ({i}, {})
+    assert get_indices(f(a, y[i], x[j])*g(x[i])) == ({j}, {})
+    assert get_indices(g(f(x[i]))) == ({i}, {})
 
-    assert get_contraction_structure(f(x[i])) == {None: set([f(x[i])])}
+    assert get_contraction_structure(f(x[i])) == {None: {f(x[i])}}
     assert get_contraction_structure(
-        f(y[i])*g(x[i])) == {(i,): set([f(y[i])*g(x[i])])}
+        f(y[i])*g(x[i])) == {(i,): {f(y[i])*g(x[i])}}
     assert get_contraction_structure(
-        f(y[i])*g(f(x[i]))) == {(i,): set([f(y[i])*g(f(x[i]))])}
+        f(y[i])*g(f(x[i]))) == {(i,): {f(y[i])*g(f(x[i]))}}
     assert get_contraction_structure(
-        f(x[j], y[i])*g(x[i])) == {(i,): set([f(x[j], y[i])*g(x[i])])}
+        f(x[j], y[i])*g(x[i])) == {(i,): {f(x[j], y[i])*g(x[i])}}
