@@ -1,13 +1,11 @@
 """Useful utility decorators. """
 
-from __future__ import print_function, division
-
 import sys
 import types
 import inspect
 
 from sympy.core.decorators import wraps
-from sympy.core.compatibility import get_function_globals, get_function_name, iterable
+from sympy.core.compatibility import iterable
 from sympy.testing.runtests import DependencyError, SymPyDocTests, PyTestReporter
 
 def threaded_factory(func, use_add):
@@ -95,7 +93,7 @@ def conserve_mpmath_dps(func):
     return func_wrapper
 
 
-class no_attrs_in_subclass(object):
+class no_attrs_in_subclass:
     """Don't 'inherit' certain attributes from a base class
 
     >>> from sympy.utilities.decorator import no_attrs_in_subclass
@@ -194,7 +192,7 @@ def public(obj):
 
     >>> from sympy.utilities.decorator import public
 
-    >>> __all__
+    >>> __all__ # noqa: F821
     Traceback (most recent call last):
     ...
     NameError: name '__all__' is not defined
@@ -203,13 +201,13 @@ def public(obj):
     ... def some_function():
     ...     pass
 
-    >>> __all__
+    >>> __all__ # noqa: F821
     ['some_function']
 
     """
     if isinstance(obj, types.FunctionType):
-        ns = get_function_globals(obj)
-        name = get_function_name(obj)
+        ns = obj.__globals__
+        name = obj.__name__
     elif isinstance(obj, (type(type), type)):
         ns = sys.modules[obj.__module__].__dict__
         name = obj.__name__
